@@ -26,10 +26,9 @@ add_cmake_option() {
 
 add_cmake_option CMAKE_BUILD_TYPE RelWithDebInfo
 add_cmake_option CMAKE_INSTALL_PREFIX /
-add_cmake_option ENABLE_ACLK On
-add_cmake_option ENABLE_CLOUD On
+add_cmake_option ENABLE_DASHBOARD on
 add_cmake_option ENABLE_DBENGINE On
-add_cmake_option ENABLE_H2O On
+add_cmake_option ENABLE_H2O Off
 add_cmake_option ENABLE_ML On
 
 add_cmake_option ENABLE_PLUGIN_APPS On
@@ -37,6 +36,8 @@ add_cmake_option ENABLE_PLUGIN_CGROUP_NETWORK On
 add_cmake_option ENABLE_PLUGIN_DEBUGFS On
 add_cmake_option ENABLE_PLUGIN_FREEIPMI On
 add_cmake_option ENABLE_PLUGIN_GO On
+add_cmake_option ENABLE_PLUGIN_PYTHON On
+add_cmake_option ENABLE_PLUGIN_CHARTS On
 add_cmake_option ENABLE_PLUGIN_LOCAL_LISTENERS On
 add_cmake_option ENABLE_PLUGIN_NFACCT On
 add_cmake_option ENABLE_PLUGIN_PERF On
