@@ -4,12 +4,8 @@
 #include "aclk_util.h"
 #include "aclk.h"
 
-#include "daemon/common.h"
-
-#include "mqtt_websockets/c-rbuf/cringbuffer.h"
-
-static int aclk_https_request(https_req_t *request, https_req_response_t *response, bool *fallback_ipv4) {
-    int rc;
+static https_client_resp_t aclk_https_request(https_req_t *request, https_req_response_t *response, bool *fallback_ipv4) {
+    https_client_resp_t rc;
     // wrapper for ACLK only which loads ACLK specific proxy settings
     // then only calls https_request
     struct mqtt_wss_proxy proxy_conf = { .host = NULL, .port = 0, .username = NULL, .password = NULL, .type = MQTT_WSS_DIRECT };
@@ -37,7 +33,7 @@ struct auth_data {
 
 #define PARSE_ENV_JSON_CHK_TYPE(it, type, name)                                                                        \
     if (json_object_get_type(json_object_iter_peek_value(it)) != type) {                                               \
-        netdata_log_error("value of key \"%s\" should be %s", name, #type);                                            \
+        netdata_log_error("ACLK: value of key \"%s\" should be %s", name, #type);                                            \
         goto exit;                                                                                                     \
     }
 
@@ -54,7 +50,7 @@ static int parse_passwd_response(const char *json_str, struct auth_data *auth) {
 
     json = json_tokener_parse(json_str);
     if (!json) {
-        netdata_log_error("JSON-C failed to parse the payload of http response of /env endpoint");
+        netdata_log_error("ACLK: JSON-C failed to parse the payload of http response of /env endpoint");
         return 1;
     }
 
@@ -87,26 +83,26 @@ static int parse_passwd_response(const char *json_str, struct auth_data *auth) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_array, JSON_KEY_TOPICS)
 
             if (aclk_generate_topic_cache(json_object_iter_peek_value(&it))) {
-                netdata_log_error("Failed to generate topic cache!");
+                netdata_log_error("ACLK: Failed to generate topic cache!");
                 goto exit;
             }
             json_object_iter_next(&it);
             continue;
         }
-        netdata_log_error("Unknown key \"%s\" in passwd response payload. Ignoring", json_object_iter_peek_name(&it));
+        netdata_log_error("ACLK: Unknown key \"%s\" in passwd response payload. Ignoring", json_object_iter_peek_name(&it));
         json_object_iter_next(&it);
     }
 
     if (!auth->client_id) {
-        netdata_log_error(JSON_KEY_CLIENTID " is compulsory key in /password response");
+        netdata_log_error("ACLK: " JSON_KEY_CLIENTID " is compulsory key in /password response");
         goto exit;
     }
     if (!auth->passwd) {
-        netdata_log_error(JSON_KEY_PASS " is compulsory in /password response");
+        netdata_log_error("ACLK: " JSON_KEY_PASS " is compulsory in /password response");
         goto exit;
     }
     if (!auth->username) {
-        netdata_log_error(JSON_KEY_USER " is compulsory in /password response");
+        netdata_log_error("ACLK: " JSON_KEY_USER " is compulsory in /password response");
         goto exit;
     }
 
@@ -125,11 +121,11 @@ exit:
 static const char *get_json_str_by_path(json_object *json, const char *path) {
     json_object *ptr;
     if (json_pointer_get(json, path, &ptr)) {
-        netdata_log_error("Missing compulsory key \"%s\" in error response", path);
+        netdata_log_error("ACLK: Missing compulsory key \"%s\" in error response", path);
         return NULL;
     }
     if (json_object_get_type(ptr) != json_type_string) {
-        netdata_log_error("Value of Key \"%s\" in error response should be string", path);
+        netdata_log_error("ACLK: Value of Key \"%s\" in error response should be string", path);
         return NULL;
     }
     return json_object_get_string(ptr);
@@ -146,7 +142,7 @@ static int aclk_parse_otp_error(const char *json_str) {
 
     json = json_tokener_parse(json_str);
     if (!json) {
-        netdata_log_error("JSON-C failed to parse the payload of http response of /env endpoint");
+        netdata_log_error("ACLK: JSON-C failed to parse the payload of http response of /env endpoint");
         return 1;
     }
 
@@ -162,7 +158,7 @@ static int aclk_parse_otp_error(const char *json_str) {
     // optional field
     if (!json_pointer_get(json, "/" JSON_KEY_ERTRY, &ptr)) {
         if (json_object_get_type(ptr) != json_type_boolean) {
-            netdata_log_error("Error response Key " "/" JSON_KEY_ERTRY " should be of boolean type");
+            netdata_log_error("ACLK: Error response Key " "/" JSON_KEY_ERTRY " should be of boolean type");
             goto exit;
         }
         block_retry = json_object_get_boolean(ptr);
@@ -171,7 +167,7 @@ static int aclk_parse_otp_error(const char *json_str) {
     // optional field
     if (!json_pointer_get(json, "/" JSON_KEY_EDELAY, &ptr)) {
         if (json_object_get_type(ptr) != json_type_int) {
-            netdata_log_error("Error response Key " "/" JSON_KEY_EDELAY " should be of integer type");
+            netdata_log_error("ACLK: Error response Key " "/" JSON_KEY_EDELAY " should be of integer type");
             goto exit;
         }
         backoff = json_object_get_int(ptr);
@@ -183,7 +179,7 @@ static int aclk_parse_otp_error(const char *json_str) {
     if (backoff > 0)
         aclk_block_until = now_monotonic_sec() + backoff;
 
-    netdata_log_error("Cloud returned EC=\"%s\", Msg-Key:\"%s\", Msg:\"%s\", BlockRetry:%s, Backoff:%ds (-1 unset by cloud)", ec, ek, emsg, block_retry > 0 ? "true" : "false", backoff);
+    netdata_log_error("ACLK: Cloud returned EC=\"%s\", Msg-Key:\"%s\", Msg:\"%s\", BlockRetry:%s, Backoff:%ds (-1 unset by cloud)", ec, ek, emsg, block_retry > 0 ? "true" : "false", backoff);
     rc = 0;
 exit:
     json_object_put(json);
@@ -204,7 +200,7 @@ static int aclk_parse_otp_error(const char *json_str) {
 
     json = json_tokener_parse(json_str);
     if (!json) {
-        netdata_log_error("JSON-C failed to parse the payload of http response of /env endpoint");
+        netdata_log_error("ACLK: JSON-C failed to parse the payload of http response of /env endpoint");
         return 1;
     }
 
@@ -235,7 +231,7 @@ static int aclk_parse_otp_error(const char *json_str) {
         }
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_EDELAY)) {
             if (json_object_get_type(json_object_iter_peek_value(&it)) != json_type_int) {
-                netdata_log_error("value of key " JSON_KEY_EDELAY " should be integer");
+                netdata_log_error("ACLK: value of key " JSON_KEY_EDELAY " should be integer");
                 goto exit;
             }
 
@@ -245,7 +241,7 @@ static int aclk_parse_otp_error(const char *json_str) {
         }
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_ERTRY)) {
             if (json_object_get_type(json_object_iter_peek_value(&it)) != json_type_boolean) {
-                netdata_log_error("value of key " JSON_KEY_ERTRY " should be integer");
+                netdata_log_error("ACLK: value of key " JSON_KEY_ERTRY " should be integer");
                 goto exit;
             }
 
@@ -253,7 +249,7 @@ static int aclk_parse_otp_error(const char *json_str) {
             json_object_iter_next(&it);
             continue;
         }
-        netdata_log_error("Unknown key \"%s\" in error response payload. Ignoring", json_object_iter_peek_name(&it));
+        netdata_log_error("ACLK: Unknown key \"%s\" in error response payload. Ignoring", json_object_iter_peek_name(&it));
         json_object_iter_next(&it);
     }
 
@@ -263,7 +259,7 @@ static int aclk_parse_otp_error(const char *json_str) {
     if (backoff > 0)
         aclk_block_until = now_monotonic_sec() + backoff;
 
-    netdata_log_error("Cloud returned EC=\"%s\", Msg-Key:\"%s\", Msg:\"%s\", BlockRetry:%s, Backoff:%ds (-1 unset by cloud)", ec, ek, emsg, block_retry > 0 ? "true" : "false", backoff);
+    netdata_log_error("ACLK: Cloud returned EC=\"%s\", Msg-Key:\"%s\", Msg:\"%s\", BlockRetry:%s, Backoff:%ds (-1 unset by cloud)", ec, ek, emsg, block_retry > 0 ? "true" : "false", backoff);
     rc = 0;
 exit:
     json_object_put(json);
@@ -271,45 +267,13 @@ exit:
 }
 #endif
 
-#if defined(OPENSSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER < OPENSSL_VERSION_110
-static EVP_ENCODE_CTX *EVP_ENCODE_CTX_new(void)
-{
-	EVP_ENCODE_CTX *ctx = OPENSSL_malloc(sizeof(*ctx));
-
-	if (ctx != NULL) {
-		memset(ctx, 0, sizeof(*ctx));
-	}
-	return ctx;
-}
-static void EVP_ENCODE_CTX_free(EVP_ENCODE_CTX *ctx)
-{
-	OPENSSL_free(ctx);
-	return;
-}
-#endif
-
 #define CHALLENGE_LEN 256
 #define CHALLENGE_LEN_BASE64 344
-inline static int base64_decode_helper(unsigned char *out, int *outl, const unsigned char *in, int in_len)
-{
-    unsigned char remaining_data[CHALLENGE_LEN];
-    EVP_ENCODE_CTX *ctx = EVP_ENCODE_CTX_new();
-    EVP_DecodeInit(ctx);
-    EVP_DecodeUpdate(ctx, out, outl, in, in_len);
-    int remainder = 0;
-    EVP_DecodeFinal(ctx, remaining_data, &remainder);
-    EVP_ENCODE_CTX_free(ctx);
-    if (remainder) {
-        netdata_log_error("Unexpected data at EVP_DecodeFinal");
-        return 1;
-    }
-    return 0;
-}
 
 #define OTP_URL_PREFIX "/api/v1/auth/node/"
-int aclk_get_otp_challenge(url_t *target, const char *agent_id, unsigned char **challenge, int *challenge_bytes, bool *fallback_ipv4)
+static https_client_resp_t aclk_get_otp_challenge(url_t *target, const char *agent_id, unsigned char **challenge, int *challenge_bytes, bool *fallback_ipv4)
 {
-    int rc = 1;
+    https_client_resp_t rc;
     https_req_t req = HTTPS_REQ_T_INITIALIZER;
     https_req_response_t resp = HTTPS_REQ_RESPONSE_T_INITIALIZER;
 
@@ -320,13 +284,15 @@ int aclk_get_otp_challenge(url_t *target, const char *agent_id, unsigned char **
     buffer_sprintf(url, "%s/node/%s/challenge", target->path, agent_id);
     req.url = (char *)buffer_tostring(url);
 
-    if (aclk_https_request(&req, &resp, fallback_ipv4)) {
-        netdata_log_error("ACLK_OTP Challenge failed");
+    rc = aclk_https_request(&req, &resp, fallback_ipv4);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: OTP Challenge failed");
         buffer_free(url);
-        return 1;
+        return rc;
     }
     if (resp.http_code != 200) {
-        netdata_log_error("ACLK_OTP Challenge HTTP code not 200 OK (got %d)", resp.http_code);
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_NOT_200;
+        netdata_log_error("ACLK: OTP Challenge HTTP code not 200 OK (got %d)", resp.http_code);
         buffer_free(url);
         if (resp.payload_size)
             aclk_parse_otp_error(resp.payload);
@@ -334,41 +300,47 @@ int aclk_get_otp_challenge(url_t *target, const char *agent_id, unsigned char **
     }
     buffer_free(url);
 
-    netdata_log_info("ACLK_OTP Got Challenge from Cloud");
-
     json_object *json = json_tokener_parse(resp.payload);
     if (!json) {
-        netdata_log_error("Couldn't parse HTTP GET challenge payload");
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: couldn't parse HTTP GET challenge payload");
         goto cleanup_resp;
     }
     json_object *challenge_json;
     if (!json_object_object_get_ex(json, "challenge", &challenge_json)) {
-        netdata_log_error("No key named \"challenge\" in the returned JSON");
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: No key named \"challenge\" in the returned JSON");
         goto cleanup_json;
     }
     if (!json_object_is_type(challenge_json, json_type_string)) {
-        netdata_log_error("\"challenge\" is not a string JSON type");
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: \"challenge\" is not a string JSON type");
         goto cleanup_json;
     }
     const char *challenge_base64;
-    if (!(challenge_base64 = json_object_get_string(challenge_json))) {
-        netdata_log_error("Failed to extract challenge from JSON object");
+    if (!((challenge_base64 = json_object_get_string(challenge_json)))) {
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: Failed to extract challenge from JSON object");
         goto cleanup_json;
     }
     if (strlen(challenge_base64) != CHALLENGE_LEN_BASE64) {
-        netdata_log_error("Received Challenge has unexpected length of %zu (expected %d)", strlen(challenge_base64), CHALLENGE_LEN_BASE64);
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: Received Challenge has unexpected length of %zu (expected %d)", strlen(challenge_base64), CHALLENGE_LEN_BASE64);
         goto cleanup_json;
     }
 
-    *challenge = mallocz((CHALLENGE_LEN_BASE64 / 4) * 3);
-    base64_decode_helper(*challenge, challenge_bytes, (const unsigned char*)challenge_base64, strlen(challenge_base64));
+    *challenge = mallocz((CHALLENGE_LEN_BASE64 / 4) * 3 + 1);
+    *challenge_bytes = netdata_base64_decode(*challenge, (const unsigned char *) challenge_base64, CHALLENGE_LEN_BASE64);
+
     if (*challenge_bytes != CHALLENGE_LEN) {
-        netdata_log_error("Unexpected challenge length of %d instead of %d", *challenge_bytes, CHALLENGE_LEN);
+        rc = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID;
+        netdata_log_error("ACLK: Unexpected challenge length of %d instead of %d", *challenge_bytes, CHALLENGE_LEN);
         freez(*challenge);
         *challenge = NULL;
         goto cleanup_json;
     }
-    rc = 0;
+
+    rc = HTTPS_CLIENT_RESP_OK;
 
 cleanup_json:
     json_object_put(json);
@@ -377,10 +349,9 @@ cleanup_resp:
     return rc;
 }
 
-int aclk_send_otp_response(const char *agent_id, const unsigned char *response, int response_bytes, url_t *target, struct auth_data *mqtt_auth, bool *fallback_ipv4)
+static https_client_resp_t aclk_send_otp_response(const char *agent_id, const unsigned char *response, int response_bytes, url_t *target, struct auth_data *mqtt_auth, bool *fallback_ipv4)
 {
-    int len;
-    int rc = 1;
+    https_client_resp_t rc;
     https_req_t req = HTTPS_REQ_T_INITIALIZER;
     https_req_response_t resp = HTTPS_REQ_RESPONSE_T_INITIALIZER;
 
@@ -391,7 +362,7 @@ int aclk_send_otp_response(const char *agent_id, const unsigned char *response,
     unsigned char base64[CHALLENGE_LEN_BASE64 + 1];
     memset(base64, 0, CHALLENGE_LEN_BASE64 + 1);
 
-    base64_encode_helper(base64, &len, response, response_bytes);
+    (void) netdata_base64_encode(base64, response, response_bytes);
 
     BUFFER *url = buffer_create(strlen(OTP_URL_PREFIX) + UUID_STR_LEN + 20, &netdata_buffers_statistics.buffers_aclk);
     BUFFER *resp_json = buffer_create(strlen(OTP_URL_PREFIX) + UUID_STR_LEN + 20, &netdata_buffers_statistics.buffers_aclk);
@@ -403,28 +374,31 @@ int aclk_send_otp_response(const char *agent_id, const unsigned char *response,
     req.payload = (char *)buffer_tostring(resp_json);
     req.payload_size = strlen(req.payload);
 
-    if (aclk_https_request(&req, &resp, fallback_ipv4)) {
-        netdata_log_error("ACLK_OTP Password error trying to post result to password");
+    rc = aclk_https_request(&req, &resp, fallback_ipv4);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: OTP Password error trying to post result to password");
         goto cleanup_buffers;
     }
     if (resp.http_code != 201) {
-        netdata_log_error("ACLK_OTP Password HTTP code not 201 Created (got %d)", resp.http_code);
+        rc = HTTPS_CLIENT_RESP_OTP_PASSWORD_NOT_201;
+        netdata_log_error("ACLK: OTP Password HTTP code not 201 Created (got %d)", resp.http_code);
         if (resp.payload_size)
             aclk_parse_otp_error(resp.payload);
         goto cleanup_response;
     }
     if (resp.payload_size == 0 || resp.payload == NULL) {
-        netdata_log_error("ACLK_OTP Password response payload is empty despite returning 201 Created!");
+        rc = HTTPS_CLIENT_RESP_OTP_PASSWORD_EMPTY;
+        netdata_log_error("ACLK: OTP Password response payload is empty despite returning 201 Created!");
         goto cleanup_response;
     }
-    netdata_log_info("ACLK_OTP Got Password from Cloud");
 
     if (parse_passwd_response(resp.payload, mqtt_auth)){
-        netdata_log_error("Error parsing response of password endpoint");
+        rc = HTTPS_CLIENT_RESP_OTP_PASSWORD_NOT_JSON;
+        netdata_log_error("ACLK: Error parsing response of password endpoint");
         goto cleanup_response;
     }
 
-    rc = 0;
+    rc = HTTPS_CLIENT_RESP_OK;
 
 cleanup_response:
     https_req_response_free(&resp);
@@ -473,52 +447,52 @@ static int private_decrypt(RSA *p_key, unsigned char * enc_data, int data_len, u
     {
         char err[512];
         ERR_error_string_n(ERR_get_error(), err, sizeof(err));
-        netdata_log_error("Decryption of the challenge failed: %s", err);
+        netdata_log_error("ACLK: Decryption of the challenge failed: %s", err);
     }
     return result;
 }
 
 #if OPENSSL_VERSION_NUMBER >= OPENSSL_VERSION_300
-int aclk_get_mqtt_otp(EVP_PKEY *p_key, char **mqtt_id, char **mqtt_usr, char **mqtt_pass, url_t *target, bool *fallback_ipv4)
+https_client_resp_t aclk_get_mqtt_otp(EVP_PKEY *p_key, char **mqtt_id, char **mqtt_usr, char **mqtt_pass, url_t *target, bool *fallback_ipv4)
 #else
-int aclk_get_mqtt_otp(RSA *p_key, char **mqtt_id, char **mqtt_usr, char **mqtt_pass, url_t *target, bool *fallback_ipv4)
+https_client_resp_t aclk_get_mqtt_otp(RSA *p_key, char **mqtt_id, char **mqtt_usr, char **mqtt_pass, url_t *target, bool *fallback_ipv4)
 #endif
 {
     unsigned char *challenge = NULL;
     int challenge_bytes;
+    https_client_resp_t rc;
 
-    char *agent_id = get_agent_claimid();
-    if (agent_id == NULL) {
-        netdata_log_error("Agent was not claimed - cannot perform challenge/response");
-        return 1;
+    CLAIM_ID claim_id = claim_id_get();
+    if (!claim_id_is_set(claim_id)) {
+        netdata_log_error("ACLK: Agent was not claimed - cannot perform challenge/response");
+        return HTTPS_CLIENT_RESP_OTP_AGENT_NOT_CLAIMED;
     }
 
     // Get Challenge
-    if (aclk_get_otp_challenge(target, agent_id, &challenge, &challenge_bytes, fallback_ipv4)) {
-        netdata_log_error("Error getting challenge");
-        freez(agent_id);
-        return 1;
+    rc = aclk_get_otp_challenge(target, claim_id.str, &challenge, &challenge_bytes, fallback_ipv4);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: error getting challenge");
+        return rc;
     }
 
     // Decrypt Challenge / Get response
     unsigned char *response_plaintext = NULL;
     int response_plaintext_bytes = private_decrypt(p_key, challenge, challenge_bytes, &response_plaintext);
     if (response_plaintext_bytes < 0) {
-        netdata_log_error("Couldn't decrypt the challenge received");
+        netdata_log_error("ACLK: Couldn't decrypt the challenge received");
         freez(response_plaintext);
         freez(challenge);
-        freez(agent_id);
-        return 1;
+        return HTTPS_CLIENT_RESP_OTP_CHALLENGE_DECRYPTION_FAILED;
     }
     freez(challenge);
 
     // Encode and Send Challenge
     struct auth_data data = { .client_id = NULL, .passwd = NULL, .username = NULL };
-    if (aclk_send_otp_response(agent_id, response_plaintext, response_plaintext_bytes, target, &data, fallback_ipv4)) {
-        netdata_log_error("Error getting response");
+    rc = aclk_send_otp_response(claim_id.str, response_plaintext, response_plaintext_bytes, target, &data, fallback_ipv4);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: Error getting response");
         freez(response_plaintext);
-        freez(agent_id);
-        return 1;
+        return rc;
     }
 
     *mqtt_pass = data.passwd;
@@ -526,8 +500,7 @@ int aclk_get_mqtt_otp(RSA *p_key, char **mqtt_id, char **mqtt_usr, char **mqtt_p
     *mqtt_id = data.client_id;
 
     freez(response_plaintext);
-    freez(agent_id);
-    return 0;
+    return HTTPS_CLIENT_RESP_OK;
 }
 
 #define JSON_KEY_ENC "encoding"
@@ -552,12 +525,12 @@ static int parse_json_env_transport(json_object *json, aclk_transport_desc_t *tr
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_TRP_TYPE)) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_string, JSON_KEY_TRP_TYPE)
             if (trp->type != ACLK_TRP_UNKNOWN) {
-                netdata_log_error(JSON_KEY_TRP_TYPE " set already");
+                netdata_log_error("ACLK: " JSON_KEY_TRP_TYPE " set already");
                 goto exit;
             }
             trp->type = aclk_transport_type_t_from_str(json_object_get_string(json_object_iter_peek_value(&it)));
             if (trp->type == ACLK_TRP_UNKNOWN) {
-                netdata_log_error(JSON_KEY_TRP_TYPE " unknown type \"%s\"", json_object_get_string(json_object_iter_peek_value(&it)));
+                netdata_log_error("ACLK: " JSON_KEY_TRP_TYPE " unknown type \"%s\"", json_object_get_string(json_object_iter_peek_value(&it)));
                 goto exit;
             }
             json_object_iter_next(&it);
@@ -567,7 +540,7 @@ static int parse_json_env_transport(json_object *json, aclk_transport_desc_t *tr
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_TRP_ENDPOINT)) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_string, JSON_KEY_TRP_ENDPOINT)
             if (trp->endpoint) {
-                netdata_log_error(JSON_KEY_TRP_ENDPOINT " set already");
+                netdata_log_error("ACLK: " JSON_KEY_TRP_ENDPOINT " set already");
                 goto exit;
             }
             trp->endpoint = strdupz(json_object_get_string(json_object_iter_peek_value(&it)));
@@ -575,17 +548,17 @@ static int parse_json_env_transport(json_object *json, aclk_transport_desc_t *tr
             continue;
         }
 
-        netdata_log_error("unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
+        netdata_log_error("ACLK: unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
         json_object_iter_next(&it);
     }
 
     if (!trp->endpoint) {
-        netdata_log_error(JSON_KEY_TRP_ENDPOINT " is missing from JSON dictionary");
+        netdata_log_error("ACLK: " JSON_KEY_TRP_ENDPOINT " is missing from JSON dictionary");
         goto exit;
     }
 
     if (trp->type == ACLK_TRP_UNKNOWN) {
-        netdata_log_error("transport type not set");
+        netdata_log_error("ACLK: transport type not set");
         goto exit;
     }
 
@@ -601,7 +574,7 @@ static int parse_json_env_transports(json_object *json_array, aclk_env_t *env) {
     json_object *obj;
 
     if (env->transports) {
-        netdata_log_error("transports have been set already");
+        netdata_log_error("ACLK: transports have been set already");
         return 1;
     }
 
@@ -613,7 +586,7 @@ static int parse_json_env_transports(json_object *json_array, aclk_env_t *env) {
         trp = callocz(1, sizeof(aclk_transport_desc_t));
         obj = json_object_array_get_idx(json_array, i);
         if (parse_json_env_transport(obj, trp)) {
-            netdata_log_error("error parsing transport idx %d", (int)i);
+            netdata_log_error("ACLK: error parsing transport idx %d", (int)i);
             freez(trp);
             return 1;
         }
@@ -629,14 +602,14 @@ static int parse_json_env_transports(json_object *json_array, aclk_env_t *env) {
 static int parse_json_backoff_int(struct json_object_iterator *it, int *out, const char* name, int min, int max) {
     if (!strcmp(json_object_iter_peek_name(it), name)) {
         if (json_object_get_type(json_object_iter_peek_value(it)) != json_type_int) {
-            netdata_log_error("Could not parse \"%s\". Not an integer as expected.", name);
+            netdata_log_error("ACLK: Could not parse \"%s\". Not an integer as expected.", name);
             return MATCHED_ERROR;
         }
 
         *out = json_object_get_int(json_object_iter_peek_value(it));
 
         if (*out < min || *out > max) {
-            netdata_log_error("Value of \"%s\"=%d out of range (%d-%d).", name, *out, min, max);
+            netdata_log_error("ACLK: Value of \"%s\"=%d out of range (%d-%d).", name, *out, min, max);
             return MATCHED_ERROR;
         }
 
@@ -678,7 +651,7 @@ static int parse_json_backoff(json_object *json, aclk_backoff_t *backoff) {
             continue;
         }
 
-        netdata_log_error("unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
+        netdata_log_error("ACLK: unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
         json_object_iter_next(&it);
     }
 
@@ -690,7 +663,7 @@ static int parse_json_env_caps(json_object *json, aclk_env_t *env) {
     const char *str;
 
     if (env->capabilities) {
-        netdata_log_error("transports have been set already");
+        netdata_log_error("ACLK: transports have been set already");
         return 1;
     }
 
@@ -705,12 +678,12 @@ static int parse_json_env_caps(json_object *json, aclk_env_t *env) {
     for (size_t i = 0; i < env->capability_count; i++) {
         obj = json_object_array_get_idx(json, i);
         if (json_object_get_type(obj) != json_type_string) {
-            netdata_log_error("Capability at index %d not a string!", (int)i);
+            netdata_log_error("ACLK: Capability at index %d not a string!", (int)i);
             return 1;
         }
         str = json_object_get_string(obj);
         if (!str) {
-            netdata_log_error("Error parsing capabilities");
+            netdata_log_error("ACLK: Error parsing capabilities");
             return 1;
         }
         env->capabilities[i] = strdupz(str);
@@ -726,7 +699,7 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
 
     json = json_tokener_parse(json_str);
     if (!json) {
-        netdata_log_error("JSON-C failed to parse the payload of http response of /env endpoint");
+        netdata_log_error("ACLK: JSON-C failed to parse the payload of http response of /env endpoint");
         return 1;
     }
 
@@ -737,7 +710,7 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_AUTH_ENDPOINT)) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_string, JSON_KEY_AUTH_ENDPOINT)
             if (env->auth_endpoint) {
-                netdata_log_error("authEndpoint set already");
+                netdata_log_error("ACLK: authEndpoint set already");
                 goto exit;
             }
             env->auth_endpoint = strdupz(json_object_get_string(json_object_iter_peek_value(&it)));
@@ -748,7 +721,7 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
         if (!strcmp(json_object_iter_peek_name(&it), JSON_KEY_ENC)) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_string, JSON_KEY_ENC)
             if (env->encoding != ACLK_ENC_UNKNOWN) {
-                netdata_log_error(JSON_KEY_ENC " set already");
+                netdata_log_error("ACLK: " JSON_KEY_ENC " set already");
                 goto exit;
             }
             env->encoding = aclk_encoding_type_t_from_str(json_object_get_string(json_object_iter_peek_value(&it)));
@@ -771,7 +744,7 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
 
             if (parse_json_backoff(json_object_iter_peek_value(&it), &env->backoff)) {
                 env->backoff.base = 0;
-                netdata_log_error("Error parsing Backoff parameters in env");
+                netdata_log_error("ACLK: Error parsing Backoff parameters in env");
                 goto exit;
             }
 
@@ -783,7 +756,7 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
             PARSE_ENV_JSON_CHK_TYPE(&it, json_type_array, JSON_KEY_CAPS)
 
             if (parse_json_env_caps(json_object_iter_peek_value(&it), env)) {
-                netdata_log_error("Error parsing capabilities list");
+                netdata_log_error("ACLK: Error parsing capabilities list");
                 goto exit;
             }
 
@@ -791,25 +764,25 @@ static int parse_json_env(const char *json_str, aclk_env_t *env) {
             continue;
         }
 
-        netdata_log_error("unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
+        netdata_log_error("ACLK: unknown JSON key in dictionary (\"%s\")", json_object_iter_peek_name(&it));
         json_object_iter_next(&it);
     }
 
     // Check all compulsory keys have been set
     if (env->transport_count < 1) {
-        netdata_log_error("env has to return at least one transport");
+        netdata_log_error("ACLK: env has to return at least one transport");
         goto exit;
     }
     if (!env->auth_endpoint) {
-        netdata_log_error(JSON_KEY_AUTH_ENDPOINT " is compulsory");
+        netdata_log_error("ACLK: " JSON_KEY_AUTH_ENDPOINT " is compulsory");
         goto exit;
     }
     if (env->encoding == ACLK_ENC_UNKNOWN) {
-        netdata_log_error(JSON_KEY_ENC " is compulsory");
+        netdata_log_error("ACLK: " JSON_KEY_ENC " is compulsory");
         goto exit;
     }
     if (!env->backoff.base) {
-        netdata_log_error(JSON_KEY_BACKOFF " is compulsory");
+        netdata_log_error("ACLK: " JSON_KEY_BACKOFF " is compulsory");
         goto exit;
     }
 
@@ -822,61 +795,58 @@ exit:
     return 1;
 }
 
-int aclk_get_env(aclk_env_t *env, const char* aclk_hostname, int aclk_port, bool *fallback_ipv4) {
+https_client_resp_t aclk_get_env(aclk_env_t *env, const char* aclk_hostname, int aclk_port, bool *fallback_ipv4) {
     BUFFER *buf = buffer_create(1024, &netdata_buffers_statistics.buffers_aclk);
 
+    https_client_resp_t rc;
     https_req_t req = HTTPS_REQ_T_INITIALIZER;
     https_req_response_t resp = HTTPS_REQ_RESPONSE_T_INITIALIZER;
 
     req.request_type = HTTP_REQ_GET;
 
-    char *agent_id = get_agent_claimid();
-    if (agent_id == NULL)
-    {
-        netdata_log_error("Agent was not claimed - cannot perform challenge/response");
+    CLAIM_ID claim_id = claim_id_get();
+    if (!claim_id_is_set(claim_id)) {
+        netdata_log_error("ACLK: failed to get ACLK environment (agent is not claimed)");
         buffer_free(buf);
-        return 1;
+        return HTTPS_CLIENT_RESP_ENV_AGENT_NOT_CLAIMED;
     }
 
-    buffer_sprintf(buf, "/api/v1/env?v=%s&cap=proto,ctx&claim_id=%s", &(NETDATA_VERSION[1]) /* skip 'v' at beginning */, agent_id);
-
-    freez(agent_id);
+    buffer_sprintf(buf, "/api/v1/env?v=%s&cap=proto,ctx&claim_id=%s", &(NETDATA_VERSION[1]) /* skip 'v' at beginning */, claim_id.str);
 
     req.host = (char*)aclk_hostname;
     req.port = aclk_port;
     req.url = buf->buffer;
-    if (aclk_https_request(&req, &resp, fallback_ipv4)) {
-        netdata_log_error("Error trying to contact env endpoint");
+    rc = aclk_https_request(&req, &resp, fallback_ipv4);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: failed to get ACLK environment (cannot contact ENV endpoint)");
         https_req_response_free(&resp);
         buffer_free(buf);
-        return 2;
+        return rc;
     }
     if (resp.http_code != 200) {
-        netdata_log_error("The HTTP code not 200 OK (Got %d)", resp.http_code);
+        netdata_log_error("ACLK: failed to get ACLK environment (ENV response code is not 200) (got %d)", resp.http_code);
         if (resp.payload_size)
             aclk_parse_otp_error(resp.payload);
         https_req_response_free(&resp);
         buffer_free(buf);
-        return 3;
+        return HTTPS_CLIENT_RESP_ENV_NOT_200;
     }
 
     if (!resp.payload || !resp.payload_size) {
-        netdata_log_error("Unexpected empty payload as response to /env call");
+        netdata_log_error("ACLK: failed to get ACLK environment (ENV response is empty)");
         https_req_response_free(&resp);
         buffer_free(buf);
-        return 4;
+        return HTTPS_CLIENT_RESP_ENV_EMPTY;
     }
 
     if (parse_json_env(resp.payload, env)) {
-        netdata_log_error("error parsing /env message");
+        netdata_log_error("ACLK: failed to get ACLK environment (ENV response is not JSON)");
         https_req_response_free(&resp);
         buffer_free(buf);
-        return 5;
+        return HTTPS_CLIENT_RESP_ENV_NOT_JSON;
     }
 
-    netdata_log_info("Getting Cloud /env successful");
-
     https_req_response_free(&resp);
     buffer_free(buf);
-    return 0;
+    return HTTPS_CLIENT_RESP_OK;
 }
