@@ -6,7 +6,131 @@
 
 #include "aclk_util.h"
 
-#include "daemon/global_statistics.h"
+#include "daemon/pulse/pulse.h"
+
+ENUM_STR_MAP_DEFINE(https_client_resp_t) = {
+    {
+        .id = HTTPS_CLIENT_RESP_OK,
+        .name = "ok",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_UNKNOWN_ERROR,
+        .name = "unknown error",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NO_MEM,
+        .name = "not enough memory",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NONBLOCK_FAILED,
+        .name = "cannot set socket to non-blocking mode",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_PROXY_NOT_200,
+        .name = "proxy did not return http/200",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NO_SSL_CTX,
+        .name = "cannot create SSL ctx",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NO_SSL_VERIFY_PATHS,
+        .name = "cannot set SSL verify paths",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NO_SSL_NEW,
+        .name = "cannot create SSL",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_NO_TLS_SNI,
+        .name = "cannot set TLS SNI",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_SSL_CONNECT_FAILED,
+        .name = "SSL_connect() failed",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_SSL_START_FAILED,
+        .name = "cannot start SSL connection",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_UNKNOWN_REQUEST_TYPE,
+        .name = "unknown https client request type",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_HEADER_WRITE_FAILED,
+        .name = "https client failed to write http header",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_PAYLOAD_WRITE_FAILED,
+        .name = "https client failed to write http payload",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_POLL_ERROR,
+        .name = "https client poll() error",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_TIMEOUT,
+        .name = "https client timeout",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_READ_ERROR,
+        .name = "https client read error",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_PARSE_ERROR,
+        .name = "https client parsing of response failed",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_ENV_AGENT_NOT_CLAIMED,
+        .name = "agent is not claimed (during /env)",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_ENV_NOT_200,
+        .name = "/env response code is not 200",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_ENV_EMPTY,
+        .name = "/env response is empty",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_ENV_NOT_JSON,
+        .name = "/env response is not JSON",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_CHALLENGE_NOT_200,
+        .name = "otp challenge response is not http/200",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_CHALLENGE_INVALID,
+        .name = "otp challenge response is invalid",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_PASSWORD_NOT_201,
+        .name = "otp password response is not http/201",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_PASSWORD_EMPTY,
+        .name = "otp password response is empty",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_PASSWORD_NOT_JSON,
+        .name = "otp password response is not JSON",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_AGENT_NOT_CLAIMED,
+        .name = "agent is not claimed (during otp)",
+    },
+    {
+        .id = HTTPS_CLIENT_RESP_OTP_CHALLENGE_DECRYPTION_FAILED,
+        .name = "otp challenge decryption failed",
+    },
+
+    // terminator
+    {.name = NULL, .id = 0}
+};
+
+ENUM_STR_DEFINE_FUNCTIONS(https_client_resp_t, HTTPS_CLIENT_RESP_UNKNOWN_ERROR, "unknown error");
 
 static const char *http_req_type_to_str(http_req_type_t req) {
     switch (req) {
@@ -34,6 +158,9 @@ void http_parse_ctx_create(http_parse_ctx *ctx, enum http_parse_state parse_stat
 
 void http_parse_ctx_destroy(http_parse_ctx *ctx)
 {
+    if(!ctx->headers)
+        return;
+
     c_rhash_iter_t iter;
     const char *key;
 
@@ -45,6 +172,7 @@ void http_parse_ctx_destroy(http_parse_ctx *ctx)
     }
 
     c_rhash_destroy(ctx->headers);
+    ctx->headers = NULL;
 }
 
 #define POLL_TO_MS 100
@@ -62,16 +190,16 @@ static int process_http_hdr(http_parse_ctx *parse_ctx, const char *key, const ch
     // we can skip the rest
     if (parse_ctx->content_length < 0 && !strcmp("content-length", key)) {
         if (parse_ctx->content_length == TRANSFER_ENCODING_CHUNKED) {
-            netdata_log_error("Content-length and transfer-encoding: chunked headers are mutually exclusive");
+            netdata_log_error("ACLK: Content-length and transfer-encoding: chunked headers are mutually exclusive");
             return 1;
         }
         if (parse_ctx->content_length != -1) {
-            netdata_log_error("Duplicate content-length header");
+            netdata_log_error("ACLK: Duplicate content-length header");
             return 1;
         }
         parse_ctx->content_length = str2u(val);
         if (parse_ctx->content_length < 0) {
-            netdata_log_error("Invalid content-length %d", parse_ctx->content_length);
+            netdata_log_error("ACLK: Invalid content-length %d", parse_ctx->content_length);
             return 1;
         }
         return 0;
@@ -79,7 +207,7 @@ static int process_http_hdr(http_parse_ctx *parse_ctx, const char *key, const ch
     if (!strcmp("transfer-encoding", key)) {
         if (!strcmp("chunked", val)) {
             if (parse_ctx->content_length != -1) {
-                netdata_log_error("Content-length and transfer-encoding: chunked headers are mutually exclusive");
+                netdata_log_error("ACLK: Content-length and transfer-encoding: chunked headers are mutually exclusive");
                 return 1;
             }
             parse_ctx->content_length = TRANSFER_ENCODING_CHUNKED;
@@ -105,19 +233,20 @@ static int parse_http_hdr(rbuf_t buf, http_parse_ctx *parse_ctx)
     int idx, idx_end;
     char buf_key[HTTP_HDR_BUFFER_SIZE];
     char buf_val[HTTP_HDR_BUFFER_SIZE];
-    char *ptr = buf_key;
+    char *ptr;
+
     if (!rbuf_find_bytes(buf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM), &idx_end)) {
-        netdata_log_error("CRLF expected");
+        netdata_log_error("ACLK: CRLF expected");
         return 1;
     }
 
     char *separator = rbuf_find_bytes(buf, HTTP_KEYVAL_SEPARATOR, strlen(HTTP_KEYVAL_SEPARATOR), &idx);
     if (!separator) {
-        netdata_log_error("Missing Key/Value separator");
+        netdata_log_error("ACLK: Missing Key/Value separator");
         return 1;
     }
     if (idx >= HTTP_HDR_BUFFER_SIZE) {
-        netdata_log_error("Key name is too long");
+        netdata_log_error("ACLK: Key name is too long");
         return 1;
     }
 
@@ -127,7 +256,7 @@ static int parse_http_hdr(rbuf_t buf, http_parse_ctx *parse_ctx)
     rbuf_bump_tail(buf, strlen(HTTP_KEYVAL_SEPARATOR));
     idx_end -= strlen(HTTP_KEYVAL_SEPARATOR) + idx;
     if (idx_end >= HTTP_HDR_BUFFER_SIZE) {
-        netdata_log_error("Value of key \"%s\" too long", buf_key);
+        netdata_log_error("ACLK: Value of key \"%s\" too long", buf_key);
         return 1;
     }
 
@@ -172,7 +301,7 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                     continue;
                 }
                 if (idx >= HTTP_HDR_BUFFER_SIZE) {
-                    netdata_log_error("Chunk size is too long");
+                    netdata_log_error("ACLK: Chunk size is too long");
                     return HTTP_PARSE_ERROR;
                 }
                 char buf_size[HTTP_HDR_BUFFER_SIZE];
@@ -180,13 +309,13 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                 buf_size[idx] = 0;
                 long chunk_size = strtol(buf_size, NULL, 16);
                 if (chunk_size < 0 || chunk_size == LONG_MAX) {
-                    netdata_log_error("Chunk size out of range");
+                    netdata_log_error("ACLK: Chunk size out of range");
                     return HTTP_PARSE_ERROR;
                 }
                 parse_ctx->chunk_size = chunk_size;
                 if (parse_ctx->chunk_size == 0) {
                     if (errno == EINVAL) {
-                        netdata_log_error("Invalid chunk size");
+                        netdata_log_error("ACLK: Invalid chunk size");
                         return HTTP_PARSE_ERROR;
                     }
                     parse_ctx->chunked_content_state = CHUNKED_CONTENT_CHUNK_END_CRLF;
@@ -216,12 +345,12 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                 char buf_crlf[strlen(HTTP_LINE_TERM)];
                 rbuf_pop(buf, buf_crlf, strlen(HTTP_LINE_TERM));
                 if (memcmp(buf_crlf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM))) {
-                    netdata_log_error("CRLF expected");
+                    netdata_log_error("ACLK: CRLF expected");
                     return HTTP_PARSE_ERROR;
                 }
                 if (parse_ctx->chunked_content_state == CHUNKED_CONTENT_FINAL_CRLF) {
                     if (parse_ctx->chunked_response_size != parse_ctx->chunked_response_written)
-                        netdata_log_error("Chunked response size mismatch");
+                        netdata_log_error("ACLK: Chunked response size mismatch");
                     chunked_response_buffer_grow_by(parse_ctx, 1);
                     parse_ctx->chunked_response[parse_ctx->chunked_response_written] = 0;
                     return HTTP_PARSE_SUCCESS;
@@ -251,28 +380,28 @@ http_parse_rc parse_http_response(rbuf_t buf, http_parse_ctx *parse_ctx)
                     if (parse_ctx->state == HTTP_PARSE_PROXY_CONNECT) {
                         if (rbuf_memcmp_n(buf, RESP_PROTO10, strlen(RESP_PROTO10))) {
                             netdata_log_error(
-                                "Expected response to start with \"%s\" or \"%s\"", RESP_PROTO, RESP_PROTO10);
+                                "ACLK: Expected response to start with \"%s\" or \"%s\"", RESP_PROTO, RESP_PROTO10);
                             return HTTP_PARSE_ERROR;
                         }
                     }
                     else {
-                        netdata_log_error("Expected response to start with \"%s\"", RESP_PROTO);
+                        netdata_log_error("ACLK: Expected response to start with \"%s\"", RESP_PROTO);
                         return HTTP_PARSE_ERROR;
                     }
                 }
                 rbuf_bump_tail(buf, strlen(RESP_PROTO));
                 if (rbuf_pop(buf, rc, 4) != 4) {
-                    netdata_log_error("Expected HTTP status code");
+                    netdata_log_error("ACLK: Expected HTTP status code");
                     return HTTP_PARSE_ERROR;
                 }
                 if (rc[3] != ' ') {
-                    netdata_log_error("Expected space after HTTP return code");
+                    netdata_log_error("ACLK: Expected space after HTTP return code");
                     return HTTP_PARSE_ERROR;
                 }
                 rc[3] = 0;
                 parse_ctx->http_code = atoi(rc);
                 if (parse_ctx->http_code < 100 || parse_ctx->http_code >= 600) {
-                    netdata_log_error("HTTP code not in range 100 to 599");
+                    netdata_log_error("ACLK: HTTP code not in range 100 to 599");
                     return HTTP_PARSE_ERROR;
                 }
 
@@ -331,7 +460,7 @@ typedef struct https_req_ctx {
 
 static int https_req_check_timedout(https_req_ctx_t *ctx) {
     if (now_realtime_sec() > ctx->req_start_time + ctx->request->timeout_s) {
-        netdata_log_error("request timed out");
+        netdata_log_error("ACLK: request timed out");
         return 1;
     }
     return 0;
@@ -365,12 +494,12 @@ static int socket_write_all(https_req_ctx_t *ctx, char *data, size_t data_len) {
     do {
         int ret = poll(&ctx->poll_fd, 1, POLL_TO_MS);
         if (ret < 0) {
-            netdata_log_error("poll error");
+            netdata_log_error("ACLK: poll error");
             return 1;
         }
         if (ret == 0) {
             if (https_req_check_timedout(ctx)) {
-                netdata_log_error("Poll timed out");
+                netdata_log_error("ACLK: Poll timed out");
                 return 2;
             }
             continue;
@@ -380,7 +509,7 @@ static int socket_write_all(https_req_ctx_t *ctx, char *data, size_t data_len) {
         if (ret > 0) {
             ctx->written += ret;
         } else if (errno != EAGAIN && errno != EWOULDBLOCK) {
-            netdata_log_error("Error writing to socket");
+            netdata_log_error("ACLK: Error writing to socket");
             return 3;
         }
     } while (ctx->written < data_len);
@@ -395,12 +524,12 @@ static int ssl_write_all(https_req_ctx_t *ctx, char *data, size_t data_len) {
     do {
         int ret = poll(&ctx->poll_fd, 1, POLL_TO_MS);
         if (ret < 0) {
-            netdata_log_error("poll error");
+            netdata_log_error("ACLK: poll error");
             return 1;
         }
         if (ret == 0) {
             if (https_req_check_timedout(ctx)) {
-                netdata_log_error("Poll timed out");
+                netdata_log_error("ACLK: Poll timed out");
                 return 2;
             }
             continue;
@@ -420,7 +549,7 @@ static int ssl_write_all(https_req_ctx_t *ctx, char *data, size_t data_len) {
                     ctx->poll_fd.events |= POLLOUT;
                     break;
                 default:
-                    netdata_log_error("SSL_write Err: %s", _ssl_err_tos(ret));
+                    netdata_log_error("ACLK: SSL_write Err: %s", _ssl_err_tos(ret));
                     return 3;
             }
         }
@@ -435,7 +564,7 @@ static inline int https_client_write_all(https_req_ctx_t *ctx, char *data, size_
     return socket_write_all(ctx, data, data_len);
 }
 
-static int read_parse_response(https_req_ctx_t *ctx) {
+static https_client_resp_t read_parse_response(https_req_ctx_t *ctx) {
     int ret;
     char *ptr;
     size_t size;
@@ -444,13 +573,13 @@ static int read_parse_response(https_req_ctx_t *ctx) {
     do {
         ret = poll(&ctx->poll_fd, 1, POLL_TO_MS);
         if (ret < 0) {
-            netdata_log_error("poll error");
-            return 1;
+            netdata_log_error("ACLK: poll error");
+            return HTTPS_CLIENT_RESP_POLL_ERROR;
         }
         if (ret == 0) {
             if (https_req_check_timedout(ctx)) {
-                netdata_log_error("Poll timed out");
-                return 2;
+                netdata_log_error("ACLK: poll() timed out");
+                return HTTPS_CLIENT_RESP_TIMEOUT;
             }
             if (!ctx->ssl_ctx)
                 continue;
@@ -478,13 +607,14 @@ static int read_parse_response(https_req_ctx_t *ctx) {
                             ctx->poll_fd.events |= POLLOUT;
                             break;
                         default:
-                            netdata_log_error("SSL_read Err: %s", _ssl_err_tos(ret));
-                            return 3;
+                            netdata_log_error("ACLK: SSL_read() Err: %s", _ssl_err_tos(ret));
+                            return HTTPS_CLIENT_RESP_READ_ERROR;
                     }
-                } else {
+                }
+                else {
                     if (errno != EAGAIN && errno != EWOULDBLOCK) {
-                        netdata_log_error("write error");
-                        return 3;
+                        netdata_log_error("ACLK: read error");
+                        return HTTPS_CLIENT_RESP_READ_ERROR;
                     }
                     ctx->poll_fd.events |= POLLIN;
                 }
@@ -493,11 +623,11 @@ static int read_parse_response(https_req_ctx_t *ctx) {
     } while (!(ret = parse_http_response(ctx->buf_rx, &ctx->parse_ctx)));
 
     if (ret != HTTP_PARSE_SUCCESS) {
-        netdata_log_error("Error parsing HTTP response");
-        return 1;
+        netdata_log_error("ACLK: error parsing HTTP response");
+        return HTTPS_CLIENT_RESP_PARSE_ERROR;
     }
 
-    return 0;
+    return HTTPS_CLIENT_RESP_OK;
 }
 
 static const char *http_methods[] = {
@@ -509,15 +639,15 @@ static const char *http_methods[] = {
 
 #define TX_BUFFER_SIZE 8192
 #define RX_BUFFER_SIZE (TX_BUFFER_SIZE*2)
-static int handle_http_request(https_req_ctx_t *ctx) {
+static https_client_resp_t handle_http_request(https_req_ctx_t *ctx) {
     BUFFER *hdr = buffer_create(TX_BUFFER_SIZE, &netdata_buffers_statistics.buffers_aclk);
-    int rc = 0;
+    https_client_resp_t rc = HTTPS_CLIENT_RESP_OK;
 
     http_req_type_t req_type = ctx->request->request_type;
 
     if (req_type >= HTTP_REQ_INVALID) {
-        netdata_log_error("Unknown HTTPS request type!");
-        rc = 1;
+        netdata_log_error("ACLK: unknown HTTPS request type!");
+        rc = HTTPS_CLIENT_RESP_UNKNOWN_REQUEST_TYPE;
         goto err_exit;
     }
     buffer_strcat(hdr, http_methods[req_type]);
@@ -555,7 +685,7 @@ static int handle_http_request(https_req_ctx_t *ctx) {
         // we remove those but during encoding we need that space in the buffer
         creds_base64_len += (1+(creds_base64_len/64)) * strlen("\n");
         char *creds_base64 = callocz(1, creds_base64_len + 1);
-        base64_encode_helper((unsigned char*)creds_base64, &creds_base64_len, (unsigned char*)creds_plain, creds_plain_len);
+        (void) netdata_base64_encode((unsigned char *)creds_base64, (unsigned char *)creds_plain, creds_plain_len);
         buffer_sprintf(hdr, "Proxy-Authorization: Basic %s\x0D\x0A", creds_base64);
         freez(creds_plain);
     }
@@ -564,26 +694,25 @@ static int handle_http_request(https_req_ctx_t *ctx) {
 
     // Send the request
     if (https_client_write_all(ctx, hdr->buffer, hdr->len)) {
-        netdata_log_error("Couldn't write HTTP request header into SSL connection");
-        rc = 2;
+        netdata_log_error("ACLK: couldn't write HTTP request header into SSL connection");
+        rc = HTTPS_CLIENT_RESP_HEADER_WRITE_FAILED;
         goto err_exit;
     }
 
     if (req_type == HTTP_REQ_POST && ctx->request->payload && ctx->request->payload_size) {
         if (https_client_write_all(ctx, ctx->request->payload, ctx->request->payload_size)) {
-            netdata_log_error("Couldn't write payload into SSL connection");
-            rc = 3;
+            netdata_log_error("ACLK: couldn't write payload into SSL connection");
+            rc = HTTPS_CLIENT_RESP_PAYLOAD_WRITE_FAILED;
             goto err_exit;
         }
     }
 
     // Read The Response
-    if (read_parse_response(ctx)) {
-        netdata_log_error("Error reading or parsing response from server");
+    rc = read_parse_response(ctx);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: error reading or parsing response from server");
         if (ctx->parse_ctx.chunked_response)
             freez(ctx->parse_ctx.chunked_response);
-        rc = 4;
-        goto err_exit;
     }
 
 err_exit:
@@ -613,40 +742,45 @@ static int cert_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
     if (!preverify_ok && err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
     {
         preverify_ok = 1;
-        netdata_log_error("Self Signed Certificate Accepted as the agent was built with ACLK_SSL_ALLOW_SELF_SIGNED");
+        netdata_log_error("ACLK: Self Signed Certificate Accepted as the agent was built with ACLK_SSL_ALLOW_SELF_SIGNED");
     }
 #endif
 
     return preverify_ok;
 }
 
-int https_request(https_req_t *request, https_req_response_t *response) {
-    int rc = 1, ret;
+https_client_resp_t https_request(https_req_t *request, https_req_response_t *response, bool *fallback_ipv4)
+{
+    https_client_resp_t rc;
+    int ret;
     char connect_port_str[PORT_STR_MAX_BYTES];
 
     const char *connect_host = request->proxy_host ? request->proxy_host : request->host;
     int connect_port = request->proxy_host ? request->proxy_port : request->port;
-    struct timeval timeout = { .tv_sec = request->timeout_s, .tv_usec = 0 };
+    struct timeval timeout = { .tv_sec = 10, .tv_usec = 0 };
 
     https_req_ctx_t *ctx = callocz(1, sizeof(https_req_ctx_t));
     ctx->req_start_time = now_realtime_sec();
 
     ctx->buf_rx = rbuf_create(RX_BUFFER_SIZE);
     if (!ctx->buf_rx) {
-        netdata_log_error("Couldn't allocate buffer for RX data");
+        rc = HTTPS_CLIENT_RESP_NO_MEM;
+        netdata_log_error("ACLK: couldn't allocate buffer for RX data");
         goto exit_req_ctx;
     }
 
     snprintfz(connect_port_str, PORT_STR_MAX_BYTES, "%d", connect_port);
 
-    ctx->sock = connect_to_this_ip46(IPPROTO_TCP, SOCK_STREAM, connect_host, 0, connect_port_str, &timeout);
+    ctx->sock = connect_to_this_ip46(IPPROTO_TCP, SOCK_STREAM, connect_host, 0, connect_port_str, &timeout, fallback_ipv4);
     if (ctx->sock < 0) {
-        netdata_log_error("Error connecting TCP socket to \"%s\"", connect_host);
+        rc = -ctx->sock;
+        netdata_log_error("ACLK: error connecting TCP socket to \"%s\"", connect_host);
         goto exit_buf_rx;
     }
 
     if (fcntl(ctx->sock, F_SETFL, fcntl(ctx->sock, F_GETFL, 0) | O_NONBLOCK) == -1) {
-        netdata_log_error("Error setting O_NONBLOCK to TCP socket.");
+        rc = HTTPS_CLIENT_RESP_NONBLOCK_FAILED;
+        netdata_log_error("ACLK: error setting O_NONBLOCK to TCP socket.");
         goto exit_sock;
     }
 
@@ -663,48 +797,54 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         req.proxy_username = request->proxy_username;
         req.proxy_password = request->proxy_password;
         ctx->request = &req;
-        if (handle_http_request(ctx)) {
-            netdata_log_error("Failed to CONNECT with proxy");
+        rc = handle_http_request(ctx);
+        if (rc != HTTPS_CLIENT_RESP_OK) {
+            netdata_log_error("ACLK: failed to CONNECT with proxy");
             http_parse_ctx_destroy(&ctx->parse_ctx);
             goto exit_sock;
         }
         if (ctx->parse_ctx.http_code != 200) {
-            netdata_log_error("Proxy didn't return 200 OK (got %d)", ctx->parse_ctx.http_code);
+            rc = HTTPS_CLIENT_RESP_PROXY_NOT_200;
+            netdata_log_error("ACLK: proxy didn't return 200 OK (got %d)", ctx->parse_ctx.http_code);
             http_parse_ctx_destroy(&ctx->parse_ctx);
             goto exit_sock;
         }
         http_parse_ctx_destroy(&ctx->parse_ctx);
-        netdata_log_info("Proxy accepted CONNECT upgrade");
     }
     ctx->request = request;
 
     ctx->ssl_ctx = netdata_ssl_create_client_ctx(0);
     if (ctx->ssl_ctx==NULL) {
-        netdata_log_error("Cannot allocate SSL context");
+        rc = HTTPS_CLIENT_RESP_NO_SSL_CTX;
+        netdata_log_error("ACLK: cannot allocate SSL context");
         goto exit_sock;
     }
 
     if (!SSL_CTX_set_default_verify_paths(ctx->ssl_ctx)) {
-        netdata_log_error("Error setting default verify paths");
+        rc = HTTPS_CLIENT_RESP_NO_SSL_VERIFY_PATHS;
+        netdata_log_error("ACLK: error setting default verify paths");
         goto exit_CTX;
     }
     SSL_CTX_set_verify(ctx->ssl_ctx, SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE, cert_verify_callback);
 
     ctx->ssl = SSL_new(ctx->ssl_ctx);
     if (ctx->ssl==NULL) {
-        netdata_log_error("Cannot allocate SSL");
+        rc = HTTPS_CLIENT_RESP_NO_SSL_NEW;
+        netdata_log_error("ACLK: cannot allocate SSL");
         goto exit_CTX;
     }
 
     if (!SSL_set_tlsext_host_name(ctx->ssl, request->host)) {
-        netdata_log_error("Error setting TLS SNI host");
+        rc = HTTPS_CLIENT_RESP_NO_TLS_SNI;
+        netdata_log_error("ACLK: error setting TLS SNI host");
         goto exit_CTX;
     }
 
     SSL_set_fd(ctx->ssl, ctx->sock);
     ret = SSL_connect(ctx->ssl);
     if (ret != -1 && ret != 1) {
-        netdata_log_error("SSL could not connect");
+        rc = HTTPS_CLIENT_RESP_SSL_CONNECT_FAILED;
+        netdata_log_error("ACLK: SSL failed to connect");
         goto exit_SSL;
     }
     if (ret == -1) {
@@ -712,14 +852,16 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         // consult SSL_connect documentation for details
         int ec = SSL_get_error(ctx->ssl, ret);
         if (ec != SSL_ERROR_WANT_READ && ec != SSL_ERROR_WANT_WRITE) {
-            netdata_log_error("Failed to start SSL connection");
+            rc = HTTPS_CLIENT_RESP_SSL_START_FAILED;
+            netdata_log_error("ACLK: failed to start SSL connection");
             goto exit_SSL;
         }
     }
 
     // The actual request here
-    if (handle_http_request(ctx)) {
-        netdata_log_error("Couldn't process request");
+    rc = handle_http_request(ctx);
+    if (rc != HTTPS_CLIENT_RESP_OK) {
+        netdata_log_error("ACLK: couldn't process request");
         http_parse_ctx_destroy(&ctx->parse_ctx);
         goto exit_SSL;
     }
@@ -734,7 +876,7 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         response->payload = mallocz(response->payload_size + 1);
         ret = rbuf_pop(ctx->buf_rx, response->payload, response->payload_size);
         if (ret != (int)response->payload_size) {
-            netdata_log_error("Payload size doesn't match remaining data on the buffer!");
+            netdata_log_error("ACLK: payload size doesn't match remaining data on the buffer!");
             response->payload_size = ret;
         }
         // normally we take payload as it is and copy it
@@ -745,9 +887,9 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         // only exact data without affixed 0x00
         ((char*)response->payload)[response->payload_size] = 0; // mallocz(response->payload_size + 1);
     }
-    netdata_log_info("HTTPS \"%s\" request to \"%s\" finished with HTTP code: %d", http_req_type_to_str(ctx->request->request_type), ctx->request->host, response->http_code);
+    netdata_log_info("ACLK: HTTPS \"%s\" request to \"%s\" finished with HTTP code: %d", http_req_type_to_str(ctx->request->request_type), ctx->request->host, response->http_code);
 
-    rc = 0;
+    rc = HTTPS_CLIENT_RESP_OK;
 
 exit_SSL:
     SSL_free(ctx->ssl);
@@ -775,7 +917,7 @@ static inline char *UNUSED_FUNCTION(min_non_null)(char *a, char *b) {
 }
 
 #define URI_PROTO_SEPARATOR "://"
-#define URL_PARSER_LOG_PREFIX "url_parser "
+#define URL_PARSER_LOG_PREFIX "ACLK: url_parser "
 
 static int parse_host_port(url_t *url) {
     char *ptr = strrchr(url->host, ':');
