@@ -39,7 +39,7 @@ SIMPLE_PATTERN *search_cgroup_paths = NULL;
 SIMPLE_PATTERN *enabled_cgroup_renames = NULL;
 SIMPLE_PATTERN *systemd_services_cgroups = NULL;
 SIMPLE_PATTERN *entrypoint_parent_process_comm = NULL;
-char *cgroups_network_interface_script = NULL;
+const char *cgroups_network_interface_script = NULL;
 int cgroups_check = 0;
 uint32_t Read_hash = 0;
 uint32_t Write_hash = 0;
@@ -73,30 +73,19 @@ struct discovery_thread discovery_thread;
 #define MAXSIZE_PROC_CMDLINE 4096
 static enum cgroups_systemd_setting cgroups_detect_systemd(const char *exec)
 {
-    pid_t command_pid;
     enum cgroups_systemd_setting retval = SYSTEMD_CGROUP_ERR;
     char buf[MAXSIZE_PROC_CMDLINE];
     char *begin, *end;
 
-    FILE *fp_child_input;
-    FILE *fp_child_output = netdata_popen(exec, &command_pid, &fp_child_input);
-
-    if (!fp_child_output)
+    POPEN_INSTANCE *pi = spawn_popen_run(exec);
+    if(!pi)
         return retval;
 
-    int fd = fileno(fp_child_output);
-    if (fd == -1 ) {
-        collector_error("Cannot get the output of \"%s\": failed to get file descriptor", exec);
-        netdata_pclose(fp_child_input, fp_child_output, command_pid);
-        return retval;
-    }
-
     struct pollfd pfd;
-    pfd.fd = fd;
+    pfd.fd = spawn_popen_read_fd(pi);
     pfd.events = POLLIN;
 
     int timeout = 3000; // milliseconds
-
     int ret = poll(&pfd, 1, timeout);
 
     if (ret == -1) {
@@ -104,7 +93,7 @@ static enum cgroups_systemd_setting cgroups_detect_systemd(const char *exec)
     } else if (ret == 0) {
         collector_info("Cannot get the output of \"%s\" within timeout (%d ms)", exec, timeout);
     } else {
-        while (fgets(buf, MAXSIZE_PROC_CMDLINE, fp_child_output) != NULL) {
+        while (fgets(buf, MAXSIZE_PROC_CMDLINE, spawn_popen_stdout(pi)) != NULL) {
             if ((begin = strstr(buf, SYSTEMD_HIERARCHY_STRING))) {
                 end = begin = begin + strlen(SYSTEMD_HIERARCHY_STRING);
                 if (!*begin)
@@ -123,7 +112,7 @@ static enum cgroups_systemd_setting cgroups_detect_systemd(const char *exec)
         }
     }
 
-    if (netdata_pclose(fp_child_input, fp_child_output, command_pid))
+    if(spawn_popen_wait(pi) != 0)
         return SYSTEMD_CGROUP_ERR;
 
     return retval;
@@ -131,41 +120,56 @@ static enum cgroups_systemd_setting cgroups_detect_systemd(const char *exec)
 
 static enum cgroups_type cgroups_try_detect_version()
 {
-    pid_t command_pid;
+    char filename[FILENAME_MAX + 1];
+    snprintfz(filename, FILENAME_MAX, "%s%s", netdata_configured_host_prefix, "/sys/fs/cgroup");
+    struct statfs fsinfo;
+
+    // https://github.com/systemd/systemd/blob/main/docs/CGROUP_DELEGATION.md#three-different-tree-setups-
+    // ├── statfs("/sys/fs/cgroup/")
+    // │   └── .f_type
+    // │       ├── CGROUP2_SUPER_MAGIC (Unified mode)
+    // │       └── TMPFS_MAGIC (Legacy or Hybrid mode)
+    //         ├── statfs("/sys/fs/cgroup/unified/")
+    //         │   └── .f_type
+    //         │       ├── CGROUP2_SUPER_MAGIC (Hybrid mode)
+    //         │       └── Otherwise, you're in legacy mode
+    if (!statfs(filename, &fsinfo)) {
+#if defined CGROUP2_SUPER_MAGIC
+        if (fsinfo.f_type == CGROUP2_SUPER_MAGIC)
+            return CGROUPS_V2;
+#endif
+#if defined TMPFS_MAGIC
+        if (fsinfo.f_type == TMPFS_MAGIC) {
+            // either hybrid or legacy
+            return CGROUPS_V1;
+        }
+#endif
+    }
+
+    collector_info("cgroups version: can't detect using statfs (fs type), falling back to heuristics.");
+
     char buf[MAXSIZE_PROC_CMDLINE];
     enum cgroups_systemd_setting systemd_setting;
     int cgroups2_available = 0;
 
     // 1. check if cgroups2 available on system at all
-    FILE *fp_child_input;
-    FILE *fp_child_output = netdata_popen("grep cgroup /proc/filesystems", &command_pid, &fp_child_input);
-    if (!fp_child_output) {
-        collector_error("popen failed");
+    POPEN_INSTANCE *pi = spawn_popen_run("grep cgroup /proc/filesystems");
+    if(!pi) {
+        collector_error("cannot run 'grep cgroup /proc/filesystems'");
         return CGROUPS_AUTODETECT_FAIL;
     }
-    while (fgets(buf, MAXSIZE_PROC_CMDLINE, fp_child_output) != NULL) {
+    while (fgets(buf, MAXSIZE_PROC_CMDLINE, spawn_popen_stdout(pi)) != NULL) {
         if (strstr(buf, "cgroup2")) {
             cgroups2_available = 1;
             break;
         }
     }
-    if(netdata_pclose(fp_child_input, fp_child_output, command_pid))
+    if(spawn_popen_wait(pi) != 0)
         return CGROUPS_AUTODETECT_FAIL;
 
     if(!cgroups2_available)
         return CGROUPS_V1;
 
-#if defined CGROUP2_SUPER_MAGIC
-    // 2. check filesystem type for the default mountpoint
-    char filename[FILENAME_MAX + 1];
-    snprintfz(filename, FILENAME_MAX, "%s%s", netdata_configured_host_prefix, "/sys/fs/cgroup");
-    struct statfs fsinfo;
-    if (!statfs(filename, &fsinfo)) {
-        if (fsinfo.f_type == CGROUP2_SUPER_MAGIC)
-            return CGROUPS_V2;
-    }
-#endif
-
     // 3. check systemd compiletime setting
     if ((systemd_setting = cgroups_detect_systemd("systemd --version")) == SYSTEMD_CGROUP_ERR)
         systemd_setting = cgroups_detect_systemd(SYSTEMD_CMD_RHEL);
@@ -225,13 +229,17 @@ void read_cgroup_plugin_configuration() {
     throttled_time_hash = simple_hash("throttled_time");
     throttled_usec_hash = simple_hash("throttled_usec");
 
-    cgroup_update_every = (int)config_get_number("plugin:cgroups", "update every", localhost->rrd_update_every);
-    if(cgroup_update_every < localhost->rrd_update_every)
+    cgroup_update_every = (int)config_get_duration_seconds("plugin:cgroups", "update every", localhost->rrd_update_every);
+    if(cgroup_update_every < localhost->rrd_update_every) {
         cgroup_update_every = localhost->rrd_update_every;
+        config_set_duration_seconds("plugin:cgroups", "update every", localhost->rrd_update_every);
+    }
 
-    cgroup_check_for_new_every = (int)config_get_number("plugin:cgroups", "check for new cgroups every", cgroup_check_for_new_every);
-    if(cgroup_check_for_new_every < cgroup_update_every)
+    cgroup_check_for_new_every = (int)config_get_duration_seconds("plugin:cgroups", "check for new cgroups every", cgroup_check_for_new_every);
+    if(cgroup_check_for_new_every < cgroup_update_every) {
         cgroup_check_for_new_every = cgroup_update_every;
+        config_set_duration_seconds("plugin:cgroups", "check for new cgroups every", cgroup_check_for_new_every);
+    }
 
     cgroup_use_unified_cgroups = config_get_boolean_ondemand("plugin:cgroups", "use unified cgroups", CONFIG_BOOLEAN_AUTO);
     if (cgroup_use_unified_cgroups == CONFIG_BOOLEAN_AUTO)
@@ -1319,7 +1327,6 @@ static void cgroup_main_cleanup(void *pptr) {
 
     static_thread->enabled = NETDATA_MAIN_THREAD_EXITING;
 
-    collector_info("cleaning up...");
     worker_unregister();
 
     usec_t max = 2 * USEC_PER_SEC, step = 50000;
@@ -1397,24 +1404,25 @@ void *cgroups_main(void *ptr) {
     cgroup_netdev_link_init();
 
     rrd_function_add_inline(localhost, NULL, "containers-vms", 10,
-                            RRDFUNCTIONS_PRIORITY_DEFAULT / 2, RRDFUNCTIONS_CGTOP_HELP,
+                            RRDFUNCTIONS_PRIORITY_DEFAULT / 2, RRDFUNCTIONS_VERSION_DEFAULT,
+                            RRDFUNCTIONS_CGTOP_HELP,
                             "top", HTTP_ACCESS_ANONYMOUS_DATA,
                             cgroup_function_cgroup_top);
 
     rrd_function_add_inline(localhost, NULL, "systemd-services", 10,
-                            RRDFUNCTIONS_PRIORITY_DEFAULT / 3, RRDFUNCTIONS_SYSTEMD_SERVICES_HELP,
+                            RRDFUNCTIONS_PRIORITY_DEFAULT / 3, RRDFUNCTIONS_VERSION_DEFAULT,
+                            RRDFUNCTIONS_SYSTEMD_SERVICES_HELP,
                             "top", HTTP_ACCESS_ANONYMOUS_DATA,
                             cgroup_function_systemd_top);
 
     heartbeat_t hb;
-    heartbeat_init(&hb);
-    usec_t step = cgroup_update_every * USEC_PER_SEC;
+    heartbeat_init(&hb, cgroup_update_every * USEC_PER_SEC);
     usec_t find_every = cgroup_check_for_new_every * USEC_PER_SEC, find_dt = 0;
 
     while(service_running(SERVICE_COLLECTORS)) {
         worker_is_idle();
 
-        usec_t hb_dt = heartbeat_next(&hb, step);
+        usec_t hb_dt = heartbeat_next(&hb);
 
         if (unlikely(!service_running(SERVICE_COLLECTORS)))
             break;
