@@ -26,7 +26,7 @@ static inline void *ebpf_cgroup_map_shm_locally(int fd, size_t length)
 {
     void *value;
 
-    value =  mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+    value =  nd_mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
     if (!value) {
         netdata_log_error("Cannot map shared memory used between eBPF and cgroup, integration between processes won't happen");
         close(shm_fd_ebpf_cgroup);
@@ -44,7 +44,7 @@ static inline void *ebpf_cgroup_map_shm_locally(int fd, size_t length)
  */
 void ebpf_unmap_cgroup_shared_memory()
 {
-    munmap(ebpf_mapped_memory, shm_ebpf_cgroup.header->body_length);
+    nd_munmap(ebpf_mapped_memory, shm_ebpf_cgroup.header->body_length);
 }
 
 /**
@@ -87,7 +87,7 @@ void ebpf_map_cgroup_shared_memory()
 
     size_t length =  header->body_length;
 
-    munmap(header, sizeof(netdata_ebpf_cgroup_shm_header_t));
+    nd_munmap(header, sizeof(netdata_ebpf_cgroup_shm_header_t));
 
     if (length <= ((sizeof(netdata_ebpf_cgroup_shm_header_t) + sizeof(netdata_ebpf_cgroup_shm_body_t)))) {
         return;
@@ -105,7 +105,7 @@ void ebpf_map_cgroup_shared_memory()
     if (shm_sem_ebpf_cgroup == SEM_FAILED) {
         netdata_log_error("Cannot create semaphore, integration between eBPF and cgroup won't happen");
         limit_try = NETDATA_EBPF_CGROUP_MAX_TRIES + 1;
-        munmap(ebpf_mapped_memory, length);
+        nd_munmap(ebpf_mapped_memory, length);
         shm_ebpf_cgroup.header = NULL;
         shm_ebpf_cgroup.body = NULL;
         close(shm_fd_ebpf_cgroup);
@@ -329,9 +329,9 @@ void ebpf_parse_cgroup_shm_data()
  */
 void ebpf_create_charts_on_systemd(ebpf_systemd_args_t *chart)
 {
-    ebpf_write_chart_cmd(NETDATA_SERVICE_FAMILY,
-                         chart->id,
+    ebpf_write_chart_cmd(chart->id,
                          chart->suffix,
+                         "",
                          chart->title,
                          chart->units,
                          chart->family,
@@ -340,9 +340,23 @@ void ebpf_create_charts_on_systemd(ebpf_systemd_args_t *chart)
                          chart->order,
                          chart->update_every,
                          chart->module);
-    ebpf_create_chart_labels("service_name", chart->id, RRDLABEL_SRC_AUTO);
+    char service_name[512];
+    snprintfz(service_name, 511, "%s", (!strstr(chart->id, "systemd_")) ? chart->id : (chart->id + 8));
+    ebpf_create_chart_labels("service_name", service_name, RRDLABEL_SRC_AUTO);
     ebpf_commit_label();
-    fprintf(stdout, "DIMENSION %s '' %s 1 1\n", chart->dimension, chart->algorithm);
+    // Let us keep original string that can be used in another place. Chart creation does not happen frequently.
+    char *move = strdupz(chart->dimension);
+    while (move) {
+        char *next_dim = strchr(move, ',');
+        if (next_dim) {
+            *next_dim = '\0';
+            next_dim++;
+        }
+
+        fprintf(stdout, "DIMENSION %s '' %s 1 1\n", move, chart->algorithm);
+        move = next_dim;
+    }
+    freez(move);
 }
 
 // --------------------------------------------------------------------------------------------------------------------
@@ -359,13 +373,12 @@ void ebpf_create_charts_on_systemd(ebpf_systemd_args_t *chart)
  */
 void *ebpf_cgroup_integration(void *ptr __maybe_unused)
 {
-    usec_t step = USEC_PER_SEC;
     int counter = NETDATA_EBPF_CGROUP_UPDATE - 1;
     heartbeat_t hb;
-    heartbeat_init(&hb);
+    heartbeat_init(&hb, USEC_PER_SEC);
     //Plugin will be killed when it receives a signal
     while (!ebpf_plugin_stop()) {
-        (void)heartbeat_next(&hb, step);
+        heartbeat_next(&hb);
 
         // We are using a small heartbeat time to wake up thread,
         // but we should not update so frequently the shared memory data
