@@ -747,8 +747,8 @@ void nfacct_signals()
 }
 
 int main(int argc, char **argv) {
-    clocks_init();
     nd_log_initialize_for_external_plugins("nfacct.plugin");
+    netdata_threads_init_for_external_plugins(0);
 
     // ------------------------------------------------------------------------
     // parse command line parameters
@@ -774,9 +774,8 @@ int main(int argc, char **argv) {
             fprintf(stderr,
                     "\n"
                     " netdata nfacct.plugin %s\n"
-                    " Copyright (C) 2015-2017 Costa Tsaousis <costa@tsaousis.gr>\n"
+                    " Copyright 2018-2025 Netdata Inc.\n"
                     " Released under GNU General Public License v3 or later.\n"
-                    " All rights reserved.\n"
                     "\n"
                     " This program is a data collector plugin for netdata.\n"
                     "\n"
@@ -809,7 +808,7 @@ int main(int argc, char **argv) {
 
     nfacct_signals();
 
-    errno = 0;
+    errno_clear();
 
     if(freq >= netdata_update_every)
         netdata_update_every = freq;
@@ -832,12 +831,11 @@ int main(int argc, char **argv) {
     time_t started_t = now_monotonic_sec();
 
     size_t iteration;
-    usec_t step = netdata_update_every * USEC_PER_SEC;
 
     heartbeat_t hb;
-    heartbeat_init(&hb);
+    heartbeat_init(&hb, netdata_update_every * USEC_PER_SEC);
     for(iteration = 0; 1; iteration++) {
-        usec_t dt = heartbeat_next(&hb, step);
+        usec_t dt = heartbeat_next(&hb);
 
         if(unlikely(netdata_exit)) break;
 
