@@ -6,6 +6,9 @@
 #include <sys/msg.h>
 #include <sys/shm.h>
 
+#define _COMMON_PLUGIN_NAME PLUGIN_PROC_NAME
+#define _COMMON_PLUGIN_MODULE_NAME "ipc"
+#include "../common-contexts/common-contexts.h"
 
 #ifndef SEMVMX
 #define SEMVMX  32767  /* <= 32767 semaphore maximum value */
@@ -179,7 +182,7 @@ static inline int ipc_sem_get_status(struct ipc_status *st) {
     return 0;
 }
 
-int ipc_msq_get_info(char *msg_filename, struct message_queue **message_queue_root) {
+static int ipc_msq_get_info(const char *msg_filename, struct message_queue **message_queue_root) {
     static procfile *ff;
     struct message_queue *msq;
 
@@ -235,7 +238,7 @@ int ipc_msq_get_info(char *msg_filename, struct message_queue **message_queue_ro
     return 0;
 }
 
-int ipc_shm_get_info(char *shm_filename, struct shm_stats *shm) {
+static int ipc_shm_get_info(const char *shm_filename, struct shm_stats *shm) {
     static procfile *ff;
 
     if(unlikely(!ff)) {
@@ -282,12 +285,12 @@ int do_ipc(int update_every, usec_t dt) {
     static struct ipc_limits limits;
     static struct ipc_status status;
     static const RRDVAR_ACQUIRED *arrays_max = NULL, *semaphores_max = NULL;
-    static RRDSET *st_semaphores = NULL, *st_arrays = NULL;
-    static RRDDIM *rd_semaphores = NULL, *rd_arrays = NULL;
-    static char *msg_filename = NULL;
+    static RRDSET *st_arrays = NULL;
+    static RRDDIM *rd_arrays = NULL;
+    static const char *msg_filename = NULL;
     static struct message_queue *message_queue_root = NULL;
     static long long dimensions_limit;
-    static char *shm_filename = NULL;
+    static const char *shm_filename = NULL;
 
     if(unlikely(do_sem == -1)) {
         do_msg = config_get_boolean("plugin:proc:ipc", "message queues", CONFIG_BOOLEAN_YES);
@@ -314,25 +317,7 @@ int do_ipc(int update_every, usec_t dt) {
             do_sem = CONFIG_BOOLEAN_NO;
         }
         else {
-            // create the charts
-            if(unlikely(!st_semaphores)) {
-                st_semaphores = rrdset_create_localhost(
-                        "system"
-                        , "ipc_semaphores"
-                        , NULL
-                        , "ipc semaphores"
-                        , NULL
-                        , "IPC Semaphores"
-                        , "semaphores"
-                        , PLUGIN_PROC_NAME
-                        , "ipc"
-                        , NETDATA_CHART_PRIO_SYSTEM_IPC_SEMAPHORES
-                        , localhost->rrd_update_every
-                        , RRDSET_TYPE_AREA
-                );
-                rd_semaphores = rrddim_add(st_semaphores, "semaphores", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            }
-
+            // create the chart
             if(unlikely(!st_arrays)) {
                 st_arrays = rrdset_create_localhost(
                         "system"
@@ -379,7 +364,6 @@ int do_ipc(int update_every, usec_t dt) {
                     rrdvar_host_variable_set(localhost, arrays_max, limits.semmni);
 
                 st_arrays->red = limits.semmni;
-                st_semaphores->red = limits.semmns;
 
                 read_limits_next = 60 / update_every;
             }
@@ -392,8 +376,7 @@ int do_ipc(int update_every, usec_t dt) {
             return 0;
         }
 
-        rrddim_set_by_pointer(st_semaphores, rd_semaphores, status.semaem);
-        rrdset_done(st_semaphores);
+        common_semaphore_ipc(status.semaem, limits.semmns, "ipc", localhost->rrd_update_every);
 
         rrddim_set_by_pointer(st_arrays, rd_arrays, status.semusz);
         rrdset_done(st_arrays);
