@@ -37,7 +37,6 @@ int do_proc_sys_devices_system_edac_mc(int update_every, usec_t dt);
 int do_proc_sys_devices_pci_aer(int update_every, usec_t dt);
 int do_proc_sys_devices_system_node(int update_every, usec_t dt);
 int do_proc_spl_kstat_zfs_arcstats(int update_every, usec_t dt);
-int do_proc_spl_kstat_zfs_pool_state(int update_every, usec_t dt);
 int do_sys_fs_btrfs(int update_every, usec_t dt);
 int do_proc_net_sockstat(int update_every, usec_t dt);
 int do_proc_net_sockstat6(int update_every, usec_t dt);
@@ -48,6 +47,7 @@ int do_proc_pagetypeinfo(int update_every, usec_t dt);
 int do_sys_class_infiniband(int update_every, usec_t dt);
 int do_sys_class_drm(int update_every, usec_t dt);
 int get_numa_node_count(void);
+int do_run_reboot_required(int update_every, usec_t dt);
 
 // metrics that need to be shared among data collectors
 extern unsigned long long zfs_arcstats_shrinkable_cache_size_bytes;
