@@ -128,7 +128,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
         do_frag_sockets    = config_get_boolean_ondemand("plugin:proc:/proc/net/sockstat", "ipv4 FRAG sockets", CONFIG_BOOLEAN_AUTO);
         do_frag_mem        = config_get_boolean_ondemand("plugin:proc:/proc/net/sockstat", "ipv4 FRAG memory", CONFIG_BOOLEAN_AUTO);
 
-        update_constants_every = config_get_number("plugin:proc:/proc/net/sockstat", "update constants every", update_constants_every);
+        update_constants_every = config_get_duration_seconds("plugin:proc:/proc/net/sockstat", "update constants every", update_constants_every);
         update_constants_count = update_constants_every;
 
         arl_sockets = arl_create("sockstat/sockets", arl_callback_str2kernel_uint_t, 60);
@@ -218,9 +218,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_sockets == CONFIG_BOOLEAN_YES || (do_sockets == CONFIG_BOOLEAN_AUTO &&
-                                            (sockstat_root.sockets_used ||
-                                             netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_sockets == CONFIG_BOOLEAN_YES || do_sockets == CONFIG_BOOLEAN_AUTO) {
         do_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -251,12 +249,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_tcp_sockets == CONFIG_BOOLEAN_YES || (do_tcp_sockets == CONFIG_BOOLEAN_AUTO &&
-                                                (sockstat_root.tcp_inuse ||
-                                                 sockstat_root.tcp_orphan ||
-                                                 sockstat_root.tcp_tw ||
-                                                 sockstat_root.tcp_alloc ||
-                                                 netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_tcp_sockets == CONFIG_BOOLEAN_YES || do_tcp_sockets == CONFIG_BOOLEAN_AUTO) {
         do_tcp_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -296,8 +289,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_tcp_mem == CONFIG_BOOLEAN_YES || (do_tcp_mem == CONFIG_BOOLEAN_AUTO &&
-                                            (sockstat_root.tcp_mem || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_tcp_mem == CONFIG_BOOLEAN_YES || do_tcp_mem == CONFIG_BOOLEAN_AUTO) {
         do_tcp_mem = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -328,9 +320,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_udp_sockets == CONFIG_BOOLEAN_YES || (do_udp_sockets == CONFIG_BOOLEAN_AUTO &&
-                                                (sockstat_root.udp_inuse ||
-                                                 netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_udp_sockets == CONFIG_BOOLEAN_YES || do_udp_sockets == CONFIG_BOOLEAN_AUTO) {
         do_udp_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -361,9 +351,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_udp_mem == CONFIG_BOOLEAN_YES || (do_udp_mem == CONFIG_BOOLEAN_AUTO &&
-                                            (sockstat_root.udp_mem ||
-                                             netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_udp_mem == CONFIG_BOOLEAN_YES || do_udp_mem == CONFIG_BOOLEAN_AUTO) {
         do_udp_mem = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -394,9 +382,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_udplite_sockets == CONFIG_BOOLEAN_YES || (do_udplite_sockets == CONFIG_BOOLEAN_AUTO &&
-                                                    (sockstat_root.udplite_inuse ||
-                                                     netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_udplite_sockets == CONFIG_BOOLEAN_YES || do_udplite_sockets == CONFIG_BOOLEAN_AUTO) {
         do_udplite_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -427,9 +413,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_raw_sockets == CONFIG_BOOLEAN_YES || (do_raw_sockets == CONFIG_BOOLEAN_AUTO &&
-                                                (sockstat_root.raw_inuse ||
-                                                 netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_raw_sockets == CONFIG_BOOLEAN_YES || do_raw_sockets == CONFIG_BOOLEAN_AUTO) {
         do_raw_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -460,9 +444,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_frag_sockets == CONFIG_BOOLEAN_YES || (do_frag_sockets == CONFIG_BOOLEAN_AUTO &&
-                                                 (sockstat_root.frag_inuse ||
-                                                  netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_frag_sockets == CONFIG_BOOLEAN_YES || do_frag_sockets == CONFIG_BOOLEAN_AUTO) {
         do_frag_sockets = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
@@ -493,9 +475,7 @@ int do_proc_net_sockstat(int update_every, usec_t dt) {
 
     // ------------------------------------------------------------------------
 
-    if(do_frag_mem == CONFIG_BOOLEAN_YES || (do_frag_mem == CONFIG_BOOLEAN_AUTO &&
-                                             (sockstat_root.frag_memory ||
-                                              netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
+    if (do_frag_mem == CONFIG_BOOLEAN_YES || do_frag_mem == CONFIG_BOOLEAN_AUTO) {
         do_frag_mem = CONFIG_BOOLEAN_YES;
 
         static RRDSET *st = NULL;
