@@ -3,7 +3,7 @@
 #ifndef NETDATA_WINDOWS_INTERNALS_H
 #define NETDATA_WINDOWS_INTERNALS_H
 
-#include <windows.h>
+#include "libnetdata/libnetdata.h"
 
 static inline ULONGLONG FileTimeToULL(FILETIME ft) {
     ULARGE_INTEGER ul;
@@ -12,7 +12,6 @@ static inline ULONGLONG FileTimeToULL(FILETIME ft) {
     return ul.QuadPart;
 }
 
-#include "perflib.h"
 #include "perflib-rrd.h"
 
 #endif //NETDATA_WINDOWS_INTERNALS_H
