@@ -4,6 +4,7 @@
 #define NETDATA_ANALYTICS_H 1
 
 #include "daemon/common.h"
+#include "database/rrdhost-system-info.h"
 
 /* Max number of seconds before the first META analytics is sent */
 #define ANALYTICS_INIT_SLEEP_SEC 120
@@ -76,9 +77,8 @@ struct analytics_data {
     bool exporting_enabled;
 };
 
-void set_late_global_environment(struct rrdhost_system_info *system_info);
+void set_late_analytics_variables(struct rrdhost_system_info *system_info);
 void analytics_free_data(void);
-void set_global_environment(void);
 void analytics_log_shell(void);
 void analytics_log_json(void);
 void analytics_log_prometheus(void);
@@ -86,6 +86,8 @@ void analytics_log_dashboard(void);
 void analytics_gather_mutable_meta_data(void);
 void analytics_report_oom_score(long long int score);
 void get_system_timezone(void);
+void analytics_reset(void);
+void analytics_init(void);
 
 typedef struct {
     const char *action;
