@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+#define RRDHOST_SYSTEM_INFO_INTERNALS
 #include <stdio.h>
 #include "./config.h"
 #include "common.h"
@@ -54,6 +55,7 @@ typedef enum __attribute__((packed)) {
     BIB_FEATURE_CONTEXTS,
     BIB_FEATURE_TIERING,
     BIB_FEATURE_ML,
+    BIB_FEATURE_ALLOCATOR,
     BIB_DB_DBENGINE,
     BIB_DB_ALLOC,
     BIB_DB_RAM,
@@ -75,6 +77,7 @@ typedef enum __attribute__((packed)) {
     BIB_LIB_LIBCAP,
     BIB_LIB_LIBCRYPTO,
     BIB_LIB_LIBYAML,
+    BIB_LIB_LIBMNL,
     BIB_PLUGIN_APPS,
     BIB_PLUGIN_LINUX_CGROUPS,
     BIB_PLUGIN_LINUX_CGROUP_NETWORK,
@@ -96,7 +99,6 @@ typedef enum __attribute__((packed)) {
     BIB_PLUGIN_SLABINFO,
     BIB_PLUGIN_XEN,
     BIB_PLUGIN_XEN_VBD_ERROR,
-    BIB_PLUGIN_LOGS_MANAGEMENT,
     BIB_EXPORT_AWS_KINESIS,
     BIB_EXPORT_GCP_PUBSUB,
     BIB_EXPORT_MONGOC,
@@ -112,6 +114,11 @@ typedef enum __attribute__((packed)) {
     BIB_EXPORT_SHELL,
     BIB_DEVEL_TRACE_ALLOCATIONS,
     BIB_DEVELOPER_MODE,
+    BIB_RUNTIME_PROFILE,
+    BIB_RUNTIME_PARENT,
+    BIB_RUNTIME_CHILD,
+    BIB_RUNTIME_MEM_TOTAL,
+    BIB_RUNTIME_MEM_AVAIL,
 
     // leave this last
     BIB_TERMINATOR,
@@ -129,7 +136,8 @@ typedef enum __attribute__((packed)) {
     BIC_LIBS,
     BIC_PLUGINS,
     BIC_EXPORTERS,
-    BIC_DEBUG_DEVEL
+    BIC_DEBUG_DEVEL,
+    BIC_RUNTIME,
 } BUILD_INFO_CATEGORY;
 
 typedef enum __attribute__((packed)) {
@@ -530,6 +538,14 @@ static struct {
                 .json = "ml",
                 .value = NULL,
         },
+        [BIB_FEATURE_ALLOCATOR] = {
+                .category = BIC_FEATURE,
+                .type = BIT_STRING,
+                .analytics = "allocator",
+                .print = "Memory Allocator",
+                .json = "allocator",
+                .value = NULL,
+        },
         [BIB_DB_DBENGINE] = {
                 .category = BIC_DATABASE,
                 .type = BIT_BOOLEAN,
@@ -698,6 +714,14 @@ static struct {
             .json = "libyaml",
             .value = NULL,
         },
+        [BIB_LIB_LIBMNL] = {
+            .category = BIC_LIBS,
+            .type = BIT_BOOLEAN,
+            .analytics = "libmnl",
+            .print = "libmnl (library for working with netfilter)",
+            .json = "libmnl",
+            .value = NULL,
+        },
         [BIB_PLUGIN_APPS] = {
                 .category = BIC_PLUGINS,
                 .type = BIT_BOOLEAN,
@@ -866,14 +890,6 @@ static struct {
                 .json = "xen-vbd-error",
                 .value = NULL,
         },
-        [BIB_PLUGIN_LOGS_MANAGEMENT] = {
-                .category = BIC_PLUGINS,
-                .type = BIT_BOOLEAN,
-                .analytics = "Logs Management",
-                .print = "Logs Management",
-                .json = "logs-management",
-                .value = NULL,
-        },
         [BIB_EXPORT_MONGOC] = {
                 .category = BIC_EXPORTERS,
                 .type = BIT_BOOLEAN,
@@ -994,6 +1010,46 @@ static struct {
                 .json = "dev-mode",
                 .value = NULL,
         },
+        [BIB_RUNTIME_PROFILE] = {
+            .category = BIC_RUNTIME,
+            .type = BIT_STRING,
+            .analytics = "ConfigProfile",
+            .print = "Profile",
+            .json = "profile",
+            .value = NULL,
+        },
+        [BIB_RUNTIME_PARENT] = {
+            .category = BIC_RUNTIME,
+            .type = BIT_BOOLEAN,
+            .analytics = "StreamParent",
+            .print = "Stream Parent (accept data from Children)",
+            .json = "parent",
+            .value = NULL,
+        },
+        [BIB_RUNTIME_CHILD] = {
+            .category = BIC_RUNTIME,
+            .type = BIT_BOOLEAN,
+            .analytics = "StreamChild",
+            .print = "Stream Child (send data to a Parent)",
+            .json = "child",
+            .value = NULL,
+        },
+        [BIB_RUNTIME_MEM_TOTAL] = {
+            .category = BIC_RUNTIME,
+            .type = BIT_STRING,
+            .analytics = "TotalMemory",
+            .print = "Total System Memory",
+            .json = "mem-total",
+            .value = NULL,
+        },
+        [BIB_RUNTIME_MEM_AVAIL] = {
+            .category = BIC_RUNTIME,
+            .type = BIT_STRING,
+            .analytics = "AvailableMemory",
+            .print = "Available System Memory",
+            .json = "mem-available",
+            .value = NULL,
+        },
 
         // leave this last
         [BIB_TERMINATOR] = {
@@ -1069,18 +1125,8 @@ __attribute__((constructor)) void initialize_build_info(void) {
 #endif
 #endif
 
-#ifdef ENABLE_ACLK
     build_info_set_status(BIB_FEATURE_CLOUD, true);
     build_info_set_status(BIB_CONNECTIVITY_ACLK, true);
-#else
-    build_info_set_status(BIB_FEATURE_CLOUD, false);
-#ifdef DISABLE_CLOUD
-    build_info_set_value(BIB_FEATURE_CLOUD, "disabled");
-#else
-    build_info_set_value(BIB_FEATURE_CLOUD, "unavailable");
-#endif
-#endif
-
     build_info_set_status(BIB_FEATURE_HEALTH, true);
     build_info_set_status(BIB_FEATURE_STREAMING, true);
     build_info_set_status(BIB_FEATURE_BACKFILLING, true);
@@ -1106,6 +1152,15 @@ __attribute__((constructor)) void initialize_build_info(void) {
     build_info_set_status(BIB_FEATURE_ML, true);
 #endif
 
+#if defined(ENABLE_MIMALLOC)
+    build_info_set_status(BIB_FEATURE_ALLOCATOR, true);
+    build_info_set_value(BIB_FEATURE_ALLOCATOR, "mimalloc");
+#else
+    build_info_set_status(BIB_FEATURE_ALLOCATOR, true);
+    build_info_set_value(BIB_FEATURE_ALLOCATOR, "system");
+#endif
+
+
 #ifdef ENABLE_DBENGINE
     build_info_set_status(BIB_DB_DBENGINE, true);
 #ifdef ENABLE_ZSTD
@@ -1126,9 +1181,7 @@ __attribute__((constructor)) void initialize_build_info(void) {
 #ifdef ENABLE_WEBRTC
     build_info_set_status(BIB_CONNECTIVITY_WEBRTC, true);
 #endif
-#ifdef ENABLE_HTTPS
     build_info_set_status(BIB_CONNECTIVITY_NATIVE_HTTPS, true);
-#endif
 #if defined(HAVE_X509_VERIFY_PARAM_set1_host) && HAVE_X509_VERIFY_PARAM_set1_host == 1
     build_info_set_status(BIB_CONNECTIVITY_TLS_HOST_VERIFY, true);
 #endif
@@ -1162,9 +1215,7 @@ __attribute__((constructor)) void initialize_build_info(void) {
 #ifdef HAVE_LIBDATACHANNEL
     build_info_set_status(BIB_LIB_LIBDATACHANNEL, true);
 #endif
-#ifdef ENABLE_OPENSSL
     build_info_set_status(BIB_LIB_OPENSSL, true);
-#endif
 #ifdef ENABLE_JSONC
     build_info_set_status(BIB_LIB_JSONC, true);
 #endif
@@ -1177,6 +1228,9 @@ __attribute__((constructor)) void initialize_build_info(void) {
 #ifdef HAVE_LIBYAML
     build_info_set_status(BIB_LIB_LIBYAML, true);
 #endif
+#ifdef HAVE_LIBMNL
+    build_info_set_status(BIB_LIB_LIBMNL, true);
+#endif
 
 #ifdef ENABLE_PLUGIN_APPS
     build_info_set_status(BIB_PLUGIN_APPS, true);
@@ -1217,9 +1271,6 @@ __attribute__((constructor)) void initialize_build_info(void) {
 #ifdef HAVE_XENSTAT_VBD_ERROR
     build_info_set_status(BIB_PLUGIN_XEN_VBD_ERROR, true);
 #endif
-#ifdef ENABLE_LOGSMANAGEMENT
-    build_info_set_status(BIB_PLUGIN_LOGS_MANAGEMENT, true);
-#endif
 
     build_info_set_status(BIB_EXPORT_PROMETHEUS_EXPORTER, true);
     build_info_set_status(BIB_EXPORT_GRAPHITE, true);
@@ -1256,10 +1307,9 @@ __attribute__((constructor)) void initialize_build_info(void) {
 // ----------------------------------------------------------------------------
 // system info
 
-int get_system_info(struct rrdhost_system_info *system_info);
 static void populate_system_info(void) {
     static bool populated = false;
-    static SPINLOCK spinlock = NETDATA_SPINLOCK_INITIALIZER;
+    static SPINLOCK spinlock = SPINLOCK_INITIALIZER;
 
     if(populated)
         return;
@@ -1278,9 +1328,18 @@ static void populate_system_info(void) {
         system_info = localhost->system_info;
     }
     else {
-        system_info = callocz(1, sizeof(struct rrdhost_system_info));
-        get_system_info(system_info);
+        bool started_spawn_server = false;
+        if(!netdata_main_spawn_server) {
+            started_spawn_server = true;
+            netdata_main_spawn_server_init(NULL, 0, NULL);
+        }
+
+        system_info = rrdhost_system_info_create();
+        rrdhost_system_info_detect(system_info);
         free_system_info = true;
+
+        if(started_spawn_server)
+            netdata_main_spawn_server_cleanup();
     }
 
     build_info_set_value_strdupz(BIB_OS_KERNEL_NAME, system_info->kernel_name);
@@ -1336,10 +1395,11 @@ char *get_value_from_key(char *buffer, char *key) {
     return s;
 }
 
-void get_install_type(char **install_type, char **prebuilt_arch, char **prebuilt_dist) {
+void get_install_type_internal(char **install_type, char **prebuilt_arch __maybe_unused, char **prebuilt_dist __maybe_unused) {
+#ifndef OS_WINDOWS
     char *install_type_filename;
 
-    int install_type_filename_len = (strlen(netdata_configured_user_config_dir) + strlen(".install-type") + 3);
+    unsigned long install_type_filename_len = (strlen(netdata_configured_user_config_dir) + strlen(".install-type") + 3);
     install_type_filename = mallocz(sizeof(char) * install_type_filename_len);
     snprintfz(install_type_filename, install_type_filename_len - 1, "%s/%s", netdata_configured_user_config_dir, ".install-type");
 
@@ -1359,6 +1419,13 @@ void get_install_type(char **install_type, char **prebuilt_arch, char **prebuilt
         fclose(fp);
     }
     freez(install_type_filename);
+#else
+    *install_type = strdupz("netdata_installer.exe");
+#endif
+}
+
+void get_install_type(struct rrdhost_system_info *system_info) {
+    get_install_type_internal(&system_info->install_type, &system_info->prebuilt_arch, &system_info->prebuilt_dist);
 }
 
 static struct {
@@ -1375,7 +1442,7 @@ static void populate_packaging_info() {
         if(!BUILD_PACKAGING_INFO.populated) {
             BUILD_PACKAGING_INFO.populated = true;
 
-            get_install_type(&BUILD_PACKAGING_INFO.install_type, &BUILD_PACKAGING_INFO.prebuilt_arch, &BUILD_PACKAGING_INFO.prebuilt_distro);
+            get_install_type_internal(&BUILD_PACKAGING_INFO.install_type, &BUILD_PACKAGING_INFO.prebuilt_arch, &BUILD_PACKAGING_INFO.prebuilt_distro);
 
             if(!BUILD_PACKAGING_INFO.install_type)
                 BUILD_PACKAGING_INFO.install_type = "unknown";
@@ -1389,9 +1456,26 @@ static void populate_packaging_info() {
             build_info_set_value(BIB_PACKAGING_INSTALL_TYPE, strdupz(BUILD_PACKAGING_INFO.install_type));
             build_info_set_value(BIB_PACKAGING_ARCHITECTURE, strdupz(BUILD_PACKAGING_INFO.prebuilt_arch));
             build_info_set_value(BIB_PACKAGING_DISTRO, strdupz(BUILD_PACKAGING_INFO.prebuilt_distro));
+
+            CLEAN_BUFFER *wb = buffer_create(0, NULL);
+            ND_PROFILE_2buffer(wb, nd_profile_detect_and_configure(false), " ");
+            build_info_set_value_strdupz(BIB_RUNTIME_PROFILE, buffer_tostring(wb));
+
+            build_info_set_status(BIB_RUNTIME_PARENT, stream_conf_is_parent(false));
+            build_info_set_status(BIB_RUNTIME_CHILD, stream_conf_is_child());
         }
         spinlock_unlock(&BUILD_PACKAGING_INFO.spinlock);
     }
+
+    OS_SYSTEM_MEMORY sm = os_system_memory(true);
+    char buf[1024];
+    snprintfz(buf, sizeof(buf), "%" PRIu64, sm.ram_total_bytes);
+    // size_snprintf(buf, sizeof(buf), sm.ram_total_bytes, "B", false);
+    build_info_set_value_strdupz(BIB_RUNTIME_MEM_TOTAL, buf);
+
+    snprintfz(buf, sizeof(buf), "%" PRIu64, sm.ram_available_bytes);
+    // size_snprintf(buf, sizeof(buf), sm.ram_available_bytes, "B", false);
+    build_info_set_value_strdupz(BIB_RUNTIME_MEM_AVAIL, buf);
 }
 
 // ----------------------------------------------------------------------------
@@ -1464,6 +1548,7 @@ void print_build_info(void) {
     print_build_info_category_to_console(BIC_PLUGINS, "Plugins");
     print_build_info_category_to_console(BIC_EXPORTERS, "Exporters");
     print_build_info_category_to_console(BIC_DEBUG_DEVEL, "Debug/Developer Features");
+    print_build_info_category_to_console(BIC_RUNTIME, "Runtime Information");
 }
 
 void build_info_to_json_object(BUFFER *b) {
@@ -1483,6 +1568,7 @@ void build_info_to_json_object(BUFFER *b) {
     print_build_info_category_to_json(b, BIC_PLUGINS, "plugins");
     print_build_info_category_to_json(b, BIC_EXPORTERS, "exporters");
     print_build_info_category_to_json(b, BIC_DEBUG_DEVEL, "debug-n-devel");
+    print_build_info_category_to_json(b, BIC_RUNTIME, "runtime");
 }
 
 void print_build_info_json(void) {
