@@ -3,34 +3,24 @@
 #include "common.h"
 #include <sched.h>
 
-char pidfile[FILENAME_MAX + 1] = "";
-char claiming_directory[FILENAME_MAX + 1];
-char netdata_exe_path[FILENAME_MAX + 1];
-char netdata_exe_file[FILENAME_MAX + 1];
+char *pidfile = NULL;
+char *netdata_exe_path = NULL;
 
 void get_netdata_execution_path(void) {
-    int ret;
-    size_t exepath_size = 0;
-    struct passwd *passwd = NULL;
-    char *user = NULL;
+    struct passwd *passwd = getpwuid(getuid());
+    char *user = (passwd && passwd->pw_name) ? passwd->pw_name : "";
 
-    passwd = getpwuid(getuid());
-    user = (passwd && passwd->pw_name) ? passwd->pw_name : "";
-
-    exepath_size = sizeof(netdata_exe_file) - 1;
-    ret = uv_exepath(netdata_exe_file, &exepath_size);
-    if (0 != ret) {
-        netdata_log_error("uv_exepath(\"%s\", %u) (user: %s) failed (%s).", netdata_exe_file, (unsigned)exepath_size, user,
-                          uv_strerror(ret));
-        fatal("Cannot start netdata without getting execution path.");
+    char b[FILENAME_MAX + 1];
+    size_t b_size = sizeof(b) - 1;
+    int ret = uv_exepath(b, &b_size);
+    if (ret != 0) {
+        fatal("Cannot start netdata without getting execution path. "
+            "(uv_exepath(\"%s\", %zu), user: '%s', failed: %s).",
+            b, b_size, user, uv_strerror(ret));
     }
+    b[b_size] = '\0';
 
-    netdata_exe_file[exepath_size] = '\0';
-
-    // macOS's dirname(3) does not modify passed string
-    char *tmpdir = strdupz(netdata_exe_file);
-    strcpy(netdata_exe_path, dirname(tmpdir));
-    freez(tmpdir);
+    netdata_exe_path = strdupz(b);
 }
 
 static void fix_directory_file_permissions(const char *dirname, uid_t uid, gid_t gid, bool recursive)
@@ -68,7 +58,7 @@ static void change_dir_ownership(const char *dir, uid_t uid, gid_t gid, bool rec
     fix_directory_file_permissions(dir, uid, gid, recursive);
 }
 
-static void clean_directory(char *dirname)
+static void clean_directory(const char *dirname)
 {
     DIR *dir = opendir(dirname);
     if(!dir) return;
@@ -89,7 +79,7 @@ static void prepare_required_directories(uid_t uid, gid_t gid) {
     change_dir_ownership(netdata_configured_varlib_dir, uid, gid, false);
     change_dir_ownership(netdata_configured_lock_dir, uid, gid, false);
     change_dir_ownership(netdata_configured_log_dir, uid, gid, false);
-    change_dir_ownership(claiming_directory, uid, gid, false);
+    change_dir_ownership(netdata_configured_cloud_dir, uid, gid, false);
 
     char filename[FILENAME_MAX + 1];
     snprintfz(filename, FILENAME_MAX, "%s/registry", netdata_configured_varlib_dir);
@@ -112,7 +102,7 @@ static int become_user(const char *username, int pid_fd) {
 
     prepare_required_directories(uid, gid);
 
-    if(pidfile[0]) {
+    if(pidfile && *pidfile) {
         if(chown(pidfile, uid, gid) == -1)
             netdata_log_error("Cannot chown '%s' to %u:%u", pidfile, (unsigned int)uid, (unsigned int)gid);
     }
@@ -198,7 +188,7 @@ static void oom_score_adj(void) {
     }
 
     // check the environment
-    char *s = getenv("OOMScoreAdjust");
+    const char *s = getenv("OOMScoreAdjust");
     if(!s || !*s) {
         snprintfz(buf, sizeof(buf) - 1, "%d", (int)wanted_score);
         s = buf;
@@ -258,7 +248,7 @@ static void oom_score_adj(void) {
 
 static void process_nice_level(void) {
 #ifdef HAVE_NICE
-    int nice_level = (int)config_get_number(CONFIG_SECTION_GLOBAL, "process nice level", 19);
+    int nice_level = (int)config_get_number(CONFIG_SECTION_GLOBAL, "process nice level", 0);
     if(nice(nice_level) == -1)
         netdata_log_error("Cannot set netdata CPU nice level to %d.", nice_level);
     else
@@ -381,14 +371,14 @@ static void sched_setscheduler_set(void) {
                     priority = (int)config_get_number(CONFIG_SECTION_GLOBAL, "process scheduling priority", priority);
 
 #ifdef HAVE_SCHED_GET_PRIORITY_MIN
-                errno = 0;
+                errno_clear();
                 if(priority < sched_get_priority_min(policy)) {
                     netdata_log_error("scheduler %s (%d) priority %d is below the minimum %d. Using the minimum.", name, policy, priority, sched_get_priority_min(policy));
                     priority = sched_get_priority_min(policy);
                 }
 #endif
 #ifdef HAVE_SCHED_GET_PRIORITY_MAX
-                errno = 0;
+                errno_clear();
                 if(priority > sched_get_priority_max(policy)) {
                     netdata_log_error("scheduler %s (%d) priority %d is above the maximum %d. Using the maximum.", name, policy, priority, sched_get_priority_max(policy));
                     priority = sched_get_priority_max(policy);
@@ -407,7 +397,7 @@ static void sched_setscheduler_set(void) {
                 .sched_priority = priority
         };
 
-        errno = 0;
+        errno_clear();
         i = sched_setscheduler(0, policy, &param);
         if(i != 0) {
             netdata_log_error("Cannot adjust netdata scheduling policy to %s (%d), with priority %d. Falling back to nice.",
@@ -442,9 +432,8 @@ int become_daemon(int dont_fork, const char *user)
             perror("cannot fork");
             exit(1);
         }
-        if(i != 0) {
-            exit(0); // the parent
-        }
+        if(i != 0) exit(0); // the parent
+        gettid_uncached();
 
         // become session leader
         if (setsid() < 0) {
@@ -458,14 +447,13 @@ int become_daemon(int dont_fork, const char *user)
             perror("cannot fork");
             exit(1);
         }
-        if(i != 0) {
-            exit(0); // the parent
-        }
+        if(i != 0) exit(0); // the parent
+        gettid_uncached();
     }
 
     // generate our pid file
     int pidfd = -1;
-    if(pidfile[0]) {
+    if(pidfile && *pidfile) {
         pidfd = open(pidfile, O_WRONLY | O_CREAT | O_CLOEXEC, 0644);
         if(pidfd >= 0) {
             if(ftruncate(pidfd, 0) != 0)
@@ -490,9 +478,6 @@ int become_daemon(int dont_fork, const char *user)
     // never become a problem
     sched_setscheduler_set();
 
-    // Set claiming directory based on user config directory with correct ownership
-    snprintfz(claiming_directory, FILENAME_MAX, "%s/cloud.d", netdata_configured_varlib_dir);
-
     if(user && *user) {
         if(become_user(user, pidfd) != 0) {
             netdata_log_error("Cannot become user '%s'. Continuing as we are.", user);
