@@ -1,13 +1,11 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "common.h"
+#include "web/api/queries/backfill.h"
 
 void *aclk_main(void *ptr);
 void *analytics_main(void *ptr);
 void *cpuidlejitter_main(void *ptr);
-void *global_statistics_main(void *ptr);
-void *global_statistics_workers_main(void *ptr);
-void *global_statistics_sqlite3_main(void *ptr);
 void *health_main(void *ptr);
 void *pluginsd_main(void *ptr);
 void *service_main(void *ptr);
@@ -15,7 +13,7 @@ void *statsd_main(void *ptr);
 void *profile_main(void *ptr);
 void *replication_thread_main(void *ptr);
 
-extern bool global_statistics_enabled;
+extern bool pulse_enabled;
 
 const struct netdata_static_thread static_threads_common[] = {
     {
@@ -46,38 +44,48 @@ const struct netdata_static_thread static_threads_common[] = {
         .start_routine = analytics_main
     },
     {
-        .name = "STATS_GLOBAL",
+        .name = "PULSE",
         .config_section = CONFIG_SECTION_PLUGINS,
-        .config_name = "netdata monitoring",
+        .config_name = "netdata pulse",
         .env_name = "NETDATA_INTERNALS_MONITORING",
-        .global_variable = &global_statistics_enabled,
-        .enabled = 0,
+        .global_variable = &pulse_enabled,
+        .enabled = 1,
         .thread = NULL,
         .init_routine = NULL,
-        .start_routine = global_statistics_main
+        .start_routine = pulse_thread_main
     },
     {
-        .name = "STATS_WORKERS",
-        .config_section = CONFIG_SECTION_PLUGINS,
-        .config_name = "netdata monitoring extended",
-        .env_name = "NETDATA_INTERNALS_EXTENDED_MONITORING",
-        .global_variable = &global_statistics_enabled,
-        .enabled = 0, // this is ignored - check main() for "netdata monitoring extended"
+        .name = "PULSE-SQLITE3",
+        .config_section = CONFIG_SECTION_PULSE,
+        .config_name = "extended",
+        .env_name = NULL,
+        .global_variable = &pulse_extended_enabled,
+        .enabled = 0, // the default value - it uses netdata.conf for users to enable it
         .thread = NULL,
         .init_routine = NULL,
-        .start_routine = global_statistics_workers_main
+        .start_routine = pulse_thread_sqlite3_main
     },
     {
-        .name = "STATS_SQLITE3",
-        .config_section = CONFIG_SECTION_PLUGINS,
-        .config_name = "netdata monitoring extended",
-        .env_name = "NETDATA_INTERNALS_EXTENDED_MONITORING",
-        .global_variable = &global_statistics_enabled,
-        .enabled = 0, // this is ignored - check main() for "netdata monitoring extended"
+        .name = "PULSE-WORKERS",
+        .config_section = CONFIG_SECTION_PULSE,
+        .config_name = "extended",
+        .env_name = NULL,
+        .global_variable = &pulse_extended_enabled,
+        .enabled = 0, // the default value - it uses netdata.conf for users to enable it
         .thread = NULL,
         .init_routine = NULL,
-        .start_routine = global_statistics_sqlite3_main
+        .start_routine = pulse_thread_workers_main
     },
+    {
+        .name = "PULSE-MEMORY",
+        .config_section = CONFIG_SECTION_PULSE,
+        .config_name = "extended",
+        .env_name = NULL,
+        .global_variable = &pulse_extended_enabled,
+        .enabled = 0, // the default value - it uses netdata.conf for users to enable it
+        .thread = NULL,
+        .init_routine = NULL,
+        .start_routine = pulse_thread_memory_extended_main},
     {
         .name = "PLUGINSD",
         .config_section = NULL,
@@ -105,8 +113,6 @@ const struct netdata_static_thread static_threads_common[] = {
         .init_routine = NULL,
         .start_routine = statsd_main
     },
-#ifndef OS_WINDOWS
-    // this crashes the debugger under windows
     {
         .name = "EXPORTING",
         .config_section = NULL,
@@ -116,7 +122,6 @@ const struct netdata_static_thread static_threads_common[] = {
         .init_routine = NULL,
         .start_routine = exporting_main
     },
-#endif
     {
         .name = "SNDR[localhost]",
         .config_section = NULL,
@@ -124,8 +129,7 @@ const struct netdata_static_thread static_threads_common[] = {
         .enabled = 0,
         .thread = NULL,
         .init_routine = NULL,
-        .start_routine = rrdpush_sender_thread
-    },
+        .start_routine = stream_sender_start_localhost},
     {
         .name = "WEB[1]",
         .config_section = NULL,
@@ -141,6 +145,7 @@ const struct netdata_static_thread static_threads_common[] = {
         .name = "h2o",
         .config_section = NULL,
         .config_name = NULL,
+        .enable_routine = httpd_is_enabled,
         .enabled = 0,
         .thread = NULL,
         .init_routine = NULL,
@@ -148,7 +153,6 @@ const struct netdata_static_thread static_threads_common[] = {
     },
 #endif
 
-#ifdef ENABLE_ACLK
     {
         .name = "ACLK_MAIN",
         .config_section = NULL,
@@ -158,7 +162,6 @@ const struct netdata_static_thread static_threads_common[] = {
         .init_routine = NULL,
         .start_routine = aclk_main
     },
-#endif
 
     {
         .name = "RRDCONTEXT",
@@ -188,6 +191,16 @@ const struct netdata_static_thread static_threads_common[] = {
         .init_routine = NULL,
         .start_routine = profile_main
     },
+    {
+        .name = "BACKFILL",
+        .config_section = NULL,
+        .config_name = NULL,
+        .enable_routine = netdata_conf_is_parent,
+        .enabled = 0,
+        .thread = NULL,
+        .init_routine = NULL,
+        .start_routine = backfill_thread
+    },
 
     // terminator
     {
