@@ -33,7 +33,7 @@ static struct rrdengine_datafile *datafile_alloc_and_init(struct rrdengine_insta
 
     spinlock_init(&datafile->users.spinlock);
     spinlock_init(&datafile->writers.spinlock);
-    spinlock_init(&datafile->extent_queries.spinlock);
+    rw_spinlock_init(&datafile->extent_epdl.spinlock);
 
     return datafile;
 }
@@ -56,17 +56,19 @@ bool datafile_acquire(struct rrdengine_datafile *df, DATAFILE_ACQUIRE_REASONS re
     return ret;
 }
 
-void datafile_release(struct rrdengine_datafile *df, DATAFILE_ACQUIRE_REASONS reason) {
+void datafile_release_with_trace(struct rrdengine_datafile *df, DATAFILE_ACQUIRE_REASONS reason, const char *func) {
     spinlock_lock(&df->users.spinlock);
     if(!df->users.lockers)
-        fatal("DBENGINE DATAFILE: cannot release a datafile that is not acquired");
+        fatal("DBENGINE DATAFILE: cannot release datafile %u of tier %u - it is not acquired, called from %s() with reason %u",
+              df->fileno, df->tier, func, reason);
 
     df->users.lockers--;
     df->users.lockers_by_reason[reason]--;
     spinlock_unlock(&df->users.spinlock);
 }
 
-bool datafile_acquire_for_deletion(struct rrdengine_datafile *df) {
+bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdown)
+{
     bool can_be_deleted = false;
 
     spinlock_lock(&df->users.spinlock);
@@ -107,7 +109,7 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df) {
 
                 if(!df->users.time_to_evict) {
                     // first time we did the above
-                    df->users.time_to_evict = now_s + 120;
+                    df->users.time_to_evict = now_s + (is_shutdown ? DATAFILE_DELETE_TIMEOUT_SHORT : DATAFILE_DELETE_TIMEOUT_LONG);
                     internal_error(true, "DBENGINE: datafile %u of tier %d is not used by any open cache pages, "
                                          "but it has %u lockers (oc:%u, pd:%u), "
                                          "%zu clean and %zu hot open cache pages "
@@ -245,12 +247,12 @@ int create_data_file(struct rrdengine_datafile *datafile)
     uv_fs_t req;
     uv_file file;
     int ret, fd;
-    struct rrdeng_df_sb *superblock;
+    struct rrdeng_df_sb *superblock = NULL;
     uv_buf_t iov;
     char path[RRDENG_PATH_MAX];
 
     generate_datafilepath(datafile, path, sizeof(path));
-    fd = open_file_for_io(path, O_CREAT | O_RDWR | O_TRUNC, &file, use_direct_io);
+    fd = open_file_for_io(path, O_CREAT | O_RDWR | O_TRUNC, &file, dbengine_use_direct_io);
     if (fd < 0) {
         ctx_fs_error(ctx);
         return fd;
@@ -291,7 +293,7 @@ int create_data_file(struct rrdengine_datafile *datafile)
 static int check_data_file_superblock(uv_file file)
 {
     int ret;
-    struct rrdeng_df_sb *superblock;
+    struct rrdeng_df_sb *superblock = NULL;
     uv_buf_t iov;
     uv_fs_t req;
 
@@ -333,7 +335,7 @@ static int load_data_file(struct rrdengine_datafile *datafile)
     char path[RRDENG_PATH_MAX];
 
     generate_datafilepath(datafile, path, sizeof(path));
-    fd = open_file_for_io(path, O_RDWR, &file, use_direct_io);
+    fd = open_file_for_io(path, O_RDWR, &file, dbengine_use_direct_io);
     if (fd < 0) {
         ctx_fs_error(ctx);
         return fd;
@@ -526,7 +528,6 @@ int init_data_files(struct rrdengine_instance *ctx)
 {
     int ret;
 
-    fatal_assert(0 == uv_rwlock_init(&ctx->datafiles.rwlock));
     ret = scan_data_files(ctx);
     if (ret < 0) {
         netdata_log_error("DBENGINE: failed to scan path \"%s\".", ctx->config.dbfiles_path);
@@ -544,7 +545,7 @@ int init_data_files(struct rrdengine_instance *ctx)
         if (ctx->loading.create_new_datafile_pair)
             create_new_datafile_pair(ctx, false);
 
-        while(rrdeng_ctx_exceeded_disk_quota(ctx))
+        while(rrdeng_ctx_tier_cap_exceeded(ctx))
             datafile_delete(ctx, ctx->datafiles.first, false, false);
     }
 
@@ -573,8 +574,8 @@ void finalize_data_files(struct rrdengine_instance *ctx)
         struct rrdengine_journalfile *journalfile = datafile->journalfile;
 
         logged = false;
-        size_t iterations = 100;
-        while(!datafile_acquire_for_deletion(datafile) && datafile != ctx->datafiles.first->prev && --iterations > 0) {
+        size_t iterations = 10;
+        while(!datafile_acquire_for_deletion(datafile, true) && datafile != ctx->datafiles.first->prev && --iterations > 0) {
             if(!logged) {
                 netdata_log_info("Waiting to acquire data file %u of tier %d to close it...", datafile->fileno, ctx->config.tier);
                 logged = true;
