@@ -196,7 +196,7 @@ static struct journal_v2_header *journalfile_v2_mounted_data_get(struct rrdengin
     spinlock_lock(&journalfile->mmap.spinlock);
 
     if(!journalfile->mmap.data) {
-        journalfile->mmap.data = mmap(NULL, journalfile->mmap.size, PROT_READ, MAP_SHARED, journalfile->mmap.fd, 0);
+        journalfile->mmap.data = nd_mmap(NULL, journalfile->mmap.size, PROT_READ, MAP_SHARED, journalfile->mmap.fd, 0);
         if (journalfile->mmap.data == MAP_FAILED) {
             internal_fatal(true, "DBENGINE: failed to re-mmap() journal file v2");
             close(journalfile->mmap.fd);
@@ -268,7 +268,7 @@ static bool journalfile_v2_mounted_data_unmount(struct rrdengine_journalfile *jo
 
     if(!journalfile->v2.refcount) {
         if(journalfile->mmap.data) {
-            if (munmap(journalfile->mmap.data, journalfile->mmap.size)) {
+            if (nd_munmap(journalfile->mmap.data, journalfile->mmap.size)) {
                 char path[RRDENG_PATH_MAX];
                 journalfile_v2_generate_path(journalfile->datafile, path, sizeof(path));
                 netdata_log_error("DBENGINE: failed to unmap index file '%s'", path);
@@ -296,7 +296,7 @@ static bool journalfile_v2_mounted_data_unmount(struct rrdengine_journalfile *jo
 void journalfile_v2_data_unmount_cleanup(time_t now_s) {
     // DO NOT WAIT ON ANY LOCK!!!
 
-    for(size_t tier = 0; tier < (size_t)storage_tiers ;tier++) {
+    for(size_t tier = 0; tier < (size_t)nd_profile.storage_tiers;tier++) {
         struct rrdengine_instance *ctx = multidb_ctx[tier];
         if(!ctx) continue;
 
@@ -572,12 +572,12 @@ int journalfile_create(struct rrdengine_journalfile *journalfile, struct rrdengi
     uv_fs_t req;
     uv_file file;
     int ret, fd;
-    struct rrdeng_jf_sb *superblock;
+    struct rrdeng_jf_sb *superblock = NULL;
     uv_buf_t iov;
     char path[RRDENG_PATH_MAX];
 
     journalfile_v1_generate_path(datafile, path, sizeof(path));
-    fd = open_file_for_io(path, O_CREAT | O_RDWR | O_TRUNC, &file, use_direct_io);
+    fd = open_file_for_io(path, O_CREAT | O_RDWR | O_TRUNC, &file, dbengine_use_direct_io);
     if (fd < 0) {
         ctx_fs_error(ctx);
         return fd;
@@ -618,7 +618,7 @@ int journalfile_create(struct rrdengine_journalfile *journalfile, struct rrdengi
 static int journalfile_check_superblock(uv_file file)
 {
     int ret;
-    struct rrdeng_jf_sb *superblock;
+    struct rrdeng_jf_sb *superblock = NULL;
     uv_buf_t iov;
     uv_fs_t req;
 
@@ -654,7 +654,7 @@ static int journalfile_check_superblock(uv_file file)
 
 static void journalfile_restore_extent_metadata(struct rrdengine_instance *ctx, struct rrdengine_journalfile *journalfile, void *buf, unsigned max_size)
 {
-    static BITMAP256 page_error_map = BITMAP256_INITIALIZER;
+    static bitmap64_t page_error_map = BITMAP64_INITIALIZER;
     unsigned i, count, payload_length, descr_size;
     struct rrdeng_jf_store_data *jf_metric_data;
 
@@ -668,20 +668,21 @@ static void journalfile_restore_extent_metadata(struct rrdengine_instance *ctx,
     }
 
     time_t now_s = max_acceptable_collected_time();
+    time_t extent_first_time_s = journalfile->v2.first_time_s ? journalfile->v2.first_time_s : LONG_MAX;
     for (i = 0; i < count ; ++i) {
-        uuid_t *temp_id;
+        nd_uuid_t *temp_id;
         uint8_t page_type = jf_metric_data->descr[i].type;
 
         if (page_type > RRDENG_PAGE_TYPE_MAX) {
-            if (!bitmap256_get_bit(&page_error_map, page_type)) {
+            if (!bitmap64_get(&page_error_map, page_type)) {
                 netdata_log_error("DBENGINE: unknown page type %d encountered.", page_type);
-                bitmap256_set_bit(&page_error_map, page_type, 1);
+                bitmap64_set(&page_error_map, page_type);
             }
             continue;
         }
 
-        temp_id = (uuid_t *)jf_metric_data->descr[i].uuid;
-        METRIC *metric = mrg_metric_get_and_acquire(main_mrg, temp_id, (Word_t) ctx);
+        temp_id = (nd_uuid_t *)jf_metric_data->descr[i].uuid;
+        METRIC *metric = mrg_metric_get_and_acquire_by_uuid(main_mrg, temp_id, (Word_t)ctx);
 
         struct rrdeng_extent_page_descr *descr = &jf_metric_data->descr[i];
         VALIDATED_PAGE_DESCRIPTOR vd = validate_extent_page_descr(
@@ -727,8 +728,18 @@ static void journalfile_restore_extent_metadata(struct rrdengine_instance *ctx,
                 journalfile->datafile,
                 jf_metric_data->extent_offset, jf_metric_data->extent_size, jf_metric_data->descr[i].page_length);
 
+        extent_first_time_s = MIN(extent_first_time_s, vd.start_time_s);
+
         mrg_metric_release(main_mrg, metric);
     }
+
+    journalfile->v2.first_time_s = extent_first_time_s;
+
+    time_t old = __atomic_load_n(&ctx->atomic.first_time_s, __ATOMIC_RELAXED);;
+    do {
+        if(old <= extent_first_time_s)
+            break;
+    } while(!__atomic_compare_exchange_n(&ctx->atomic.first_time_s, &old, extent_first_time_s, false, __ATOMIC_RELAXED, __ATOMIC_RELAXED));
 }
 
 /*
@@ -799,7 +810,7 @@ static uint64_t journalfile_iterate_transactions(struct rrdengine_instance *ctx,
     int ret;
     uint64_t pos, pos_i, max_id, id;
     unsigned size_bytes;
-    void *buf;
+    void *buf = NULL;
     uv_buf_t iov;
     uv_fs_t req;
 
@@ -1076,7 +1087,7 @@ int journalfile_v2_load(struct rrdengine_instance *ctx, struct rrdengine_journal
     }
 
     usec_t mmap_start_ut = now_monotonic_usec();
-    uint8_t *data_start = mmap(NULL, journal_v2_file_size, PROT_READ, MAP_SHARED, fd, 0);
+    uint8_t *data_start = nd_mmap(NULL, journal_v2_file_size, PROT_READ, MAP_SHARED, fd, 0);
     if (data_start == MAP_FAILED) {
         close(fd);
         return 1;
@@ -1094,7 +1105,7 @@ int journalfile_v2_load(struct rrdengine_instance *ctx, struct rrdengine_journal
         else
             error_report("File %s is invalid and it will be rebuilt", path_v2);
 
-        if (unlikely(munmap(data_start, journal_v2_file_size)))
+        if (unlikely(nd_munmap(data_start, journal_v2_file_size)))
             netdata_log_error("DBENGINE: failed to unmap '%s'", path_v2);
 
         close(fd);
@@ -1105,7 +1116,7 @@ int journalfile_v2_load(struct rrdengine_instance *ctx, struct rrdengine_journal
     uint32_t entries = j2_header->metric_count;
 
     if (unlikely(!entries)) {
-        if (unlikely(munmap(data_start, journal_v2_file_size)))
+        if (unlikely(nd_munmap(data_start, journal_v2_file_size)))
             netdata_log_error("DBENGINE: failed to unmap '%s'", path_v2);
 
         close(fd);
@@ -1144,7 +1155,7 @@ static int journalfile_metric_compare (const void *item1, const void *item2)
     const struct jv2_metrics_info *metric1 = ((struct journal_metric_list_to_sort *) item1)->metric_info;
     const struct jv2_metrics_info *metric2 = ((struct journal_metric_list_to_sort *) item2)->metric_info;
 
-    return memcmp(metric1->uuid, metric2->uuid, sizeof(uuid_t));
+    return memcmp(metric1->uuid, metric2->uuid, sizeof(nd_uuid_t));
 }
 
 
@@ -1327,7 +1338,7 @@ void journalfile_migrate_to_v2_callback(Word_t section, unsigned datafile_fileno
     total_file_size  += sizeof(struct journal_v2_block_trailer);
 
     int fd_v2;
-    uint8_t *data_start = netdata_mmap(path, total_file_size, MAP_SHARED, 0, false, &fd_v2);
+    uint8_t *data_start = nd_mmap_advanced(path, total_file_size, MAP_SHARED, 0, false, true, &fd_v2);
     uint8_t *data = data_start;
 
     memset(data_start, 0, extent_offset);
@@ -1387,6 +1398,10 @@ void journalfile_migrate_to_v2_callback(Word_t section, unsigned datafile_fileno
         max_time_s = MAX(max_time_s, metric_info->last_time_s);
     }
 
+    // Check if not properly set in the loop above to prevent overflow
+    if (min_time_s == LONG_MAX)
+        min_time_s = 0;
+
     // Store in the header
     j2_header.start_time_ut = min_time_s * USEC_PER_SEC;
     j2_header.end_time_ut = max_time_s * USEC_PER_SEC;
@@ -1479,7 +1494,7 @@ void journalfile_migrate_to_v2_callback(Word_t section, unsigned datafile_fileno
         resize_file_to = sizeof(j2_header);
     }
 
-    netdata_munmap(data_start, total_file_size);
+    nd_munmap(data_start, total_file_size);
     freez(uuid_list);
 
     if (likely(resize_file_to == total_file_size))
@@ -1511,7 +1526,7 @@ int journalfile_load(struct rrdengine_instance *ctx, struct rrdengine_journalfil
 
     journalfile_v1_generate_path(datafile, path, sizeof(path));
 
-    fd = open_file_for_io(path, O_RDWR, &file, use_direct_io);
+    fd = open_file_for_io(path, O_RDWR, &file, dbengine_use_direct_io);
     if (fd < 0) {
         ctx_fs_error(ctx);
 
