@@ -4,20 +4,22 @@
 #include "libnetdata/locks/locks.h"
 #include "rrddiskprotocol.h"
 
-typedef int32_t REFCOUNT;
-#define REFCOUNT_DELETING (-100)
-
 struct metric {
-    uuid_t uuid;                    // never changes
     Word_t section;                 // never changes
+    UUIDMAP_ID uuid;                 // never changes
+
+    REFCOUNT refcount;
+    uint8_t partition;
+
+    uint32_t latest_update_every_s; // the latest data collection frequency
 
     time_t first_time_s;            // the timestamp of the oldest point in the database
     time_t latest_time_s_clean;     // the timestamp of the newest point in the database
     time_t latest_time_s_hot;       // the timestamp of the latest point that has been collected (not yet stored)
-    uint32_t latest_update_every_s; // the latest data collection frequency
+
+#ifdef NETDATA_INTERNAL_CHECKS
     pid_t writer;
-    uint8_t partition;
-    REFCOUNT refcount;
+#endif
 
     // THIS IS allocated with malloc()
     // YOU HAVE TO INITIALIZE IT YOURSELF !
@@ -45,16 +47,14 @@ struct metric {
 static struct aral_statistics mrg_aral_statistics;
 
 struct mrg {
-    size_t partitions;
-
     struct mrg_partition {
         ARAL *aral;                 // not protected by our spinlock - it has its own
 
         RW_SPINLOCK rw_spinlock;
-        Pvoid_t uuid_judy;          // JudyHS: each UUID has a JudyL of sections (tiers)
+        Pvoid_t uuid_judy;          // JudyL: each UUID has a JudyL of sections (tiers)
 
         struct mrg_statistics stats;
-    } index[];
+    } index[UUIDMAP_PARTITIONS];
 };
 
 static inline void MRG_STATS_DUPLICATE_ADD(MRG *mrg, size_t partition) {
@@ -90,28 +90,8 @@ static inline void MRG_STATS_DELETE_MISS(MRG *mrg, size_t partition) {
 #define mrg_index_write_lock(mrg, partition) rw_spinlock_write_lock(&(mrg)->index[partition].rw_spinlock)
 #define mrg_index_write_unlock(mrg, partition) rw_spinlock_write_unlock(&(mrg)->index[partition].rw_spinlock)
 
-static inline void mrg_stats_size_judyl_change(MRG *mrg, size_t mem_before_judyl, size_t mem_after_judyl, size_t partition) {
-    if(mem_after_judyl > mem_before_judyl)
-        __atomic_add_fetch(&mrg->index[partition].stats.size, mem_after_judyl - mem_before_judyl, __ATOMIC_RELAXED);
-    else if(mem_after_judyl < mem_before_judyl)
-        __atomic_sub_fetch(&mrg->index[partition].stats.size, mem_before_judyl - mem_after_judyl, __ATOMIC_RELAXED);
-}
-
-static inline void mrg_stats_size_judyhs_added_uuid(MRG *mrg, size_t partition) {
-    __atomic_add_fetch(&mrg->index[partition].stats.size, JUDYHS_INDEX_SIZE_ESTIMATE(sizeof(uuid_t)), __ATOMIC_RELAXED);
-}
-
-static inline void mrg_stats_size_judyhs_removed_uuid(MRG *mrg, size_t partition) {
-    __atomic_sub_fetch(&mrg->index[partition].stats.size, JUDYHS_INDEX_SIZE_ESTIMATE(sizeof(uuid_t)), __ATOMIC_RELAXED);
-}
-
-static inline size_t uuid_partition(MRG *mrg __maybe_unused, uuid_t *uuid) {
-    uint8_t *u = (uint8_t *)uuid;
-
-    size_t n;
-    memcpy(&n, &u[UUID_SZ - sizeof(size_t)], sizeof(size_t));
-
-    return n % mrg->partitions;
+static inline void mrg_stats_judy_mem(MRG *mrg, size_t partition, int64_t judy_mem) {
+    __atomic_add_fetch(&mrg->index[partition].stats.size, judy_mem, __ATOMIC_RELAXED);
 }
 
 static inline time_t mrg_metric_get_first_time_s_smart(MRG *mrg __maybe_unused, METRIC *metric) {
@@ -131,130 +111,127 @@ static inline time_t mrg_metric_get_first_time_s_smart(MRG *mrg __maybe_unused,
     return first_time_s;
 }
 
-static void metric_log(MRG *mrg __maybe_unused, METRIC *metric, const char *msg) {
+static inline void metric_log(MRG *mrg __maybe_unused, METRIC *metric, const char *msg) {
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)metric->section;
 
-    char uuid[UUID_STR_LEN];
-    uuid_unparse_lower(metric->uuid, uuid);
+    nd_uuid_t uuid;
+    uuidmap_uuid(metric->uuid, uuid);
+    char uuid_txt[UUID_STR_LEN];
+    uuid_unparse_lower(uuid, uuid_txt);
     nd_log(NDLS_DAEMON, NDLP_ERR,
            "METRIC: %s on %s at tier %d, refcount %d, partition %u, "
-           "retention [%ld - %ld (hot), %ld (clean)], update every %"PRIu32", "
-           "writer pid %d "
-           "--- PLEASE OPEN A GITHUB ISSUE TO REPORT THIS LOG LINE TO NETDATA --- ",
+           "retention [%ld - %ld (hot), %ld (clean)], update every %"PRIu32
+#ifdef NETDATA_INTERNAL_CHECKS
+           ", writer pid %d "
+#endif
+           " --- PLEASE OPEN A GITHUB ISSUE TO REPORT THIS LOG LINE TO NETDATA --- ",
            msg,
-           uuid,
+           uuid_txt,
            ctx->config.tier,
            metric->refcount,
            metric->partition,
            metric->first_time_s,
            metric->latest_time_s_hot,
            metric->latest_time_s_clean,
-           metric->latest_update_every_s,
-           (int)metric->writer
+           metric->latest_update_every_s
+#ifdef NETDATA_INTERNAL_CHECKS
+           , (int)metric->writer
+#endif
     );
 }
 
 static inline bool acquired_metric_has_retention(MRG *mrg, METRIC *metric) {
     time_t first, last;
     mrg_metric_get_retention(mrg, metric, &first, &last, NULL);
-    return (!first || !last || first > last);
+    bool rc = (first != 0 && last != 0 && first <= last);
+
+    if(!rc && __atomic_load_n(&mrg->index[metric->partition].stats.writers, __ATOMIC_RELAXED) > 0)
+        rc = true;
+
+    return rc;
 }
 
 static inline void acquired_for_deletion_metric_delete(MRG *mrg, METRIC *metric) {
-    size_t partition = metric->partition;
+    JudyAllocThreadPulseReset();
 
-    size_t mem_before_judyl, mem_after_judyl;
+    size_t partition = metric->partition;
 
     mrg_index_write_lock(mrg, partition);
 
-    Pvoid_t *sections_judy_pptr = JudyHSGet(mrg->index[partition].uuid_judy, &metric->uuid, sizeof(uuid_t));
+    Pvoid_t *sections_judy_pptr = JudyLGet(mrg->index[partition].uuid_judy, metric->uuid, PJE0);
+    if(unlikely(sections_judy_pptr == PJERR))
+        fatal("METRIC: corrupted JudyL");
+
     if(unlikely(!sections_judy_pptr || !*sections_judy_pptr)) {
         MRG_STATS_DELETE_MISS(mrg, partition);
         mrg_index_write_unlock(mrg, partition);
         return;
     }
 
-    mem_before_judyl = JudyLMemUsed(*sections_judy_pptr);
     int rc = JudyLDel(sections_judy_pptr, metric->section, PJE0);
-    mem_after_judyl = JudyLMemUsed(*sections_judy_pptr);
-    mrg_stats_size_judyl_change(mrg, mem_before_judyl, mem_after_judyl, partition);
-
     if(unlikely(!rc)) {
         MRG_STATS_DELETE_MISS(mrg, partition);
         mrg_index_write_unlock(mrg, partition);
+        mrg_stats_judy_mem(mrg, partition, JudyAllocThreadPulseGetAndReset());
         return;
     }
 
     if(!*sections_judy_pptr) {
-        rc = JudyHSDel(&mrg->index[partition].uuid_judy, &metric->uuid, sizeof(uuid_t), PJE0);
+        rc = JudyLDel(&mrg->index[partition].uuid_judy, metric->uuid, PJE0);
+
         if(unlikely(!rc))
-            fatal("DBENGINE METRIC: cannot delete UUID from JudyHS");
-        mrg_stats_size_judyhs_removed_uuid(mrg, partition);
+            fatal("DBENGINE METRIC: cannot delete UUID from JudyL");
     }
 
     MRG_STATS_DELETED_METRIC(mrg, partition);
 
     mrg_index_write_unlock(mrg, partition);
-
+    uuidmap_free(metric->uuid);
     aral_freez(mrg->index[partition].aral, metric);
+    mrg_stats_judy_mem(mrg, partition, JudyAllocThreadPulseGetAndReset());
 }
 
 static inline bool metric_acquire(MRG *mrg, METRIC *metric) {
-    REFCOUNT expected, desired;
-
-    expected = __atomic_load_n(&metric->refcount, __ATOMIC_RELAXED);
-
-    do {
-        if(unlikely(expected < 0))
-            return false;
-
-        desired = expected + 1;
-
-    } while(!__atomic_compare_exchange_n(&metric->refcount, &expected, desired, false, __ATOMIC_ACQUIRE, __ATOMIC_RELAXED));
+    REFCOUNT rc = refcount_acquire_advanced(&metric->refcount);
+    if(!REFCOUNT_ACQUIRED(rc))
+        return false;
 
     size_t partition = metric->partition;
 
-    if(desired == 1)
-        __atomic_add_fetch(&mrg->index[partition].stats.entries_referenced, 1, __ATOMIC_RELAXED);
+    if(rc == 1)
+        __atomic_add_fetch(&mrg->index[partition].stats.entries_acquired, 1, __ATOMIC_RELAXED);
 
     __atomic_add_fetch(&mrg->index[partition].stats.current_references, 1, __ATOMIC_RELAXED);
 
     return true;
 }
 
-static inline bool metric_release(MRG *mrg, METRIC *metric, bool delete_if_last_without_retention) {
+static inline bool metric_release(MRG *mrg, METRIC *metric) {
     size_t partition = metric->partition;
-    REFCOUNT expected, desired;
 
-    expected = __atomic_load_n(&metric->refcount, __ATOMIC_RELAXED);
+    REFCOUNT refcount = refcount_release(&metric->refcount);
 
-    do {
-        if(expected <= 0) {
-            metric_log(mrg, metric, "refcount is zero or negative during release");
-            fatal("METRIC: refcount is %d (zero or negative) during release", expected);
-        }
+    if(!refcount && !acquired_metric_has_retention(mrg, metric) && refcount_acquire_for_deletion(&metric->refcount))
+        refcount = REFCOUNT_DELETED;
 
-        if(expected == 1 && delete_if_last_without_retention && !acquired_metric_has_retention(mrg, metric))
-            desired = REFCOUNT_DELETING;
-        else
-            desired = expected - 1;
+    if(refcount == 0 || refcount == REFCOUNT_DELETED) {
+        __atomic_sub_fetch(&mrg->index[partition].stats.entries_acquired, 1, __ATOMIC_RELAXED);
 
-    } while(!__atomic_compare_exchange_n(&metric->refcount, &expected, desired, false, __ATOMIC_RELEASE, __ATOMIC_RELAXED));
-
-    if(desired == 0 || desired == REFCOUNT_DELETING) {
-        __atomic_sub_fetch(&mrg->index[partition].stats.entries_referenced, 1, __ATOMIC_RELAXED);
-
-        if(desired == REFCOUNT_DELETING)
+        if(refcount == REFCOUNT_DELETED)
             acquired_for_deletion_metric_delete(mrg, metric);
     }
 
     __atomic_sub_fetch(&mrg->index[partition].stats.current_references, 1, __ATOMIC_RELAXED);
 
-    return desired == REFCOUNT_DELETING;
+    return refcount == REFCOUNT_DELETED;
 }
 
 static inline METRIC *metric_add_and_acquire(MRG *mrg, MRG_ENTRY *entry, bool *ret) {
-    size_t partition = uuid_partition(mrg, entry->uuid);
+    JudyAllocThreadPulseReset();
+
+    UUIDMAP_ID id = uuidmap_create(*entry->uuid);
+
+    size_t partition = uuid_to_uuidmap_partition(*entry->uuid);
 
     METRIC *allocation = aral_mallocz(mrg->index[partition].aral);
     Pvoid_t *PValue;
@@ -262,20 +239,11 @@ static inline METRIC *metric_add_and_acquire(MRG *mrg, MRG_ENTRY *entry, bool *r
     while(1) {
         mrg_index_write_lock(mrg, partition);
 
-        size_t mem_before_judyl, mem_after_judyl;
-
-        Pvoid_t *sections_judy_pptr = JudyHSIns(&mrg->index[partition].uuid_judy, entry->uuid, sizeof(uuid_t), PJE0);
+        Pvoid_t *sections_judy_pptr = JudyLIns(&mrg->index[partition].uuid_judy, id, PJE0);
         if (unlikely(!sections_judy_pptr || sections_judy_pptr == PJERR))
-            fatal("DBENGINE METRIC: corrupted UUIDs JudyHS array");
+            fatal("DBENGINE METRIC: corrupted UUIDs JudyL array");
 
-        if (unlikely(!*sections_judy_pptr))
-            mrg_stats_size_judyhs_added_uuid(mrg, partition);
-
-        mem_before_judyl = JudyLMemUsed(*sections_judy_pptr);
         PValue = JudyLIns(sections_judy_pptr, entry->section, PJE0);
-        mem_after_judyl = JudyLMemUsed(*sections_judy_pptr);
-        mrg_stats_size_judyl_change(mrg, mem_before_judyl, mem_after_judyl, partition);
-
         if (unlikely(!PValue || PValue == PJERR))
             fatal("DBENGINE METRIC: corrupted section JudyL array");
 
@@ -293,8 +261,10 @@ static inline METRIC *metric_add_and_acquire(MRG *mrg, MRG_ENTRY *entry, bool *r
             if (ret)
                 *ret = false;
 
+            uuidmap_free(id);
             aral_freez(mrg->index[partition].aral, allocation);
 
+            mrg_stats_judy_mem(mrg, partition, JudyAllocThreadPulseGetAndReset());
             return metric;
         }
 
@@ -302,17 +272,22 @@ static inline METRIC *metric_add_and_acquire(MRG *mrg, MRG_ENTRY *entry, bool *r
     }
 
     METRIC *metric = allocation;
-    uuid_copy(metric->uuid, *entry->uuid);
+    metric->uuid = id;
     metric->section = entry->section;
     metric->first_time_s = MAX(0, entry->first_time_s);
     metric->latest_time_s_clean = MAX(0, entry->last_time_s);
     metric->latest_time_s_hot = 0;
     metric->latest_update_every_s = entry->latest_update_every_s;
+#ifdef NETDATA_INTERNAL_CHECKS
     metric->writer = 0;
+#endif
     metric->refcount = 1;
     metric->partition = partition;
     *PValue = metric;
 
+    __atomic_add_fetch(&mrg->index[partition].stats.entries_acquired, 1, __ATOMIC_RELAXED);
+    __atomic_add_fetch(&mrg->index[partition].stats.current_references, 1, __ATOMIC_RELAXED);
+
     MRG_STATS_ADDED_METRIC(mrg, partition);
 
     mrg_index_write_unlock(mrg, partition);
@@ -320,16 +295,17 @@ static inline METRIC *metric_add_and_acquire(MRG *mrg, MRG_ENTRY *entry, bool *r
     if(ret)
         *ret = true;
 
+    mrg_stats_judy_mem(mrg, partition, JudyAllocThreadPulseGetAndReset());
     return metric;
 }
 
-static inline METRIC *metric_get_and_acquire(MRG *mrg, uuid_t *uuid, Word_t section) {
-    size_t partition = uuid_partition(mrg, uuid);
+static inline METRIC *metric_get_and_acquire_by_id(MRG *mrg, UUIDMAP_ID id, Word_t section) {
+    size_t partition = uuidmap_id_to_partition(id);
 
     while(1) {
         mrg_index_read_lock(mrg, partition);
 
-        Pvoid_t *sections_judy_pptr = JudyHSGet(mrg->index[partition].uuid_judy, uuid, sizeof(uuid_t));
+        Pvoid_t *sections_judy_pptr = JudyLGet(mrg->index[partition].uuid_judy, id, PJE0);
         if (unlikely(!sections_judy_pptr)) {
             mrg_index_read_unlock(mrg, partition);
             MRG_STATS_SEARCH_MISS(mrg, partition);
@@ -360,31 +336,25 @@ static inline METRIC *metric_get_and_acquire(MRG *mrg, uuid_t *uuid, Word_t sect
 // ----------------------------------------------------------------------------
 // public API
 
-inline MRG *mrg_create(ssize_t partitions) {
-    if(partitions < 1)
-        partitions = get_netdata_cpus();
+inline MRG *mrg_create(void) {
+    MRG *mrg = callocz(1, sizeof(MRG));
 
-    MRG *mrg = callocz(1, sizeof(MRG) + sizeof(struct mrg_partition) * partitions);
-    mrg->partitions = partitions;
-
-    for(size_t i = 0; i < mrg->partitions ; i++) {
+    for(size_t i = 0; i < _countof(mrg->index) ; i++) {
         rw_spinlock_init(&mrg->index[i].rw_spinlock);
 
         char buf[ARAL_MAX_NAME + 1];
         snprintfz(buf, ARAL_MAX_NAME, "mrg[%zu]", i);
 
-        mrg->index[i].aral = aral_create(buf, sizeof(METRIC), 0, 16384, &mrg_aral_statistics, NULL, NULL, false, false);
+        mrg->index[i].aral = aral_create(buf, sizeof(METRIC), 0, 16384, &mrg_aral_statistics, NULL, NULL,
+                                         false, false, true);
     }
+    pulse_aral_register_statistics(&mrg_aral_statistics, "mrg");
 
     return mrg;
 }
 
-inline size_t mrg_aral_structures(void) {
-    return aral_structures_from_stats(&mrg_aral_statistics);
-}
-
-inline size_t mrg_aral_overhead(void) {
-    return aral_overhead_from_stats(&mrg_aral_statistics);
+struct aral_statistics *mrg_aral_stats(void) {
+    return &mrg_aral_statistics;
 }
 
 inline void mrg_destroy(MRG *mrg __maybe_unused) {
@@ -394,7 +364,7 @@ inline void mrg_destroy(MRG *mrg __maybe_unused) {
     // to delete entries, the caller needs to keep pointers to them
     // and delete them one by one
 
-    ;
+    pulse_aral_unregister(mrg->index[0].aral);
 }
 
 inline METRIC *mrg_metric_add_and_acquire(MRG *mrg, MRG_ENTRY entry, bool *ret) {
@@ -404,12 +374,19 @@ inline METRIC *mrg_metric_add_and_acquire(MRG *mrg, MRG_ENTRY entry, bool *ret)
     return metric_add_and_acquire(mrg, &entry, ret);
 }
 
-inline METRIC *mrg_metric_get_and_acquire(MRG *mrg, uuid_t *uuid, Word_t section) {
-    return metric_get_and_acquire(mrg, uuid, section);
+inline METRIC *mrg_metric_get_and_acquire_by_uuid(MRG *mrg, nd_uuid_t *uuid, Word_t section) {
+    UUIDMAP_ID id = uuidmap_create(*uuid);
+    METRIC *metric = metric_get_and_acquire_by_id(mrg, id, section);
+    uuidmap_free(id);
+    return metric;
+}
+
+inline METRIC *mrg_metric_get_and_acquire_by_id(MRG *mrg, UUIDMAP_ID id, Word_t section) {
+    return metric_get_and_acquire_by_id(mrg, id, section);
 }
 
 inline bool mrg_metric_release_and_delete(MRG *mrg, METRIC *metric) {
-    return metric_release(mrg, metric, true);
+    return metric_release(mrg, metric);
 }
 
 inline METRIC *mrg_metric_dup(MRG *mrg, METRIC *metric) {
@@ -418,15 +395,19 @@ inline METRIC *mrg_metric_dup(MRG *mrg, METRIC *metric) {
 }
 
 inline void mrg_metric_release(MRG *mrg, METRIC *metric) {
-    metric_release(mrg, metric, false);
+    metric_release(mrg, metric);
 }
 
 inline Word_t mrg_metric_id(MRG *mrg __maybe_unused, METRIC *metric) {
     return (Word_t)metric;
 }
 
-inline uuid_t *mrg_metric_uuid(MRG *mrg __maybe_unused, METRIC *metric) {
-    return &metric->uuid;
+inline nd_uuid_t *mrg_metric_uuid(MRG *mrg __maybe_unused, METRIC *metric) {
+    return uuidmap_uuid_ptr(metric->uuid);
+}
+
+inline UUIDMAP_ID mrg_metric_uuidmap_id_dup(MRG *mrg __maybe_unused, METRIC *metric) {
+    return uuidmap_dup(metric->uuid);
 }
 
 inline Word_t mrg_metric_section(MRG *mrg __maybe_unused, METRIC *metric) {
@@ -436,6 +417,9 @@ inline Word_t mrg_metric_section(MRG *mrg __maybe_unused, METRIC *metric) {
 inline bool mrg_metric_set_first_time_s(MRG *mrg __maybe_unused, METRIC *metric, time_t first_time_s) {
     internal_fatal(first_time_s < 0, "DBENGINE METRIC: timestamp is negative");
 
+    if(first_time_s == LONG_MAX)
+        first_time_s = 0;
+
     if(unlikely(first_time_s < 0))
         return false;
 
@@ -452,8 +436,8 @@ inline void mrg_metric_expand_retention(MRG *mrg __maybe_unused, METRIC *metric,
     internal_fatal(last_time_s > max_acceptable_collected_time(),
                    "DBENGINE METRIC: metric last time is in the future");
 
-    if(first_time_s > 0)
-        set_metric_field_with_condition(metric->first_time_s, first_time_s, _current <= 0 || _wanted < _current);
+    if(first_time_s > 0 && first_time_s != LONG_MAX)
+        set_metric_field_with_condition(metric->first_time_s, first_time_s, _current <= 0 || (_wanted != 0 && _wanted != LONG_MAX && _wanted < _current));
 
     if(last_time_s > 0) {
         if(set_metric_field_with_condition(metric->latest_time_s_clean, last_time_s, _current <= 0 || _wanted > _current) &&
@@ -468,7 +452,7 @@ inline void mrg_metric_expand_retention(MRG *mrg __maybe_unused, METRIC *metric,
 
 inline bool mrg_metric_set_first_time_s_if_bigger(MRG *mrg __maybe_unused, METRIC *metric, time_t first_time_s) {
     internal_fatal(first_time_s < 0, "DBENGINE METRIC: timestamp is negative");
-    return set_metric_field_with_condition(metric->first_time_s, first_time_s, _wanted > _current);
+    return set_metric_field_with_condition(metric->first_time_s, first_time_s, _wanted != 0 && _wanted != LONG_MAX && _wanted > _current);
 }
 
 inline time_t mrg_metric_get_first_time_s(MRG *mrg __maybe_unused, METRIC *metric) {
@@ -598,9 +582,10 @@ inline uint32_t mrg_metric_get_update_every_s(MRG *mrg __maybe_unused, METRIC *m
     return __atomic_load_n(&metric->latest_update_every_s, __ATOMIC_RELAXED);
 }
 
+#ifdef NETDATA_INTERNAL_CHECKS
 inline bool mrg_metric_set_writer(MRG *mrg, METRIC *metric) {
     pid_t expected = __atomic_load_n(&metric->writer, __ATOMIC_RELAXED);
-    pid_t wanted = gettid();
+    pid_t wanted = gettid_cached();
     bool done = true;
 
     do {
@@ -637,9 +622,10 @@ inline bool mrg_metric_clear_writer(MRG *mrg, METRIC *metric) {
 
     return done;
 }
+#endif
 
 inline void mrg_update_metric_retention_and_granularity_by_uuid(
-        MRG *mrg, Word_t section, uuid_t *uuid,
+        MRG *mrg, Word_t section, nd_uuid_t *uuid,
         time_t first_time_s, time_t last_time_s,
         uint32_t update_every_s, time_t now_s)
 {
@@ -671,7 +657,7 @@ inline void mrg_update_metric_retention_and_granularity_by_uuid(
     }
 
     bool added = false;
-    METRIC *metric = mrg_metric_get_and_acquire(mrg, uuid, section);
+    METRIC *metric = mrg_metric_get_and_acquire_by_uuid(mrg, uuid, section);
     if (!metric) {
         MRG_ENTRY entry = {
                 .uuid = uuid,
@@ -713,9 +699,9 @@ inline void mrg_update_metric_retention_and_granularity_by_uuid(
 inline void mrg_get_statistics(MRG *mrg, struct mrg_statistics *s) {
     memset(s, 0, sizeof(struct mrg_statistics));
 
-    for(size_t i = 0; i < mrg->partitions ;i++) {
+    for(size_t i = 0; i < _countof(mrg->index) ;i++) {
         s->entries += __atomic_load_n(&mrg->index[i].stats.entries, __ATOMIC_RELAXED);
-        s->entries_referenced += __atomic_load_n(&mrg->index[i].stats.entries_referenced, __ATOMIC_RELAXED);
+        s->entries_acquired += __atomic_load_n(&mrg->index[i].stats.entries_acquired, __ATOMIC_RELAXED);
         s->size += __atomic_load_n(&mrg->index[i].stats.size, __ATOMIC_RELAXED);
         s->current_references += __atomic_load_n(&mrg->index[i].stats.current_references, __ATOMIC_RELAXED);
         s->additions += __atomic_load_n(&mrg->index[i].stats.additions, __ATOMIC_RELAXED);
@@ -729,14 +715,14 @@ inline void mrg_get_statistics(MRG *mrg, struct mrg_statistics *s) {
         s->writers_conflicts += __atomic_load_n(&mrg->index[i].stats.writers_conflicts, __ATOMIC_RELAXED);
     }
 
-    s->size += sizeof(MRG) + sizeof(struct mrg_partition) * mrg->partitions;
+    s->size += sizeof(MRG);
 }
 
 // ----------------------------------------------------------------------------
 // unit test
 
 struct mrg_stress_entry {
-    uuid_t uuid;
+    nd_uuid_t uuid;
     time_t after;
     time_t before;
 };
@@ -757,13 +743,13 @@ static void *mrg_stress(void *ptr) {
     ssize_t end = (ssize_t)t->entries;
     ssize_t step = 1;
 
-    if(gettid() % 2) {
+    if(gettid_cached() % 2) {
         start = (ssize_t)t->entries - 1;
         end = -1;
         step = -1;
     }
 
-    while(!__atomic_load_n(&t->stop, __ATOMIC_RELAXED)) {
+    while(!__atomic_load_n(&t->stop, __ATOMIC_RELAXED) && !nd_thread_signaled_to_cancel()) {
         for (ssize_t i = start; i != end; i += step) {
             struct mrg_stress_entry *e = &t->array[i];
 
@@ -786,12 +772,12 @@ static void *mrg_stress(void *ptr) {
 }
 
 int mrg_unittest(void) {
-    MRG *mrg = mrg_create(0);
+    MRG *mrg = mrg_create();
     METRIC *m1_t0, *m2_t0, *m3_t0, *m4_t0;
     METRIC *m1_t1, *m2_t1, *m3_t1, *m4_t1;
     bool ret;
 
-    uuid_t test_uuid;
+    nd_uuid_t test_uuid;
     uuid_generate(test_uuid);
     MRG_ENTRY entry = {
             .uuid = &test_uuid,
@@ -811,7 +797,7 @@ int mrg_unittest(void) {
     if(ret)
         fatal("DBENGINE METRIC: managed to add the same metric twice");
 
-    m3_t0 = mrg_metric_get_and_acquire(mrg, entry.uuid, entry.section);
+    m3_t0 = mrg_metric_get_and_acquire_by_uuid(mrg, entry.uuid, entry.section);
     if(m3_t0 != m1_t0)
         fatal("DBENGINE METRIC: cannot find the metric added");
 
@@ -835,7 +821,7 @@ int mrg_unittest(void) {
     if(ret)
         fatal("DBENGINE METRIC: managed to add the same metric twice in (section 0)");
 
-    m3_t1 = mrg_metric_get_and_acquire(mrg, entry.uuid, entry.section);
+    m3_t1 = mrg_metric_get_and_acquire_by_uuid(mrg, entry.uuid, entry.section);
     if(m3_t1 != m1_t1)
         fatal("DBENGINE METRIC: cannot find the metric added (section %zu)", (size_t)entry.section);
 
@@ -849,7 +835,7 @@ int mrg_unittest(void) {
     if(!mrg_metric_release_and_delete(mrg, m1_t0))
         fatal("DBENGINE METRIC: cannot delete the first metric");
 
-    m4_t1 = mrg_metric_get_and_acquire(mrg, entry.uuid, entry.section);
+    m4_t1 = mrg_metric_get_and_acquire_by_uuid(mrg, entry.uuid, entry.section);
     if(m4_t1 != m1_t1)
         fatal("DBENGINE METRIC: cannot find the metric added (section %zu), after deleting the first one", (size_t)entry.section);
 
@@ -869,7 +855,7 @@ int mrg_unittest(void) {
         fatal("DBENGINE METRIC: invalid entries counter");
 
     size_t entries = 1000000;
-    size_t threads = mrg->partitions / 3 + 1;
+    size_t threads = _countof(mrg->index) / 3 + 1;
     size_t tiers = 3;
     size_t run_for_secs = 5;
     netdata_log_info("preparing stress test of %zu entries...", entries);
@@ -902,23 +888,21 @@ int mrg_unittest(void) {
 
     usec_t started_ut = now_monotonic_usec();
 
-    pthread_t th[threads];
+    ND_THREAD *th[threads];
     for(size_t i = 0; i < threads ; i++) {
         char buf[15 + 1];
         snprintfz(buf, sizeof(buf) - 1, "TH[%zu]", i);
-        netdata_thread_create(&th[i], buf,
-                              NETDATA_THREAD_OPTION_JOINABLE | NETDATA_THREAD_OPTION_DONT_LOG,
-                              mrg_stress, &t);
+        th[i] = nd_thread_create(buf, NETDATA_THREAD_OPTION_JOINABLE | NETDATA_THREAD_OPTION_DONT_LOG, mrg_stress, &t);
     }
 
     sleep_usec(run_for_secs * USEC_PER_SEC);
     __atomic_store_n(&t.stop, true, __ATOMIC_RELAXED);
 
     for(size_t i = 0; i < threads ; i++)
-        netdata_thread_cancel(th[i]);
+        nd_thread_signal_cancel(th[i]);
 
     for(size_t i = 0; i < threads ; i++)
-        netdata_thread_join(th[i], NULL);
+        nd_thread_join(th[i]);
 
     usec_t ended_ut = now_monotonic_usec();
 
