@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-#define NETDATA_RRD_INTERNALS
+
 #include "pdc.h"
 #include "dbengine-compression.h"
 
@@ -53,10 +53,12 @@ void pdc_init(void) {
             "dbengine-pdc",
             sizeof(PDC),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false
+            NULL, NULL, false, false, true
             );
+
+    pulse_aral_register(pdc_globals.pdc.ar, "pdc");
 }
 
 PDC *pdc_get(void) {
@@ -69,8 +71,8 @@ static void pdc_release(PDC *pdc) {
     aral_freez(pdc_globals.pdc.ar, pdc);
 }
 
-size_t pdc_cache_size(void) {
-    return aral_overhead(pdc_globals.pdc.ar) + aral_structures(pdc_globals.pdc.ar);
+struct aral_statistics *pdc_aral_stats(void) {
+    return aral_get_statistics(pdc_globals.pdc.ar);
 }
 
 // ----------------------------------------------------------------------------
@@ -81,10 +83,11 @@ void page_details_init(void) {
             "dbengine-pd",
             sizeof(struct page_details),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false
+            NULL, NULL, false, false, true
     );
+    pulse_aral_register(pdc_globals.pd.ar, "pd");
 }
 
 struct page_details *page_details_get(void) {
@@ -97,8 +100,8 @@ static void page_details_release(struct page_details *pd) {
     aral_freez(pdc_globals.pd.ar, pd);
 }
 
-size_t pd_cache_size(void) {
-    return aral_overhead(pdc_globals.pd.ar) + aral_structures(pdc_globals.pd.ar);
+struct aral_statistics *pd_aral_stats(void) {
+    return aral_get_statistics(pdc_globals.pd.ar);
 }
 
 // ----------------------------------------------------------------------------
@@ -109,10 +112,11 @@ void epdl_init(void) {
             "dbengine-epdl",
             sizeof(EPDL),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false
+            NULL, NULL, false, false, true
     );
+    pulse_aral_register(pdc_globals.epdl.ar, "epdl");
 }
 
 static EPDL *epdl_get(void) {
@@ -125,8 +129,8 @@ static void epdl_release(EPDL *epdl) {
     aral_freez(pdc_globals.epdl.ar, epdl);
 }
 
-size_t epdl_cache_size(void) {
-    return aral_overhead(pdc_globals.epdl.ar) + aral_structures(pdc_globals.epdl.ar);
+struct aral_statistics *epdl_aral_stats(void) {
+    return aral_get_statistics(pdc_globals.epdl.ar);
 }
 
 // ----------------------------------------------------------------------------
@@ -137,10 +141,12 @@ void deol_init(void) {
             "dbengine-deol",
             sizeof(DEOL),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false
+            NULL, NULL, false, false, true
     );
+
+    pulse_aral_register(pdc_globals.deol.ar, "deol");
 }
 
 static DEOL *deol_get(void) {
@@ -153,8 +159,8 @@ static void deol_release(DEOL *deol) {
     aral_freez(pdc_globals.deol.ar, deol);
 }
 
-size_t deol_cache_size(void) {
-    return aral_overhead(pdc_globals.deol.ar) + aral_structures(pdc_globals.deol.ar);
+struct aral_statistics *deol_aral_stats(void) {
+    return aral_get_statistics(pdc_globals.deol.ar);
 }
 
 // ----------------------------------------------------------------------------
@@ -176,7 +182,7 @@ static struct {
 
 } extent_buffer_globals = {
         .protected = {
-                .spinlock = NETDATA_SPINLOCK_INITIALIZER,
+                .spinlock = SPINLOCK_INITIALIZER,
                 .available_items = NULL,
                 .available = 0,
         },
@@ -184,12 +190,12 @@ static struct {
                 .allocated = 0,
                 .allocated_bytes = 0,
         },
-        .max_size = MAX_PAGES_PER_EXTENT * RRDENG_BLOCK_SIZE,
+        .max_size = MAX_EXTENT_UNCOMPRESSED_SIZE
 };
 
 void extent_buffer_init(void) {
-    size_t max_extent_uncompressed = MAX_PAGES_PER_EXTENT * RRDENG_BLOCK_SIZE;
-    size_t max_size = (size_t)LZ4_compressBound(MAX_PAGES_PER_EXTENT * RRDENG_BLOCK_SIZE);
+    size_t max_extent_uncompressed = MAX_EXTENT_UNCOMPRESSED_SIZE;
+    size_t max_size = (size_t)LZ4_compressBound(MAX_EXTENT_UNCOMPRESSED_SIZE);
     if(max_size < max_extent_uncompressed)
         max_size = max_extent_uncompressed;
 
@@ -449,50 +455,75 @@ void epdl_cmd_dequeued(void *epdl_ptr) {
     epdl->cmd = NULL;
 }
 
-static struct rrdeng_cmd *epdl_get_cmd(void *epdl_ptr) {
+static inline struct rrdeng_cmd *epdl_get_cmd(void *epdl_ptr) {
     EPDL *epdl = epdl_ptr;
     return epdl->cmd;
 }
 
+static EPDL_EXTENT *epdl_find_extent_base(EPDL *epdl) {
+    EPDL_EXTENT *e = NULL;
+    rw_spinlock_read_lock(&epdl->datafile->extent_epdl.spinlock);
+    Pvoid_t *PValue = JudyLGet(epdl->datafile->extent_epdl.epdl_per_extent, epdl->extent_offset, PJE0);
+    internal_fatal(PValue == PJERR, "DBENGINE: corrupted pending extent judy");
+    if(PValue)
+        e = *PValue;
+    rw_spinlock_read_unlock(&epdl->datafile->extent_epdl.spinlock);
+
+    if(!e) {
+        EPDL_EXTENT *e_to_free = NULL;
+        e = callocz(1, sizeof(*e));
+
+        rw_spinlock_write_lock(&epdl->datafile->extent_epdl.spinlock);
+        Pvoid_t *PValue = JudyLIns(&epdl->datafile->extent_epdl.epdl_per_extent, epdl->extent_offset, PJE0);
+        internal_fatal(!PValue || PValue == PJERR, "DBENGINE: corrupted pending extent judy");
+        if(!*PValue) {
+            *PValue = e;
+            spinlock_init(&e->spinlock);
+        }
+        else {
+            e_to_free = e;
+            e = *PValue;
+        }
+        rw_spinlock_write_unlock(&epdl->datafile->extent_epdl.spinlock);
+
+        freez(e_to_free);
+    }
+
+    return e;
+}
+
 static bool epdl_pending_add(EPDL *epdl) {
+    EPDL_EXTENT *e = epdl_find_extent_base(epdl);
+    spinlock_lock(&e->spinlock);
+
     bool added_new;
+    if(unlikely(e->base)) {
+        added_new = false;
+        epdl->head_to_datafile_extent_queries_pending_for_extent = false;
 
-    spinlock_lock(&epdl->datafile->extent_queries.spinlock);
-    Pvoid_t *PValue = JudyLIns(&epdl->datafile->extent_queries.pending_epdl_by_extent_offset_judyL, epdl->extent_offset, PJE0);
-    internal_fatal(!PValue || PValue == PJERR, "DBENGINE: corrupted pending extent judy");
+        __atomic_add_fetch(&rrdeng_cache_efficiency_stats.pages_load_extent_merged, 1, __ATOMIC_RELAXED);
 
-    EPDL *base = *PValue;
-
-    if(!base) {
+//        if(e->base->pdc->priority > epdl->pdc->priority) {
+//            e->base->pdc->priority = epdl->pdc->priority;
+//            rrdeng_req_cmd(epdl_get_cmd, e->base, epdl->pdc->priority);
+//        }
+    }
+    else {
         added_new = true;
         epdl->head_to_datafile_extent_queries_pending_for_extent = true;
     }
-    else {
-        added_new = false;
-        epdl->head_to_datafile_extent_queries_pending_for_extent = false;
-        __atomic_add_fetch(&rrdeng_cache_efficiency_stats.pages_load_extent_merged, 1, __ATOMIC_RELAXED);
 
-        if(base->pdc->priority > epdl->pdc->priority)
-            rrdeng_req_cmd(epdl_get_cmd, base, epdl->pdc->priority);
-    }
-
-    DOUBLE_LINKED_LIST_APPEND_ITEM_UNSAFE(base, epdl, query.prev, query.next);
-    *PValue = base;
-
-    spinlock_unlock(&epdl->datafile->extent_queries.spinlock);
+    DOUBLE_LINKED_LIST_APPEND_ITEM_UNSAFE(e->base, epdl, query.prev, query.next);
+    spinlock_unlock(&e->spinlock);
 
     return added_new;
 }
 
 static void epdl_pending_del(EPDL *epdl) {
-    spinlock_lock(&epdl->datafile->extent_queries.spinlock);
-    if(epdl->head_to_datafile_extent_queries_pending_for_extent) {
-        epdl->head_to_datafile_extent_queries_pending_for_extent = false;
-        int rc = JudyLDel(&epdl->datafile->extent_queries.pending_epdl_by_extent_offset_judyL, epdl->extent_offset, PJE0);
-        (void) rc;
-        internal_fatal(!rc, "DBENGINE: epdl not found in pending list");
-    }
-    spinlock_unlock(&epdl->datafile->extent_queries.spinlock);
+    EPDL_EXTENT *e = epdl_find_extent_base(epdl);
+    spinlock_lock(&e->spinlock);
+    e->base = NULL;
+    spinlock_unlock(&e->spinlock);
 }
 
 void pdc_to_epdl_router(struct rrdengine_instance *ctx, PDC *pdc, execute_extent_page_details_list_t exec_first_extent_list, execute_extent_page_details_list_t exec_rest_extent_list)
@@ -651,7 +682,7 @@ inline VALIDATED_PAGE_DESCRIPTOR validate_extent_page_descr(const struct rrdeng_
     }
 
     return validate_page(
-            (uuid_t *)descr->uuid,
+            (nd_uuid_t *)descr->uuid,
             start_time_s,
             end_time_s,
             0,
@@ -665,7 +696,7 @@ inline VALIDATED_PAGE_DESCRIPTOR validate_extent_page_descr(const struct rrdeng_
 }
 
 VALIDATED_PAGE_DESCRIPTOR validate_page(
-        uuid_t *uuid,
+        nd_uuid_t *uuid,
         time_t start_time_s,
         time_t end_time_s,
         uint32_t update_every_s,                // can be zero, if unknown
@@ -710,7 +741,7 @@ VALIDATED_PAGE_DESCRIPTOR validate_page(
 
     // allow to be called without update every (when loading pages from disk)
     if(!update_every_s) {
-        vd.update_every_s = (vd.entries > 1) ? ((vd.end_time_s - vd.start_time_s) / (time_t) (vd.entries - 1))
+        vd.update_every_s = (vd.entries > 1) ? ((uint32_t)(vd.end_time_s - vd.start_time_s) / (vd.entries - 1))
                                              : overwrite_zero_update_every_s;
 
         update_every_s = vd.update_every_s;
@@ -898,7 +929,7 @@ static void epdl_extent_loading_error_log(struct rrdengine_instance *ctx, EPDL *
                 start_time_s = pd->first_time_s;
                 end_time_s = pd->last_time_s;
                 METRIC *metric = (METRIC *)pd->metric_id;
-                uuid_t *u = mrg_metric_uuid(main_mrg, metric);
+                nd_uuid_t *u = mrg_metric_uuid(main_mrg, metric);
                 uuid_unparse_lower(*u, uuid);
                 used_epdl = true;
             }
@@ -943,7 +974,8 @@ static bool epdl_populate_pages_from_extent_data(
 {
     unsigned i, count;
     void *uncompressed_buf = NULL;
-    uint32_t payload_length, payload_offset, trailer_offset, uncompressed_payload_length = 0;
+    uint64_t payload_length, payload_offset, trailer_offset;
+    uint32_t uncompressed_payload_length = 0;
     bool have_read_error = false;
     /* persistent structures */
     struct rrdeng_df_extent_header *header;
@@ -1010,7 +1042,7 @@ static bool epdl_populate_pages_from_extent_data(
             uncompressed_payload_length += header->descr[i].page_length;
         }
 
-        if(unlikely(uncompressed_payload_length > MAX_PAGES_PER_EXTENT * RRDENG_BLOCK_SIZE))
+        if(unlikely(uncompressed_payload_length > MAX_EXTENT_UNCOMPRESSED_SIZE))
             have_read_error = true;
 
         if(likely(!have_read_error)) {
@@ -1053,7 +1085,7 @@ static bool epdl_populate_pages_from_extent_data(
             continue;
         }
 
-        METRIC *metric = mrg_metric_get_and_acquire(main_mrg, &header->descr[i].uuid, (Word_t)ctx);
+        METRIC *metric = mrg_metric_get_and_acquire_by_uuid(main_mrg, &header->descr[i].uuid, (Word_t)ctx);
         Word_t metric_id = (Word_t)metric;
         if(!metric) {
             char log[200 + 1];
@@ -1126,6 +1158,7 @@ static bool epdl_populate_pages_from_extent_data(
         PGC_PAGE *page = pgc_page_add_and_acquire(main_cache, page_entry, &added);
         if (false == added) {
             pgd_free(pgd);
+            pgd = pgc_page_data(page);
             stats_cache_hit_while_inserting++;
             stats_data_from_main_cache++;
         }
@@ -1177,9 +1210,9 @@ static bool epdl_populate_pages_from_extent_data(
     return true;
 }
 
-static inline void *datafile_extent_read(struct rrdengine_instance *ctx, uv_file file, unsigned pos, unsigned size_bytes)
+static inline void *datafile_extent_read(struct rrdengine_instance *ctx, uv_file file, uint64_t pos, unsigned size_bytes)
 {
-    void *buffer;
+    void *buffer = NULL;
     uv_fs_t request;
 
     unsigned real_io_size = ALIGN_BYTES_CEILING(size_bytes);
@@ -1188,7 +1221,7 @@ static inline void *datafile_extent_read(struct rrdengine_instance *ctx, uv_file
         fatal("DBENGINE: posix_memalign(): %s", strerror(ret));
 
     uv_buf_t iov = uv_buf_init(buffer, real_io_size);
-    ret = uv_fs_read(NULL, &request, file, &iov, 1, pos, NULL);
+    ret = uv_fs_read(NULL, &request, file, &iov, 1, (int64_t)pos, NULL);
     if (unlikely(-1 == ret)) {
         ctx_io_error(ctx);
         posix_memfree(buffer);
@@ -1256,9 +1289,10 @@ void epdl_find_extent_and_populate_pages(struct rrdengine_instance *ctx, EPDL *e
         void *extent_data = datafile_extent_read(ctx, epdl->file, epdl->extent_offset, epdl->extent_size);
         if(extent_data != NULL) {
 
-            void *copied_extent_compressed_data = dbengine_extent_alloc(epdl->extent_size);
-            memcpy(copied_extent_compressed_data, extent_data, epdl->extent_size);
+            void *tmp = dbengine_extent_alloc(epdl->extent_size);
+            memcpy(tmp, extent_data, epdl->extent_size);
             datafile_extent_read_free(extent_data);
+            extent_data = tmp;
 
             if(worker)
                 worker_is_busy(UV_EVENT_DBENGINE_EXTENT_CACHE_LOOKUP);
@@ -1272,11 +1306,11 @@ void epdl_find_extent_and_populate_pages(struct rrdengine_instance *ctx, EPDL *e
                     .size = epdl->extent_size,
                     .end_time_s = 0,
                     .update_every_s = 0,
-                    .data = copied_extent_compressed_data,
+                    .data = extent_data,
             }, &added);
 
             if (!added) {
-                dbengine_extent_free(copied_extent_compressed_data, epdl->extent_size);
+                dbengine_extent_free(extent_data, epdl->extent_size);
                 internal_fatal(epdl->extent_size != pgc_page_data_size(extent_cache, extent_cache_page),
                                "DBENGINE: cache size does not match the expected size");
             }
