@@ -1,15 +1,10 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-#define NETDATA_RRD_INTERNALS
 
 #include "rrdengine.h"
 #include "pdc.h"
 #include "dbengine-compression.h"
 
-rrdeng_stats_t global_io_errors = 0;
-rrdeng_stats_t global_fs_errors = 0;
-rrdeng_stats_t rrdeng_reserved_file_descriptors = 0;
-rrdeng_stats_t global_pg_cache_over_half_dirty_events = 0;
-rrdeng_stats_t global_flushing_pressure_page_deletions = 0;
+struct rrdeng_global_stats global_stats = { 0 };
 
 unsigned rrdeng_pages_per_extent = DEFAULT_PAGES_PER_EXTENT;
 
@@ -45,7 +40,9 @@ struct rrdeng_main {
     bool shutdown;
 
     size_t flushes_running;
-    size_t evictions_running;
+    size_t evict_main_running;
+    size_t evict_open_running;
+    size_t evict_extent_running;
     size_t cleanup_running;
 
     struct {
@@ -86,13 +83,14 @@ struct rrdeng_main {
         .loop = {},
         .async = {},
         .timer = {},
+        .retention_timer = {},
         .flushes_running = 0,
-        .evictions_running = 0,
+        .evict_main_running = 0,
         .cleanup_running = 0,
 
         .cmd_queue = {
                 .unsafe = {
-                        .spinlock = NETDATA_SPINLOCK_INITIALIZER,
+                        .spinlock = SPINLOCK_INITIALIZER,
                 },
         }
 };
@@ -138,12 +136,15 @@ struct rrdeng_work {
 
 static void work_request_init(void) {
     rrdeng_main.work_cmd.ar = aral_create(
-            "dbengine-work-cmd",
-            sizeof(struct rrdeng_work),
-            0,
-            65536, NULL,
-            NULL, NULL, false, false
+        "dbengine-work-cmd",
+        sizeof(struct rrdeng_work),
+        0,
+        0,
+        NULL,
+        NULL, NULL, false, false, true
     );
+
+    pulse_aral_register(rrdeng_main.work_cmd.ar, "workers");
 }
 
 enum LIBUV_WORKERS_STATUS {
@@ -259,9 +260,11 @@ void page_descriptors_init(void) {
             "dbengine-descriptors",
             sizeof(struct page_descr_with_data),
             0,
-            65536 * 4,
+            0,
             NULL,
-            NULL, NULL, false, false);
+            NULL, NULL, false, false, true);
+
+    pulse_aral_register(rrdeng_main.descriptors.ar, "descriptors");
 }
 
 struct page_descr_with_data *page_descriptor_get(void) {
@@ -282,10 +285,12 @@ static void extent_io_descriptor_init(void) {
             "dbengine-extent-io",
             sizeof(struct extent_io_descriptor),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false
+            NULL, NULL, false, false, true
             );
+
+    pulse_aral_register(rrdeng_main.xt_io_descr.ar, "extent io");
 }
 
 static struct extent_io_descriptor *extent_io_descriptor_get(void) {
@@ -306,9 +311,11 @@ void rrdeng_query_handle_init(void) {
             "dbengine-query-handles",
             sizeof(struct rrdeng_query_handle),
             0,
-            65536,
+            0,
             NULL,
-            NULL, NULL, false, false);
+            NULL, NULL, false, false, true);
+
+    pulse_aral_register(rrdeng_main.handles.ar, "query handles");
 }
 
 struct rrdeng_query_handle *rrdeng_query_handle_get(void) {
@@ -336,7 +343,7 @@ static struct {
     } atomics;
 } wal_globals = {
         .protected = {
-                .spinlock = NETDATA_SPINLOCK_INITIALIZER,
+                .spinlock = SPINLOCK_INITIALIZER,
                 .available_items = NULL,
                 .available = 0,
         },
@@ -351,7 +358,7 @@ static void wal_cleanup1(void) {
     if(!spinlock_trylock(&wal_globals.protected.spinlock))
         return;
 
-    if(wal_globals.protected.available_items && wal_globals.protected.available > storage_tiers) {
+    if(wal_globals.protected.available_items && wal_globals.protected.available > nd_profile.storage_tiers) {
         wal = wal_globals.protected.available_items;
         DOUBLE_LINKED_LIST_REMOVE_ITEM_UNSAFE(wal_globals.protected.available_items, wal, cache.prev, cache.next);
         wal_globals.protected.available--;
@@ -426,9 +433,11 @@ static void rrdeng_cmd_queue_init(void) {
     rrdeng_main.cmd_queue.ar = aral_create("dbengine-opcodes",
                                            sizeof(struct rrdeng_cmd),
                                            0,
-                                           65536,
+                                           0,
                                            NULL,
-                                           NULL, NULL, false, false);
+                                           NULL, NULL, false, false, true);
+
+    pulse_aral_register(rrdeng_main.cmd_queue.ar, "opcodes");
 }
 
 static inline STORAGE_PRIORITY rrdeng_enq_cmd_map_opcode_to_priority(enum rrdeng_opcode opcode, STORAGE_PRIORITY priority) {
@@ -515,18 +524,19 @@ static inline bool rrdeng_cmd_has_waiting_opcodes_in_lower_priorities(STORAGE_PR
 static inline struct rrdeng_cmd rrdeng_deq_cmd(bool from_worker) {
     struct rrdeng_cmd *cmd = NULL;
     enum LIBUV_WORKERS_STATUS status = work_request_full();
-
     STORAGE_PRIORITY min_priority, max_priority;
-    min_priority = STORAGE_PRIORITY_INTERNAL_DBENGINE;
-    max_priority = (status != LIBUV_WORKERS_RELAXED) ? STORAGE_PRIORITY_INTERNAL_DBENGINE : STORAGE_PRIORITY_INTERNAL_MAX_DONT_USE - 1;
 
-    if(from_worker) {
+    if(unlikely(from_worker)) {
         if(status == LIBUV_WORKERS_CRITICAL)
             return opcode_empty;
 
         min_priority = STORAGE_PRIORITY_INTERNAL_QUERY_PREP;
         max_priority = STORAGE_PRIORITY_BEST_EFFORT;
     }
+    else {
+        min_priority = STORAGE_PRIORITY_INTERNAL_DBENGINE;
+        max_priority = (status != LIBUV_WORKERS_RELAXED) ? STORAGE_PRIORITY_INTERNAL_DBENGINE : STORAGE_PRIORITY_INTERNAL_MAX_DONT_USE - 1;
+    }
 
     // find an opcode to execute from the queue
     spinlock_lock(&rrdeng_main.cmd_queue.unsafe.spinlock);
@@ -574,15 +584,6 @@ static inline struct rrdeng_cmd rrdeng_deq_cmd(bool from_worker) {
 
 // ----------------------------------------------------------------------------
 
-void *dbengine_extent_alloc(size_t size) {
-    void *extent = mallocz(size);
-    return extent;
-}
-
-void dbengine_extent_free(void *extent, size_t size __maybe_unused) {
-    freez(extent);
-}
-
 static void journalfile_extent_build(struct rrdengine_instance *ctx, struct extent_io_descriptor *xt_io_descr) {
     unsigned count, payload_length, descr_size, size_bytes;
     void *buf;
@@ -1128,7 +1129,7 @@ static void update_metrics_first_time_s(struct rrdengine_instance *ctx, struct r
 
     size_t added = 0;
     for (size_t index = 0; index < count; ++index) {
-        METRIC *metric = mrg_metric_get_and_acquire(main_mrg, &uuid_list[index].uuid, (Word_t) ctx);
+        METRIC *metric = mrg_metric_get_and_acquire_by_uuid(main_mrg, &uuid_list[index].uuid, (Word_t)ctx);
         if (!metric)
             continue;
 
@@ -1218,7 +1219,7 @@ void datafile_delete(struct rrdengine_instance *ctx, struct rrdengine_datafile *
     if(worker)
         worker_is_busy(UV_EVENT_DBENGINE_DATAFILE_DELETE_WAIT);
 
-    bool datafile_got_for_deletion = datafile_acquire_for_deletion(datafile);
+    bool datafile_got_for_deletion = datafile_acquire_for_deletion(datafile, false);
 
     if (update_retention)
         update_metrics_first_time_s(ctx, datafile, datafile->next, worker);
@@ -1227,7 +1228,7 @@ void datafile_delete(struct rrdengine_instance *ctx, struct rrdengine_datafile *
         if(worker)
             worker_is_busy(UV_EVENT_DBENGINE_DATAFILE_DELETE_WAIT);
 
-        datafile_got_for_deletion = datafile_acquire_for_deletion(datafile);
+        datafile_got_for_deletion = datafile_acquire_for_deletion(datafile, false);
 
         if (!datafile_got_for_deletion) {
             netdata_log_info("DBENGINE: waiting for data file '%s/"
@@ -1279,6 +1280,22 @@ void datafile_delete(struct rrdengine_instance *ctx, struct rrdengine_datafile *
         netdata_log_info("DBENGINE: deleted data file \"%s\".", path);
         deleted_bytes += datafile_bytes;
     }
+
+    {
+        rw_spinlock_write_lock(&datafile->extent_epdl.spinlock);
+        bool first = true;
+        Word_t idx = 0;
+        Pvoid_t *PValue;
+        while ((PValue = JudyLFirstThenNext(datafile->extent_epdl.epdl_per_extent, &idx, &first))) {
+            EPDL_EXTENT *e = *PValue;
+            internal_fatal(e->base, "The should not be any EPDLs ");
+            freez(e);
+            *PValue = NULL;
+        }
+        JudyLFreeArray(&datafile->extent_epdl.epdl_per_extent, PJE0);
+        rw_spinlock_write_unlock(&datafile->extent_epdl.spinlock);
+    }
+
     freez(journal_file);
     freez(datafile);
 
@@ -1304,7 +1321,10 @@ static void after_flush_all_hot_and_dirty_pages_of_section(struct rrdengine_inst
 static void *flush_all_hot_and_dirty_pages_of_section_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *uv_work_req __maybe_unused) {
     worker_is_busy(UV_EVENT_DBENGINE_QUIESCE);
     pgc_flush_all_hot_and_dirty_pages(main_cache, (Word_t)ctx);
-    completion_mark_complete(&ctx->quiesce.completion);
+
+    for(size_t i = 0; i < pgc_max_flushers() ; i++)
+        rrdeng_enq_cmd(NULL, RRDENG_OPCODE_FLUSH_MAIN, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+
     return data;
 }
 
@@ -1376,17 +1396,41 @@ static void *cache_flush_tp_worker(struct rrdengine_instance *ctx __maybe_unused
         return data;
 
     worker_is_busy(UV_EVENT_DBENGINE_FLUSH_MAIN_CACHE);
-    pgc_flush_pages(main_cache, 0);
+    while (pgc_flush_pages(main_cache))
+        yield_the_processor();
 
     return data;
 }
 
-static void *cache_evict_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *req __maybe_unused) {
+static void *cache_evict_main_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *req __maybe_unused) {
     if (!main_cache)
         return data;
 
     worker_is_busy(UV_EVENT_DBENGINE_EVICT_MAIN_CACHE);
-    pgc_evict_pages(main_cache, 0, 0);
+    while (pgc_evict_pages(main_cache, 0, 0))
+        yield_the_processor();
+
+    return data;
+}
+
+static void *cache_evict_open_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *req __maybe_unused) {
+    if (!open_cache)
+        return data;
+
+    worker_is_busy(UV_EVENT_DBENGINE_EVICT_OPEN_CACHE);
+    while (pgc_evict_pages(open_cache, 0, 0))
+        yield_the_processor();
+
+    return data;
+}
+
+static void *cache_evict_extent_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *req __maybe_unused) {
+    if (!extent_cache)
+        return data;
+
+    worker_is_busy(UV_EVENT_DBENGINE_EVICT_EXTENT_CACHE);
+    while (pgc_evict_pages(extent_cache, 0, 0))
+        yield_the_processor();
 
     return data;
 }
@@ -1465,6 +1509,10 @@ void pdc_route_synchronously(struct rrdengine_instance *ctx, struct page_details
     pdc_to_epdl_router(ctx, pdc, epdl_populate_pages_synchronously, epdl_populate_pages_synchronously);
 }
 
+void pdc_route_synchronously_first(struct rrdengine_instance *ctx, struct page_details_control *pdc) {
+    pdc_to_epdl_router(ctx, pdc, epdl_populate_pages_synchronously, epdl_populate_pages_asynchronously);
+}
+
 #define MAX_RETRIES_TO_START_INDEX (100)
 static void *journal_v2_indexing_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *uv_work_req __maybe_unused) {
     unsigned count = 0;
@@ -1517,7 +1565,7 @@ static void *journal_v2_indexing_tp_worker(struct rrdengine_instance *ctx __mayb
             break;
     }
 
-    errno = 0;
+    errno_clear();
     if(count)
         nd_log(NDLS_DAEMON, NDLP_DEBUG,
                "DBENGINE: journal indexing done; %u files processed",
@@ -1532,8 +1580,16 @@ static void after_do_cache_flush(struct rrdengine_instance *ctx __maybe_unused,
     rrdeng_main.flushes_running--;
 }
 
-static void after_do_cache_evict(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
-    rrdeng_main.evictions_running--;
+static void after_do_main_cache_evict(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
+    rrdeng_main.evict_main_running--;
+}
+
+static void after_do_open_cache_evict(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
+    rrdeng_main.evict_open_running--;
+}
+
+static void after_do_extent_cache_evict(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
+    rrdeng_main.evict_extent_running--;
 }
 
 static void after_journal_v2_indexing(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
@@ -1541,25 +1597,24 @@ static void after_journal_v2_indexing(struct rrdengine_instance *ctx __maybe_unu
     rrdeng_enq_cmd(ctx, RRDENG_OPCODE_DATABASE_ROTATE, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
 }
 
-struct rrdeng_buffer_sizes rrdeng_get_buffer_sizes(void) {
+struct rrdeng_buffer_sizes rrdeng_pulse_memory_sizes(void) {
     return (struct rrdeng_buffer_sizes) {
-            .pgc         = pgc_aral_overhead() + pgc_aral_structures(),
-            .mrg         = mrg_aral_overhead() + mrg_aral_structures(),
-            .opcodes     = aral_overhead(rrdeng_main.cmd_queue.ar) + aral_structures(rrdeng_main.cmd_queue.ar),
-            .handles     = aral_overhead(rrdeng_main.handles.ar) + aral_structures(rrdeng_main.handles.ar),
-            .descriptors = aral_overhead(rrdeng_main.descriptors.ar) + aral_structures(rrdeng_main.descriptors.ar),
-            .wal         = __atomic_load_n(&wal_globals.atomics.allocated, __ATOMIC_RELAXED) * (sizeof(WAL) + RRDENG_BLOCK_SIZE),
-            .workers     = aral_overhead(rrdeng_main.work_cmd.ar),
-            .pdc         = pdc_cache_size(),
-            .xt_io       = aral_overhead(rrdeng_main.xt_io_descr.ar) + aral_structures(rrdeng_main.xt_io_descr.ar),
-            .xt_buf      = extent_buffer_cache_size(),
-            .epdl        = epdl_cache_size(),
-            .deol        = deol_cache_size(),
-            .pd          = pd_cache_size(),
-
-#ifdef PDC_USE_JULYL
-            .julyl       = julyl_cache_size(),
-#endif
+        .as = {
+            [RRDENG_MEM_PGC]            = pgc_aral_stats(),
+            [RRDENG_MEM_PGD]            = pgd_aral_stats(),
+            [RRDENG_MEM_MRG]            = mrg_aral_stats(),
+            [RRDENG_MEM_PDC]            = pdc_aral_stats(),
+            [RRDENG_MEM_EPDL]           = epdl_aral_stats(),
+            [RRDENG_MEM_DEOL]           = deol_aral_stats(),
+            [RRDENG_MEM_PD]             = pd_aral_stats(),
+            [RRDENG_MEM_OPCODES]        = aral_get_statistics(rrdeng_main.cmd_queue.ar),
+            [RRDENG_MEM_HANDLES]        = aral_get_statistics(rrdeng_main.handles.ar),
+            [RRDENG_MEM_DESCRIPTORS]    = aral_get_statistics(rrdeng_main.descriptors.ar),
+            [RRDENG_MEM_WORKERS]        = aral_get_statistics(rrdeng_main.work_cmd.ar),
+            [RRDENG_MEM_XT_IO]          = aral_get_statistics(rrdeng_main.xt_io_descr.ar),
+        },
+        .wal    = __atomic_load_n(&wal_globals.atomics.allocated, __ATOMIC_RELAXED) * (sizeof(WAL) + RRDENG_BLOCK_SIZE),
+        .xt_buf = extent_buffer_cache_size(),
     };
 }
 
@@ -1582,14 +1637,10 @@ static void *cleanup_tp_worker(struct rrdengine_instance *ctx __maybe_unused, vo
         }
     }
 
-#ifdef PDC_USE_JULYL
-    julyl_cleanup1();
-#endif
-
     return data;
 }
 
-uint64_t get_used_disk_space(struct rrdengine_instance *ctx)
+uint64_t rrdeng_get_used_disk_space(struct rrdengine_instance *ctx)
 {
     uint64_t active_space = 0;
 
@@ -1630,7 +1681,7 @@ bool rrdeng_ctx_tier_cap_exceeded(struct rrdengine_instance *ctx)
         // only 1 datafile available
         return false;
 
-    uint64_t estimated_disk_space = get_used_disk_space(ctx);
+    uint64_t estimated_disk_space = rrdeng_get_used_disk_space(ctx);
     time_t retention = get_tier_retention(ctx);
 
     if (ctx->config.max_retention_s && retention > ctx->config.max_retention_s)
@@ -1642,8 +1693,7 @@ bool rrdeng_ctx_tier_cap_exceeded(struct rrdengine_instance *ctx)
     return false;
 }
 
-void retention_timer_cb(uv_timer_t *handle)
-{
+static void retention_timer_cb(uv_timer_t *handle) {
     if (!localhost)
         return;
 
@@ -1651,7 +1701,7 @@ void retention_timer_cb(uv_timer_t *handle)
     uv_stop(handle->loop);
     uv_update_time(handle->loop);
 
-    for (size_t tier = 0; tier < storage_tiers; tier++) {
+    for (size_t tier = 0; tier < nd_profile.storage_tiers; tier++) {
         STORAGE_ENGINE *eng = localhost->db[tier].eng;
         if (!eng || eng->seb != STORAGE_ENGINE_BACKEND_DBENGINE)
             continue;
@@ -1663,7 +1713,7 @@ void retention_timer_cb(uv_timer_t *handle)
     worker_is_idle();
 }
 
-void timer_cb(uv_timer_t* handle) {
+static void timer_per_sec_cb(uv_timer_t* handle) {
     worker_is_busy(RRDENG_TIMER_CB);
     uv_stop(handle->loop);
     uv_update_time(handle->loop);
@@ -1672,14 +1722,17 @@ void timer_cb(uv_timer_t* handle) {
     worker_set_metric(RRDENG_WORKS_DISPATCHED, (NETDATA_DOUBLE)__atomic_load_n(&rrdeng_main.work_cmd.atomics.dispatched, __ATOMIC_RELAXED));
     worker_set_metric(RRDENG_WORKS_EXECUTING, (NETDATA_DOUBLE)__atomic_load_n(&rrdeng_main.work_cmd.atomics.executing, __ATOMIC_RELAXED));
 
-    rrdeng_enq_cmd(NULL, RRDENG_OPCODE_FLUSH_INIT, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
-    rrdeng_enq_cmd(NULL, RRDENG_OPCODE_EVICT_INIT, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+    // rrdeng_enq_cmd(NULL, RRDENG_OPCODE_EVICT_MAIN, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+    // rrdeng_enq_cmd(NULL, RRDENG_OPCODE_EVICT_OPEN, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+    // rrdeng_enq_cmd(NULL, RRDENG_OPCODE_EVICT_EXTENT, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+    rrdeng_enq_cmd(NULL, RRDENG_OPCODE_FLUSH_MAIN, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
     rrdeng_enq_cmd(NULL, RRDENG_OPCODE_CLEANUP, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
 
     worker_is_idle();
 }
 
 static void dbengine_initialize_structures(void) {
+    pgd_init_arals();
     pgc_and_mrg_initialize();
 
     pdc_init();
@@ -1691,13 +1744,12 @@ static void dbengine_initialize_structures(void) {
     rrdeng_query_handle_init();
     page_descriptors_init();
     extent_buffer_init();
-    pgd_init_arals();
     extent_io_descriptor_init();
 }
 
 bool rrdeng_dbengine_spawn(struct rrdengine_instance *ctx __maybe_unused) {
     static bool spawned = false;
-    static SPINLOCK spinlock = NETDATA_SPINLOCK_INITIALIZER;
+    static SPINLOCK spinlock = SPINLOCK_INITIALIZER;
 
     spinlock_lock(&spinlock);
 
@@ -1768,7 +1820,7 @@ static inline void worker_dispatch_query_prep(struct rrdeng_cmd cmd, bool from_w
         work_dispatch(ctx, pdc, NULL, cmd.opcode, query_prep_tp_worker, NULL);
 }
 
-uint64_t get_directory_free_bytes_space(struct rrdengine_instance *ctx)
+uint64_t rrdeng_get_directory_free_bytes_space(struct rrdengine_instance *ctx)
 {
     uint64_t free_bytes = 0;
     struct statvfs buff_statvfs;
@@ -1778,7 +1830,7 @@ uint64_t get_directory_free_bytes_space(struct rrdengine_instance *ctx)
     return (free_bytes - (free_bytes * 5 / 100));
 }
 
-void calculate_tier_disk_space_percentage(void)
+void rrdeng_calculate_tier_disk_space_percentage(void)
 {
     uint64_t tier_space[RRD_STORAGE_TIERS];
 
@@ -1786,7 +1838,7 @@ void calculate_tier_disk_space_percentage(void)
         return;
 
     uint64_t total_diskspace = 0;
-    for(size_t tier = 0; tier < storage_tiers ;tier++) {
+    for(size_t tier = 0; tier < nd_profile.storage_tiers;tier++) {
         STORAGE_ENGINE *eng = localhost->db[tier].eng;
         if (!eng || eng->seb != STORAGE_ENGINE_BACKEND_DBENGINE) {
             tier_space[tier] = 0;
@@ -1794,96 +1846,18 @@ void calculate_tier_disk_space_percentage(void)
         }
         uint64_t tier_disk_space = multidb_ctx[tier]->config.max_disk_space ?
                                        multidb_ctx[tier]->config.max_disk_space :
-                                       get_directory_free_bytes_space(multidb_ctx[tier]);
+                                       rrdeng_get_directory_free_bytes_space(multidb_ctx[tier]);
         total_diskspace += tier_disk_space;
         tier_space[tier] = tier_disk_space;
     }
 
     if (total_diskspace) {
-        for (size_t tier = 0; tier < storage_tiers; tier++) {
+        for (size_t tier = 0; tier < nd_profile.storage_tiers; tier++) {
             multidb_ctx[tier]->config.disk_percentage = (100 * tier_space[tier] / total_diskspace);
         }
     }
 }
 
-void dbengine_retention_statistics(void)
-{
-    static bool init = false;
-    static DBENGINE_TIER_STATS stats[RRD_STORAGE_TIERS];
-
-    if (!localhost)
-        return;
-
-    calculate_tier_disk_space_percentage();
-
-    for (size_t tier = 0; tier < storage_tiers; tier++) {
-        STORAGE_ENGINE *eng = localhost->db[tier].eng;
-        if (!eng || eng->seb != STORAGE_ENGINE_BACKEND_DBENGINE)
-            continue;
-
-        if (init == false) {
-            char id[200];
-            snprintfz(id, sizeof(id) - 1, "dbengine_retention_tier%zu", tier);
-            stats[tier].st = rrdset_create_localhost(
-                "netdata",
-                id,
-                NULL,
-                "dbengine retention",
-                "netdata.dbengine_tier_retention",
-                "dbengine space and time retention",
-                "%",
-                "netdata",
-                "stats",
-                134900, // before "dbengine memory" (dbengine2_statistics_charts)
-                10,
-                RRDSET_TYPE_LINE);
-
-            stats[tier].rd_space = rrddim_add(stats[tier].st, "space", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            stats[tier].rd_time = rrddim_add(stats[tier].st, "time", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-
-            char tier_str[5];
-            snprintfz(tier_str, 4, "%zu", tier);
-            rrdlabels_add(stats[tier].st->rrdlabels, "tier", tier_str, RRDLABEL_SRC_AUTO);
-
-            rrdset_flag_set(stats[tier].st, RRDSET_FLAG_METADATA_UPDATE);
-            rrdhost_flag_set(stats[tier].st->rrdhost, RRDHOST_FLAG_METADATA_UPDATE);
-            rrdset_metadata_updated(stats[tier].st);
-        }
-
-        time_t first_time_s = storage_engine_global_first_time_s(eng->seb, localhost->db[tier].si);
-        time_t retention = first_time_s ? now_realtime_sec() - first_time_s : 0;
-
-        //
-        // Note: storage_engine_disk_space_used is the exact diskspace (as reported by api/v2/node_instances
-        //       get_used_disk_space is used to determine if database cleanup (file rotation should happen)
-        //                           and adds to the disk space used the desired file size of the active
-        //                           datafile
-        uint64_t disk_space = get_used_disk_space(multidb_ctx[tier]);
-        //uint64_t disk_space = storage_engine_disk_space_used(eng->seb, localhost->db[tier].si);
-
-        uint64_t config_disk_space = storage_engine_disk_space_max(eng->seb, localhost->db[tier].si);
-        if (!config_disk_space) {
-            config_disk_space = get_directory_free_bytes_space(multidb_ctx[tier]);
-            config_disk_space += disk_space;
-        }
-
-        collected_number disk_percentage = (collected_number) (config_disk_space ? 100 * disk_space / config_disk_space : 0);
-
-        collected_number retention_percentage = (collected_number)multidb_ctx[tier]->config.max_retention_s ?
-                                                    100 * retention / multidb_ctx[tier]->config.max_retention_s :
-                                                    0;
-
-        if (retention_percentage > 100)
-            retention_percentage = 100;
-
-        rrddim_set_by_pointer(stats[tier].st, stats[tier].rd_space, (collected_number) disk_percentage);
-        rrddim_set_by_pointer(stats[tier].st, stats[tier].rd_time, (collected_number) retention_percentage);
-
-        rrdset_done(stats[tier].st);
-    }
-    init = true;
-}
-
 void dbengine_event_loop(void* arg) {
     sanity_check();
     uv_thread_set_name_np("DBENGINE");
@@ -1900,8 +1874,8 @@ void dbengine_event_loop(void* arg) {
     worker_register_job_name(RRDENG_OPCODE_FLUSHED_TO_OPEN,                          "flushed to open");
     worker_register_job_name(RRDENG_OPCODE_DATABASE_ROTATE,                          "db rotate");
     worker_register_job_name(RRDENG_OPCODE_JOURNAL_INDEX,                            "journal index");
-    worker_register_job_name(RRDENG_OPCODE_FLUSH_INIT,                               "flush init");
-    worker_register_job_name(RRDENG_OPCODE_EVICT_INIT,                               "evict init");
+    worker_register_job_name(RRDENG_OPCODE_FLUSH_MAIN,                               "flush init");
+    worker_register_job_name(RRDENG_OPCODE_EVICT_MAIN,                               "evict init");
     worker_register_job_name(RRDENG_OPCODE_CTX_SHUTDOWN,                             "ctx shutdown");
     worker_register_job_name(RRDENG_OPCODE_CTX_QUIESCE,                              "ctx quiesce");
     worker_register_job_name(RRDENG_OPCODE_SHUTDOWN_EVLOOP,                          "dbengine shutdown");
@@ -1914,8 +1888,8 @@ void dbengine_event_loop(void* arg) {
     worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_FLUSHED_TO_OPEN,      "flushed to open cb");
     worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_DATABASE_ROTATE,      "db rotate cb");
     worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_JOURNAL_INDEX,        "journal index cb");
-    worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_FLUSH_INIT,           "flush init cb");
-    worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_EVICT_INIT,           "evict init cb");
+    worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_FLUSH_MAIN,           "flush init cb");
+    worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_EVICT_MAIN,           "evict init cb");
     worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_CTX_SHUTDOWN,         "ctx shutdown cb");
     worker_register_job_name(RRDENG_OPCODE_MAX + RRDENG_OPCODE_CTX_QUIESCE,          "ctx quiesce cb");
 
@@ -1932,7 +1906,7 @@ void dbengine_event_loop(void* arg) {
     struct rrdeng_cmd cmd;
     main->tid = gettid_cached();
 
-    fatal_assert(0 == uv_timer_start(&main->timer, timer_cb, TIMER_PERIOD_MS, TIMER_PERIOD_MS));
+    fatal_assert(0 == uv_timer_start(&main->timer, timer_per_sec_cb, TIMER_PERIOD_MS, TIMER_PERIOD_MS));
     fatal_assert(0 == uv_timer_start(&main->retention_timer, retention_timer_cb, TIMER_PERIOD_MS * 60, TIMER_PERIOD_MS * 60));
 
     bool shutdown = false;
@@ -1941,7 +1915,15 @@ void dbengine_event_loop(void* arg) {
         uv_run(&main->loop, UV_RUN_DEFAULT);
 
         /* wait for commands */
+        size_t count = 0;
         do {
+            count++;
+
+            if(count % 100 == 0) {
+                worker_is_idle();
+                uv_run(&main->loop, UV_RUN_NOWAIT);
+            }
+
             worker_is_busy(RRDENG_OPCODE_MAX);
             cmd = rrdeng_deq_cmd(RRDENG_OPCODE_NOOP);
             opcode = cmd.opcode;
@@ -1974,18 +1956,34 @@ void dbengine_event_loop(void* arg) {
                     break;
                 }
 
-                case RRDENG_OPCODE_FLUSH_INIT: {
-                    if(rrdeng_main.flushes_running < (size_t)(libuv_worker_threads / 4)) {
+                case RRDENG_OPCODE_FLUSH_MAIN: {
+                    if(rrdeng_main.flushes_running < pgc_max_flushers()) {
                         rrdeng_main.flushes_running++;
                         work_dispatch(NULL, NULL, NULL, opcode, cache_flush_tp_worker, after_do_cache_flush);
                     }
                     break;
                 }
 
-                case RRDENG_OPCODE_EVICT_INIT: {
-                    if(!rrdeng_main.evictions_running) {
-                        rrdeng_main.evictions_running++;
-                        work_dispatch(NULL, NULL, NULL, opcode, cache_evict_tp_worker, after_do_cache_evict);
+                case RRDENG_OPCODE_EVICT_MAIN: {
+                    if(rrdeng_main.evict_main_running < pgc_max_evictors()) {
+                        rrdeng_main.evict_main_running++;
+                        work_dispatch(NULL, NULL, NULL, opcode, cache_evict_main_tp_worker, after_do_main_cache_evict);
+                    }
+                    break;
+                }
+
+                case RRDENG_OPCODE_EVICT_OPEN: {
+                    if(rrdeng_main.evict_open_running < pgc_max_evictors()) {
+                        rrdeng_main.evict_open_running++;
+                        work_dispatch(NULL, NULL, NULL, opcode, cache_evict_open_tp_worker, after_do_open_cache_evict);
+                    }
+                    break;
+                }
+
+                case RRDENG_OPCODE_EVICT_EXTENT: {
+                    if(rrdeng_main.evict_extent_running < pgc_max_evictors()) {
+                        rrdeng_main.evict_extent_running++;
+                        work_dispatch(NULL, NULL, NULL, opcode, cache_evict_extent_tp_worker, after_do_extent_cache_evict);
                     }
                     break;
                 }
@@ -2049,6 +2047,7 @@ void dbengine_event_loop(void* arg) {
 
                 case RRDENG_OPCODE_SHUTDOWN_EVLOOP: {
                     uv_close((uv_handle_t *)&main->async, NULL);
+
                     (void) uv_timer_stop(&main->timer);
                     uv_close((uv_handle_t *)&main->timer, NULL);
 
@@ -2075,6 +2074,6 @@ void dbengine_event_loop(void* arg) {
     }
 
     nd_log(NDLS_DAEMON, NDLP_DEBUG, "Shutting down dbengine thread");
-    uv_loop_close(&main->loop);
+    (void) uv_loop_close(&main->loop);
     worker_unregister();
 }
