@@ -5,18 +5,18 @@
 #include "dbengine-compression.h"
 
 /* Default global database instance */
-struct rrdengine_instance multidb_ctx_storage_tier0;
-struct rrdengine_instance multidb_ctx_storage_tier1;
-struct rrdengine_instance multidb_ctx_storage_tier2;
-struct rrdengine_instance multidb_ctx_storage_tier3;
-struct rrdengine_instance multidb_ctx_storage_tier4;
+struct rrdengine_instance multidb_ctx_storage_tier0 = { 0 };
+struct rrdengine_instance multidb_ctx_storage_tier1 = { 0 };
+struct rrdengine_instance multidb_ctx_storage_tier2 = { 0 };
+struct rrdengine_instance multidb_ctx_storage_tier3 = { 0 };
+struct rrdengine_instance multidb_ctx_storage_tier4 = { 0 };
 
 #define mrg_metric_ctx(metric) (struct rrdengine_instance *)mrg_metric_section(main_mrg, metric)
 
 #if RRD_STORAGE_TIERS != 5
 #error RRD_STORAGE_TIERS is not 5 - you need to add allocations here
 #endif
-struct rrdengine_instance *multidb_ctx[RRD_STORAGE_TIERS];
+struct rrdengine_instance *multidb_ctx[RRD_STORAGE_TIERS] = { 0 };
 uint8_t tier_page_type[RRD_STORAGE_TIERS] = {
     RRDENG_PAGE_TYPE_GORILLA_32BIT,
     RRDENG_PAGE_TYPE_ARRAY_TIER1,
@@ -26,8 +26,10 @@ uint8_t tier_page_type[RRD_STORAGE_TIERS] = {
 
 #if defined(ENV32BIT)
 size_t tier_page_size[RRD_STORAGE_TIERS] = {2048, 1024, 192, 192, 192};
+size_t tier_quota_mb[RRD_STORAGE_TIERS] = {512, 512, 512, 0, 0};
 #else
 size_t tier_page_size[RRD_STORAGE_TIERS] = {4096, 2048, 384, 384, 384};
+size_t tier_quota_mb[RRD_STORAGE_TIERS] = {1024, 1024, 1024, 128, 64};
 #endif
 
 #if RRDENG_PAGE_TYPE_MAX != 2
@@ -40,17 +42,31 @@ size_t page_type_size[256] = {
         [RRDENG_PAGE_TYPE_GORILLA_32BIT] = sizeof(storage_number)
 };
 
+static inline void initialize_single_ctx(struct rrdengine_instance *ctx) {
+    memset(ctx, 0, sizeof(*ctx));
+    uv_rwlock_init(&ctx->datafiles.rwlock);
+    rw_spinlock_init(&ctx->njfv2idx.spinlock);
+}
+
 __attribute__((constructor)) void initialize_multidb_ctx(void) {
     multidb_ctx[0] = &multidb_ctx_storage_tier0;
     multidb_ctx[1] = &multidb_ctx_storage_tier1;
     multidb_ctx[2] = &multidb_ctx_storage_tier2;
     multidb_ctx[3] = &multidb_ctx_storage_tier3;
     multidb_ctx[4] = &multidb_ctx_storage_tier4;
+
+    for(int i = 0; i < RRD_STORAGE_TIERS ; i++)
+        initialize_single_ctx(multidb_ctx[i]);
 }
 
+uint64_t dbengine_out_of_memory_protection = 0;
+bool dbengine_use_all_ram_for_caches = false;
 int db_engine_journal_check = 0;
-int default_rrdeng_disk_quota_mb = 256;
-int default_multidb_disk_quota_mb = 256;
+bool new_dbengine_defaults = false;
+bool legacy_multihost_db_space = false;
+int default_rrdeng_disk_quota_mb = RRDENG_DEFAULT_TIER_DISK_SPACE_MB;
+int default_multidb_disk_quota_mb = RRDENG_DEFAULT_TIER_DISK_SPACE_MB;
+RRD_BACKFILL default_backfill = RRD_BACKFILL_NEW;
 
 #if defined(ENV32BIT)
 int default_rrdeng_page_cache_mb = 16;
@@ -80,7 +96,7 @@ static inline bool rrdeng_page_alignment_release(struct pg_alignment *pa) {
 }
 
 // charts call this
-STORAGE_METRICS_GROUP *rrdeng_metrics_group_get(STORAGE_INSTANCE *si __maybe_unused, uuid_t *uuid __maybe_unused) {
+STORAGE_METRICS_GROUP *rrdeng_metrics_group_get(STORAGE_INSTANCE *si __maybe_unused, nd_uuid_t *uuid __maybe_unused) {
     struct pg_alignment *pa = callocz(1, sizeof(struct pg_alignment));
     rrdeng_page_alignment_acquire(pa);
     return (STORAGE_METRICS_GROUP *)pa;
@@ -98,27 +114,19 @@ void rrdeng_metrics_group_release(STORAGE_INSTANCE *si __maybe_unused, STORAGE_M
 // metric handle for legacy dbs
 
 /* This UUID is not unique across hosts */
-void rrdeng_generate_legacy_uuid(const char *dim_id, const char *chart_id, uuid_t *ret_uuid)
+void rrdeng_generate_unittest_uuid(const char *dim_id, const char *chart_id, nd_uuid_t *ret_uuid)
 {
-    EVP_MD_CTX *evpctx;
-    unsigned char hash_value[EVP_MAX_MD_SIZE];
-    unsigned int hash_len;
-
-    evpctx = EVP_MD_CTX_create();
-    EVP_DigestInit_ex(evpctx, EVP_sha256(), NULL);
-    EVP_DigestUpdate(evpctx, dim_id, strlen(dim_id));
-    EVP_DigestUpdate(evpctx, chart_id, strlen(chart_id));
-    EVP_DigestFinal_ex(evpctx, hash_value, &hash_len);
-    EVP_MD_CTX_destroy(evpctx);
-    fatal_assert(hash_len > sizeof(uuid_t));
-    memcpy(ret_uuid, hash_value, sizeof(uuid_t));
+    CLEAN_BUFFER *wb = buffer_create(100, NULL);
+    buffer_sprintf(wb,"%s.%s", dim_id, chart_id);
+    ND_UUID uuid = UUID_generate_from_hash(buffer_tostring(wb), buffer_strlen(wb));
+    uuid_copy(*ret_uuid, uuid.uuid);
 }
 
-static METRIC *rrdeng_metric_get_legacy(STORAGE_INSTANCE *si, const char *rd_id, const char *st_id) {
+static METRIC *rrdeng_metric_unittest(STORAGE_INSTANCE *si, const char *rd_id, const char *st_id) {
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
-    uuid_t legacy_uuid;
-    rrdeng_generate_legacy_uuid(rd_id, st_id, &legacy_uuid);
-    return mrg_metric_get_and_acquire(main_mrg, &legacy_uuid, (Word_t) ctx);
+    nd_uuid_t legacy_uuid;
+    rrdeng_generate_unittest_uuid(rd_id, st_id, &legacy_uuid);
+    return mrg_metric_get_and_acquire_by_uuid(main_mrg, &legacy_uuid, (Word_t)ctx);
 }
 
 // ----------------------------------------------------------------------------
@@ -134,12 +142,17 @@ STORAGE_METRIC_HANDLE *rrdeng_metric_dup(STORAGE_METRIC_HANDLE *smh) {
     return (STORAGE_METRIC_HANDLE *) mrg_metric_dup(main_mrg, metric);
 }
 
-STORAGE_METRIC_HANDLE *rrdeng_metric_get(STORAGE_INSTANCE *si, uuid_t *uuid) {
+STORAGE_METRIC_HANDLE *rrdeng_metric_get_by_uuid(STORAGE_INSTANCE *si, nd_uuid_t *uuid) {
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
-    return (STORAGE_METRIC_HANDLE *) mrg_metric_get_and_acquire(main_mrg, uuid, (Word_t) ctx);
+    return (STORAGE_METRIC_HANDLE *)mrg_metric_get_and_acquire_by_uuid(main_mrg, uuid, (Word_t)ctx);
 }
 
-static METRIC *rrdeng_metric_create(STORAGE_INSTANCE *si, uuid_t *uuid) {
+STORAGE_METRIC_HANDLE *rrdeng_metric_get_by_id(STORAGE_INSTANCE *si, UUIDMAP_ID id) {
+    struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
+    return (STORAGE_METRIC_HANDLE *)mrg_metric_get_and_acquire_by_id(main_mrg, id, (Word_t)ctx);
+}
+
+static METRIC *rrdeng_metric_create(STORAGE_INSTANCE *si, nd_uuid_t *uuid) {
     internal_fatal(!si, "DBENGINE: STORAGE_INSTANCE is NULL");
 
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
@@ -162,28 +175,25 @@ STORAGE_METRIC_HANDLE *rrdeng_metric_get_or_create(RRDDIM *rd, STORAGE_INSTANCE
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
     METRIC *metric;
 
-    metric = mrg_metric_get_and_acquire(main_mrg, &rd->metric_uuid, (Word_t) ctx);
+    metric = mrg_metric_get_and_acquire_by_id(main_mrg, rd->uuid, (Word_t) ctx);
 
     if(unlikely(!metric)) {
-        if(unlikely(ctx->config.legacy)) {
-            // this is a single host database
-            // generate uuid from the chart and dimensions ids
-            // and overwrite the one supplied by rrddim
-            metric = rrdeng_metric_get_legacy(si, rrddim_id(rd), rrdset_id(rd->rrdset));
+        if(unlikely(unittest_running)) {
+            metric = rrdeng_metric_unittest(si, rrddim_id(rd), rrdset_id(rd->rrdset));
             if (metric)
-                uuid_copy(rd->metric_uuid, *mrg_metric_uuid(main_mrg, metric));
+                rd->uuid = mrg_metric_uuidmap_id_dup(main_mrg, metric);
         }
 
         if(likely(!metric))
-            metric = rrdeng_metric_create(si, &rd->metric_uuid);
+            metric = rrdeng_metric_create(si, uuidmap_uuid_ptr(rd->uuid));
     }
 
 #ifdef NETDATA_INTERNAL_CHECKS
-    if(uuid_memcmp(&rd->metric_uuid, mrg_metric_uuid(main_mrg, metric)) != 0) {
+    if(!uuid_eq(*uuidmap_uuid_ptr(rd->uuid), *mrg_metric_uuid(main_mrg, metric))) {
         char uuid1[UUID_STR_LEN + 1];
         char uuid2[UUID_STR_LEN + 1];
 
-        uuid_unparse(rd->metric_uuid, uuid1);
+        uuid_unparse(*uuidmap_uuid_ptr(rd->uuid), uuid1);
         uuid_unparse(*mrg_metric_uuid(main_mrg, metric), uuid2);
         fatal("DBENGINE: uuids do not match, asked for metric '%s', but got metric '%s'", uuid1, uuid2);
     }
@@ -219,7 +229,7 @@ static inline bool check_completed_page_consistency(struct rrdeng_collect_handle
 
     struct rrdengine_instance *ctx = mrg_metric_ctx(handle->metric);
 
-    uuid_t *uuid = mrg_metric_uuid(main_mrg, handle->metric);
+    nd_uuid_t *uuid = mrg_metric_uuid(main_mrg, handle->metric);
     time_t start_time_s = pgc_page_start_time_s(handle->pgc_page);
     time_t end_time_s = pgc_page_end_time_s(handle->pgc_page);
     uint32_t update_every_s = pgc_page_update_every_s(handle->pgc_page);
@@ -258,6 +268,8 @@ STORAGE_COLLECT_HANDLE *rrdeng_store_metric_init(STORAGE_METRIC_HANDLE *smh, uin
     METRIC *metric = (METRIC *)smh;
     struct rrdengine_instance *ctx = mrg_metric_ctx(metric);
 
+    RRDENG_COLLECT_HANDLE_OPTIONS options = 0;
+#ifdef NETDATA_INTERNAL_CHECKS
     bool is_1st_metric_writer = true;
     if(!mrg_metric_set_writer(main_mrg, metric)) {
         is_1st_metric_writer = false;
@@ -265,6 +277,12 @@ STORAGE_COLLECT_HANDLE *rrdeng_store_metric_init(STORAGE_METRIC_HANDLE *smh, uin
         uuid_unparse(*mrg_metric_uuid(main_mrg, metric), uuid);
         netdata_log_error("DBENGINE: metric '%s' is already collected and should not be collected twice - expect gaps on the charts", uuid);
     }
+    if(is_1st_metric_writer)
+        options = RRDENG_1ST_METRIC_WRITER;
+    else
+        __atomic_add_fetch(&ctx->atomic.collectors_running_duplicate, 1, __ATOMIC_RELAXED);
+
+#endif
 
     metric = mrg_metric_dup(main_mrg, metric);
 
@@ -276,16 +294,13 @@ STORAGE_COLLECT_HANDLE *rrdeng_store_metric_init(STORAGE_METRIC_HANDLE *smh, uin
     
     handle->pgc_page = NULL;
     handle->page_data = NULL;
-    handle->page_data_size = 0;
-    
+
     handle->page_position = 0;
     handle->page_entries_max = 0;
     handle->update_every_ut = (usec_t)update_every * USEC_PER_SEC;
-    handle->options = is_1st_metric_writer ? RRDENG_1ST_METRIC_WRITER : 0;
+    handle->options = options;
 
     __atomic_add_fetch(&ctx->atomic.collectors_running, 1, __ATOMIC_RELAXED);
-    if(!is_1st_metric_writer)
-        __atomic_add_fetch(&ctx->atomic.collectors_running_duplicate, 1, __ATOMIC_RELAXED);
 
     mrg_metric_set_update_every(main_mrg, metric, update_every);
 
@@ -336,7 +351,6 @@ void rrdeng_store_metric_flush_current_page(STORAGE_COLLECT_HANDLE *sch) {
     handle->page_position = 0;
     handle->page_entries_max = 0;
     handle->page_data = NULL;
-    handle->page_data_size = 0;
 
     // important!
     // we should never zero page end time ut, because this will allow
@@ -352,8 +366,7 @@ void rrdeng_store_metric_flush_current_page(STORAGE_COLLECT_HANDLE *sch) {
 static void rrdeng_store_metric_create_new_page(struct rrdeng_collect_handle *handle,
                                                 struct rrdengine_instance *ctx,
                                                 usec_t point_in_time_ut,
-                                                PGD *data,
-                                                size_t data_size) {
+                                                PGD *data) {
     time_t point_in_time_s = (time_t)(point_in_time_ut / USEC_PER_SEC);
     const uint32_t update_every_s = (uint32_t)(handle->update_every_ut / USEC_PER_SEC);
 
@@ -362,7 +375,7 @@ static void rrdeng_store_metric_create_new_page(struct rrdeng_collect_handle *ha
             .metric_id = mrg_metric_id(main_mrg, handle->metric),
             .start_time_s = point_in_time_s,
             .end_time_s = point_in_time_s,
-            .size = data_size,
+            .size = pgd_memory_footprint(data),
             .data = data,
             .update_every_s = update_every_s,
             .hot = true
@@ -402,7 +415,7 @@ static void rrdeng_store_metric_create_new_page(struct rrdeng_collect_handle *ha
         pgc_page = pgc_page_add_and_acquire(main_cache, page_entry, &added);
     }
 
-    handle->page_entries_max = data_size / CTX_POINT_SIZE_BYTES(ctx);
+    handle->page_entries_max = pgd_capacity(data);
     handle->page_start_time_ut = point_in_time_ut;
     handle->page_end_time_ut = point_in_time_ut;
     handle->page_position = 1; // zero is already in our data
@@ -433,7 +446,7 @@ static size_t aligned_allocation_entries(size_t max_slots, size_t target_slot, t
     return slots;
 }
 
-static PGD *rrdeng_alloc_new_page_data(struct rrdeng_collect_handle *handle, size_t *data_size, usec_t point_in_time_ut) {
+static PGD *rrdeng_alloc_new_page_data(struct rrdeng_collect_handle *handle, usec_t point_in_time_ut) {
     struct rrdengine_instance *ctx = mrg_metric_ctx(handle->metric);
 
     PGD *d = NULL;
@@ -453,24 +466,18 @@ static PGD *rrdeng_alloc_new_page_data(struct rrdeng_collect_handle *handle, siz
     if(slots < 3)
         slots = 3;
 
-    size_t size = slots * CTX_POINT_SIZE_BYTES(ctx);
+    size_t size = slots * CTX_POINT_SIZE_BYTES(ctx); (void)size;
 
     // internal_error(true, "PAGE ALLOC %zu bytes (%zu max)", size, max_size);
 
     internal_fatal(slots < 3 || slots > max_slots, "ooops! wrong distribution of metrics across time");
     internal_fatal(size > tier_page_size[ctx->config.tier] || size < CTX_POINT_SIZE_BYTES(ctx) * 2, "ooops! wrong page size");
 
-    *data_size = size;
-
     switch (ctx->config.page_type) {
         case RRDENG_PAGE_TYPE_ARRAY_32BIT:
         case RRDENG_PAGE_TYPE_ARRAY_TIER1:
-            d = pgd_create(ctx->config.page_type, slots);
-            break;
         case RRDENG_PAGE_TYPE_GORILLA_32BIT:
-            // ignore slots, and use the fixed number of slots per gorilla buffer.
-            // gorilla will automatically add more buffers if needed.
-            d = pgd_create(ctx->config.page_type, RRDENG_GORILLA_32BIT_BUFFER_SLOTS);
+            d = pgd_create(ctx->config.page_type, slots);
             break;
         default:
             fatal("Unknown page type: %uc\n", ctx->config.page_type);
@@ -480,7 +487,7 @@ static PGD *rrdeng_alloc_new_page_data(struct rrdeng_collect_handle *handle, siz
     return d;
 }
 
-static void rrdeng_store_metric_append_point(STORAGE_COLLECT_HANDLE *sch,
+static ALWAYS_INLINE void rrdeng_store_metric_append_point(STORAGE_COLLECT_HANDLE *sch,
                                              const usec_t point_in_time_ut,
                                              const NETDATA_DOUBLE n,
                                              const NETDATA_DOUBLE min_value,
@@ -493,24 +500,25 @@ static void rrdeng_store_metric_append_point(STORAGE_COLLECT_HANDLE *sch,
     struct rrdengine_instance *ctx = mrg_metric_ctx(handle->metric);
 
     if(unlikely(!handle->page_data))
-        handle->page_data = rrdeng_alloc_new_page_data(handle, &handle->page_data_size, point_in_time_ut);
+        handle->page_data = rrdeng_alloc_new_page_data(handle, point_in_time_ut);
 
     timing_step(TIMING_STEP_DBENGINE_CHECK_DATA);
 
-    pgd_append_point(handle->page_data,
-                     point_in_time_ut,
-                     n, min_value, max_value, count, anomaly_count, flags,
-                     handle->page_position);
+    size_t additional_bytes = pgd_append_point(handle->page_data,
+                                               point_in_time_ut,
+                                               n, min_value, max_value, count, anomaly_count, flags,
+                                               handle->page_position);
 
     timing_step(TIMING_STEP_DBENGINE_PACK);
 
     if(unlikely(!handle->pgc_page)) {
-        rrdeng_store_metric_create_new_page(handle, ctx, point_in_time_ut, handle->page_data, handle->page_data_size);
+        rrdeng_store_metric_create_new_page(handle, ctx, point_in_time_ut, handle->page_data);
         // handle->position is set to 1 already
     }
     else {
         // update an existing page
-        pgc_page_hot_set_end_time_s(main_cache, handle->pgc_page, (time_t) (point_in_time_ut / USEC_PER_SEC));
+        pgc_page_hot_set_end_time_s(main_cache, handle->pgc_page,
+                                    (time_t) (point_in_time_ut / USEC_PER_SEC), additional_bytes);
         handle->page_end_time_ut = point_in_time_ut;
 
         if(unlikely(++handle->page_position >= handle->page_entries_max)) {
@@ -562,7 +570,7 @@ static void store_metric_next_error_log(struct rrdeng_collect_handle *handle __m
 #endif
 }
 
-void rrdeng_store_metric_next(STORAGE_COLLECT_HANDLE *sch,
+ALWAYS_INLINE void rrdeng_store_metric_next(STORAGE_COLLECT_HANDLE *sch,
                               const usec_t point_in_time_ut,
                               const NETDATA_DOUBLE n,
                               const NETDATA_DOUBLE min_value,
@@ -657,11 +665,14 @@ int rrdeng_store_metric_finalize(STORAGE_COLLECT_HANDLE *sch) {
     rrdeng_page_alignment_release(handle->alignment);
 
     __atomic_sub_fetch(&ctx->atomic.collectors_running, 1, __ATOMIC_RELAXED);
+
+#ifdef NETDATA_INTERNAL_CHECKS
     if(!(handle->options & RRDENG_1ST_METRIC_WRITER))
         __atomic_sub_fetch(&ctx->atomic.collectors_running_duplicate, 1, __ATOMIC_RELAXED);
 
     if((handle->options & RRDENG_1ST_METRIC_WRITER) && !mrg_metric_clear_writer(main_mrg, handle->metric))
         internal_fatal(true, "DBENGINE: metric is already released");
+#endif
 
     time_t first_time_s, last_time_s;
     mrg_metric_get_retention(main_mrg, handle->metric, &first_time_s, &last_time_s, NULL);
@@ -695,10 +706,10 @@ void rrdeng_store_metric_change_collection_frequency(STORAGE_COLLECT_HANDLE *sch
 // query ops
 
 #ifdef NETDATA_INTERNAL_CHECKS
-SPINLOCK global_query_handle_spinlock = NETDATA_SPINLOCK_INITIALIZER;
+SPINLOCK global_query_handle_spinlock = SPINLOCK_INITIALIZER;
 static struct rrdeng_query_handle *global_query_handle_ll = NULL;
 static void register_query_handle(struct rrdeng_query_handle *handle) {
-    handle->query_pid = gettid();
+    handle->query_pid = gettid_cached();
     handle->started_time_s = now_realtime_sec();
 
     spinlock_lock(&global_query_handle_spinlock);
@@ -731,8 +742,6 @@ void rrdeng_load_metric_init(STORAGE_METRIC_HANDLE *smh,
 {
     usec_t started_ut = now_monotonic_usec();
 
-    netdata_thread_disable_cancelability();
-
     METRIC *metric = (METRIC *)smh;
     struct rrdengine_instance *ctx = mrg_metric_ctx(metric);
     struct rrdeng_query_handle *handle;
@@ -766,8 +775,8 @@ void rrdeng_load_metric_init(STORAGE_METRIC_HANDLE *smh,
 
         handle->dt_s = db_update_every_s;
         if (!handle->dt_s) {
-            handle->dt_s = default_rrd_update_every;
-            mrg_metric_set_update_every_s_if_zero(main_mrg, metric, default_rrd_update_every);
+            handle->dt_s = nd_profile.update_every;
+            mrg_metric_set_update_every_s_if_zero(main_mrg, metric, nd_profile.update_every);
         }
 
         seqh->handle = (STORAGE_QUERY_HANDLE *) handle;
@@ -778,7 +787,7 @@ void rrdeng_load_metric_init(STORAGE_METRIC_HANDLE *smh,
 
         pg_cache_preload(handle);
 
-        __atomic_add_fetch(&rrdeng_cache_efficiency_stats.query_time_init, now_monotonic_usec() - started_ut, __ATOMIC_RELAXED);
+        time_and_count_add(&rrdeng_cache_efficiency_stats.query_time_init, now_monotonic_usec() - started_ut);
     }
     else {
         handle->start_time_s = start_time_s;
@@ -794,7 +803,7 @@ void rrdeng_load_metric_init(STORAGE_METRIC_HANDLE *smh,
     }
 }
 
-static bool rrdeng_load_page_next(struct storage_engine_query_handle *seqh, bool debug_this __maybe_unused) {
+static ALWAYS_INLINE bool rrdeng_load_page_next(struct storage_engine_query_handle *seqh, bool debug_this __maybe_unused) {
     struct rrdeng_query_handle *handle = (struct rrdeng_query_handle *)seqh->handle;
     struct rrdengine_instance *ctx = mrg_metric_ctx(handle->metric);
 
@@ -865,7 +874,7 @@ static bool rrdeng_load_page_next(struct storage_engine_query_handle *seqh, bool
 // Returns the metric and sets its timestamp into current_time
 // IT IS REQUIRED TO **ALWAYS** SET ALL RETURN VALUES (current_time, end_time, flags)
 // IT IS REQUIRED TO **ALWAYS** KEEP TRACK OF TIME, EVEN OUTSIDE THE DATABASE BOUNDARIES
-STORAGE_POINT rrdeng_load_metric_next(struct storage_engine_query_handle *seqh) {
+ALWAYS_INLINE STORAGE_POINT rrdeng_load_metric_next(struct storage_engine_query_handle *seqh) {
     struct rrdeng_query_handle *handle = (struct rrdeng_query_handle *)seqh->handle;
     STORAGE_POINT sp;
 
@@ -916,13 +925,14 @@ void rrdeng_load_metric_finalize(struct storage_engine_query_handle *seqh)
         pgdc_reset(&handle->pgdc, NULL, UINT32_MAX);
     }
 
-    if(!pdc_release_and_destroy_if_unreferenced(handle->pdc, false, false))
+    if(handle->pdc) {
         __atomic_store_n(&handle->pdc->workers_should_stop, true, __ATOMIC_RELAXED);
+        pdc_release_and_destroy_if_unreferenced(handle->pdc, false, false);
+    }
 
     unregister_query_handle(handle);
     rrdeng_query_handle_release(handle);
     seqh->handle = NULL;
-    netdata_thread_enable_cancelability();
 }
 
 time_t rrdeng_load_align_to_optimal_before(struct storage_engine_query_handle *seqh) {
@@ -957,15 +967,32 @@ time_t rrdeng_metric_oldest_time(STORAGE_METRIC_HANDLE *smh) {
     return oldest_time_s;
 }
 
-bool rrdeng_metric_retention_by_uuid(STORAGE_INSTANCE *si, uuid_t *dim_uuid, time_t *first_entry_s, time_t *last_entry_s)
-{
+bool rrdeng_metric_retention_by_uuid(STORAGE_INSTANCE *si, nd_uuid_t *dim_uuid, time_t *first_entry_s, time_t *last_entry_s) {
     struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
     if (unlikely(!ctx)) {
         netdata_log_error("DBENGINE: invalid STORAGE INSTANCE to %s()", __FUNCTION__);
         return false;
     }
 
-    METRIC *metric = mrg_metric_get_and_acquire(main_mrg, dim_uuid, (Word_t) ctx);
+    METRIC *metric = mrg_metric_get_and_acquire_by_uuid(main_mrg, dim_uuid, (Word_t)ctx);
+    if (unlikely(!metric))
+        return false;
+
+    mrg_metric_get_retention(main_mrg, metric, first_entry_s, last_entry_s, NULL);
+
+    mrg_metric_release(main_mrg, metric);
+
+    return true;
+}
+
+bool rrdeng_metric_retention_by_id(STORAGE_INSTANCE *si, UUIDMAP_ID id, time_t *first_entry_s, time_t *last_entry_s) {
+    struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
+    if (unlikely(!ctx)) {
+        netdata_log_error("DBENGINE: invalid STORAGE INSTANCE to %s()", __FUNCTION__);
+        return false;
+    }
+
+    METRIC *metric = mrg_metric_get_and_acquire_by_id(main_mrg, id, (Word_t)ctx);
     if (unlikely(!metric))
         return false;
 
@@ -1052,13 +1079,13 @@ void rrdeng_get_37_statistics(struct rrdengine_instance *ctx, unsigned long long
     array[27] = 0; // (uint64_t)__atomic_load_n(&ctx->stats.page_cache_descriptors, __ATOMIC_RELAXED);
     array[28] = (uint64_t)__atomic_load_n(&ctx->stats.io_errors, __ATOMIC_RELAXED);
     array[29] = (uint64_t)__atomic_load_n(&ctx->stats.fs_errors, __ATOMIC_RELAXED);
-    array[30] = (uint64_t)__atomic_load_n(&global_io_errors, __ATOMIC_RELAXED); // used
-    array[31] = (uint64_t)__atomic_load_n(&global_fs_errors, __ATOMIC_RELAXED); // used
-    array[32] = (uint64_t)__atomic_load_n(&rrdeng_reserved_file_descriptors, __ATOMIC_RELAXED); // used
+    array[30] = (uint64_t)__atomic_load_n(&global_stats.global_io_errors, __ATOMIC_RELAXED); // used
+    array[31] = (uint64_t)__atomic_load_n(&global_stats.global_fs_errors, __ATOMIC_RELAXED); // used
+    array[32] = (uint64_t)__atomic_load_n(&global_stats.rrdeng_reserved_file_descriptors, __ATOMIC_RELAXED); // used
     array[33] = 0; // (uint64_t)__atomic_load_n(&ctx->stats.pg_cache_over_half_dirty_events, __ATOMIC_RELAXED);
-    array[34] = (uint64_t)__atomic_load_n(&global_pg_cache_over_half_dirty_events, __ATOMIC_RELAXED); // used
+    array[34] = (uint64_t)__atomic_load_n(&global_stats.global_pg_cache_over_half_dirty_events, __ATOMIC_RELAXED); // used
     array[35] = 0; // (uint64_t)__atomic_load_n(&ctx->stats.flushing_pressure_page_deletions, __ATOMIC_RELAXED);
-    array[36] = (uint64_t)__atomic_load_n(&global_flushing_pressure_page_deletions, __ATOMIC_RELAXED); // used
+    array[36] = (uint64_t)__atomic_load_n(&global_stats.global_flushing_pressure_page_deletions, __ATOMIC_RELAXED); // used
     array[37] = 0; //(uint64_t)pg_cache->active_descriptors;
 
     fatal_assert(RRDENG_NR_STATS == 38);
@@ -1071,15 +1098,15 @@ static void rrdeng_populate_mrg(struct rrdengine_instance *ctx) {
         datafiles++;
     uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
 
-    ssize_t cpus = (ssize_t)get_netdata_cpus() / (ssize_t)storage_tiers;
+    ssize_t cpus = (ssize_t)netdata_conf_cpus() / (ssize_t)nd_profile.storage_tiers;
     if(cpus > (ssize_t)datafiles)
         cpus = (ssize_t)datafiles;
 
     if(cpus > (ssize_t)libuv_worker_threads)
         cpus = (ssize_t)libuv_worker_threads;
 
-    if(cpus >= (ssize_t)get_netdata_cpus() / 2)
-        cpus = (ssize_t)(get_netdata_cpus() / 2 - 1);
+    if(cpus >= (ssize_t)netdata_conf_cpus() / 2)
+        cpus = (ssize_t)(netdata_conf_cpus() / 2 - 1);
 
     if(cpus < 1)
         cpus = 1;
@@ -1125,63 +1152,64 @@ void rrdeng_readiness_wait(struct rrdengine_instance *ctx) {
     ctx->loading.populate_mrg.array = NULL;
     ctx->loading.populate_mrg.size = 0;
 
+    if(__atomic_load_n(&ctx->atomic.first_time_s, __ATOMIC_RELAXED) == LONG_MAX)
+        __atomic_store_n(&ctx->atomic.first_time_s, now_realtime_sec(), __ATOMIC_RELAXED);
+
     netdata_log_info("DBENGINE: tier %d is ready for data collection and queries", ctx->config.tier);
 }
 
-bool rrdeng_is_legacy(STORAGE_INSTANCE *si) {
-    struct rrdengine_instance *ctx = (struct rrdengine_instance *)si;
-    return ctx->config.legacy;
-}
-
-void rrdeng_exit_mode(struct rrdengine_instance *ctx) {
-    __atomic_store_n(&ctx->quiesce.exit_mode, true, __ATOMIC_RELAXED);
-}
 /*
  * Returns 0 on success, negative on error
  */
-int rrdeng_init(struct rrdengine_instance **ctxp, const char *dbfiles_path,
-                unsigned disk_space_mb, size_t tier) {
+int rrdeng_init(
+    struct rrdengine_instance **ctxp,
+    const char *dbfiles_path,
+    unsigned disk_space_mb,
+    size_t tier,
+    time_t max_retention_s)
+{
     struct rrdengine_instance *ctx;
     uint32_t max_open_files;
 
     max_open_files = rlimit_nofile.rlim_cur / 4;
 
     /* reserve RRDENG_FD_BUDGET_PER_INSTANCE file descriptors for this instance */
-    rrd_stat_atomic_add(&rrdeng_reserved_file_descriptors, RRDENG_FD_BUDGET_PER_INSTANCE);
-    if (rrdeng_reserved_file_descriptors > max_open_files) {
+    rrd_stat_atomic_add(&global_stats.rrdeng_reserved_file_descriptors, RRDENG_FD_BUDGET_PER_INSTANCE);
+    if (global_stats.rrdeng_reserved_file_descriptors > max_open_files) {
         netdata_log_error(
             "Exceeded the budget of available file descriptors (%u/%u), cannot create new dbengine instance.",
-            (unsigned)rrdeng_reserved_file_descriptors,
+            (unsigned)global_stats.rrdeng_reserved_file_descriptors,
             (unsigned)max_open_files);
 
-        rrd_stat_atomic_add(&global_fs_errors, 1);
-        rrd_stat_atomic_add(&rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
+        rrd_stat_atomic_add(&global_stats.global_fs_errors, 1);
+        rrd_stat_atomic_add(&global_stats.rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
         return UV_EMFILE;
     }
 
-    if(NULL == ctxp) {
+    if(ctxp) {
+        *ctxp = ctx = mallocz(sizeof(*ctx));
+        initialize_single_ctx(ctx);
+    }
+    else
         ctx = multidb_ctx[tier];
-        memset(ctx, 0, sizeof(*ctx));
-        ctx->config.legacy = false;
-    }
-    else {
-        *ctxp = ctx = callocz(1, sizeof(*ctx));
-        ctx->config.legacy = true;
-    }
 
     ctx->config.tier = (int)tier;
     ctx->config.page_type = tier_page_type[tier];
     ctx->config.global_compress_alg = dbengine_default_compression();
-    if (disk_space_mb < RRDENG_MIN_DISK_SPACE_MB)
-        disk_space_mb = RRDENG_MIN_DISK_SPACE_MB;
-    ctx->config.max_disk_space = disk_space_mb * 1048576LLU;
+
     strncpyz(ctx->config.dbfiles_path, dbfiles_path, sizeof(ctx->config.dbfiles_path) - 1);
     ctx->config.dbfiles_path[sizeof(ctx->config.dbfiles_path) - 1] = '\0';
 
+    if (disk_space_mb && disk_space_mb < RRDENG_MIN_DISK_SPACE_MB)
+        disk_space_mb = RRDENG_MIN_DISK_SPACE_MB;
+
+    ctx->config.max_disk_space = disk_space_mb * 1048576LLU;
+
+    ctx->config.max_retention_s = max_retention_s;
+
     ctx->atomic.transaction_id = 1;
     ctx->quiesce.enabled = false;
 
-    rw_spinlock_init(&ctx->njfv2idx.spinlock);
     ctx->atomic.first_time_s = LONG_MAX;
     ctx->atomic.metrics = 0;
     ctx->atomic.samples = 0;
@@ -1192,13 +1220,13 @@ int rrdeng_init(struct rrdengine_instance **ctxp, const char *dbfiles_path,
         return 0;
     }
 
-    if (ctx->config.legacy) {
+    if (unittest_running) {
         freez(ctx);
         if (ctxp)
             *ctxp = NULL;
     }
 
-    rrd_stat_atomic_add(&rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
+    rrd_stat_atomic_add(&global_stats.rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
     return UV_EIO;
 }
 
@@ -1223,40 +1251,39 @@ int rrdeng_exit(struct rrdengine_instance *ctx) {
     size_t count = 10;
     while(__atomic_load_n(&ctx->atomic.collectors_running, __ATOMIC_RELAXED) && count && !unittest_running) {
         if(!logged) {
-            netdata_log_info("DBENGINE: waiting for collectors to finish on tier %d...", (ctx->config.legacy) ? -1 : ctx->config.tier);
+            netdata_log_info("DBENGINE: waiting for collectors to finish on tier %d...", ctx->config.tier);
             logged = true;
         }
         sleep_usec(100 * USEC_PER_MS);
         count--;
     }
 
-    netdata_log_info("DBENGINE: flushing main cache for tier %d", (ctx->config.legacy) ? -1 : ctx->config.tier);
     pgc_flush_all_hot_and_dirty_pages(main_cache, (Word_t)ctx);
 
-    netdata_log_info("DBENGINE: shutting down tier %d", (ctx->config.legacy) ? -1 : ctx->config.tier);
     struct completion completion = {};
     completion_init(&completion);
     rrdeng_enq_cmd(ctx, RRDENG_OPCODE_CTX_SHUTDOWN, NULL, &completion, STORAGE_PRIORITY_BEST_EFFORT, NULL, NULL);
+
     completion_wait_for(&completion);
     completion_destroy(&completion);
 
-    finalize_rrd_files(ctx);
+    // No need to release the datafiles list
+    //finalize_rrd_files(ctx);
 
-    if(ctx->config.legacy)
+    if (unittest_running) //(ctx->config.unittest)
         freez(ctx);
 
-    rrd_stat_atomic_add(&rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
+    rrd_stat_atomic_add(&global_stats.rrdeng_reserved_file_descriptors, -RRDENG_FD_BUDGET_PER_INSTANCE);
     return 0;
 }
 
-void rrdeng_prepare_exit(struct rrdengine_instance *ctx) {
+void rrdeng_quiesce(struct rrdengine_instance *ctx) {
     if (NULL == ctx)
         return;
 
     // FIXME - ktsaou - properly cleanup ctx
     // 1. make sure all collectors are stopped
 
-    completion_init(&ctx->quiesce.completion);
     rrdeng_enq_cmd(ctx, RRDENG_OPCODE_CTX_QUIESCE, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
 }
 
@@ -1299,7 +1326,7 @@ static void populate_v2_statistics(struct rrdengine_datafile *datafile, RRDENG_S
             if(likely(points > 1))
                 update_every_s = (time_t) ((end_time_s - start_time_s) / (points - 1));
             else {
-                update_every_s = (time_t) (default_rrd_update_every * get_tier_grouping(datafile->ctx->config.tier));
+                update_every_s = (time_t) (nd_profile.update_every * get_tier_grouping(datafile->ctx->config.tier));
                 stats->single_point_pages++;
             }
 
@@ -1372,7 +1399,9 @@ RRDENG_SIZE_STATS rrdeng_size_statistics(struct rrdengine_instance *ctx) {
     }
 
 //    stats.sizeof_metric = 0;
-    stats.sizeof_datafile = struct_natural_alignment(sizeof(struct rrdengine_datafile)) + struct_natural_alignment(sizeof(struct rrdengine_journalfile));
+    stats.sizeof_datafile =
+        natural_alignment(sizeof(struct rrdengine_datafile)) +
+                            natural_alignment(sizeof(struct rrdengine_journalfile));
     stats.sizeof_page_in_cache = 0; // struct_natural_alignment(sizeof(struct page_cache_descr));
     stats.sizeof_point_data = page_type_size[ctx->config.page_type];
     stats.sizeof_page_data = tier_page_size[ctx->config.tier];
@@ -1381,7 +1410,7 @@ RRDENG_SIZE_STATS rrdeng_size_statistics(struct rrdengine_instance *ctx) {
 //    stats.sizeof_metric_in_index = 40;
 //    stats.sizeof_page_in_index = 24;
 
-    stats.default_granularity_secs = (size_t)default_rrd_update_every * get_tier_grouping(ctx->config.tier);
+    stats.default_granularity_secs = (size_t)nd_profile.update_every * get_tier_grouping(ctx->config.tier);
 
     return stats;
 }
