@@ -3,9 +3,10 @@
 #include "sqlite_functions.h"
 #include "sqlite_aclk_alert.h"
 
-#ifdef ENABLE_ACLK
 #include "../../aclk/aclk_alarm_api.h"
 
+extern __thread bool is_health_thread;
+
 #define SQLITE3_COLUMN_STRDUPZ_OR_NULL(res, param)                                                                     \
     ({                                                                                                                 \
         int _param = (param);                                                                                          \
@@ -19,10 +20,19 @@
 
 static inline bool is_event_from_alert_variable_config(int64_t unique_id, nd_uuid_t *host_id)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_SELECT_VARIABLE_ALERT_BY_UNIQUE_ID, &res))
-        return false;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_SELECT_VARIABLE_ALERT_BY_UNIQUE_ID, &compiled_res))
+                return false;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_SELECT_VARIABLE_ALERT_BY_UNIQUE_ID, &res))
+            return false;
+    }
 
     bool ret = false;
 
@@ -35,21 +45,31 @@ static inline bool is_event_from_alert_variable_config(int64_t unique_id, nd_uui
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
     return ret;
 }
 
-#define MAX_REMOVED_PERIOD 604800 //a week
-
 #define SQL_UPDATE_ALERT_VERSION_TRANSITION                                                                            \
     "UPDATE alert_version SET unique_id = @unique_id WHERE health_log_id = @health_log_id"
 
 static void update_alert_version_transition(int64_t health_log_id, int64_t unique_id)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_UPDATE_ALERT_VERSION_TRANSITION, &res))
-        return;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_UPDATE_ALERT_VERSION_TRANSITION, &compiled_res))
+                return;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_UPDATE_ALERT_VERSION_TRANSITION, &res))
+            return;
+    }
 
     int param = 0;
     SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, unique_id));
@@ -62,7 +82,10 @@ static void update_alert_version_transition(int64_t health_log_id, int64_t uniqu
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
 }
 
 //decide if some events should be sent or not
@@ -71,10 +94,19 @@ done:
 
 static bool cloud_status_matches(int64_t health_log_id, RRDCALC_STATUS status)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_SELECT_LAST_ALERT_STATUS, &res))
-        return true;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_SELECT_LAST_ALERT_STATUS, &compiled_res))
+                return true;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_SELECT_LAST_ALERT_STATUS, &res))
+            return true;
+    }
 
     bool send = false;
 
@@ -90,7 +122,10 @@ static bool cloud_status_matches(int64_t health_log_id, RRDCALC_STATUS status)
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
     return send;
 }
 
@@ -109,32 +144,44 @@ done:
 //
 static int insert_alert_to_submit_queue(RRDHOST *host, int64_t health_log_id, uint32_t unique_id, RRDCALC_STATUS status)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
     if (cloud_status_matches(health_log_id, status)) {
         update_alert_version_transition(health_log_id, unique_id);
         return 1;
     }
 
-    if (is_event_from_alert_variable_config(unique_id, &host->host_uuid))
+    if (is_event_from_alert_variable_config(unique_id, &host->host_id.uuid))
         return 2;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_QUEUE_ALERT_TO_CLOUD, &res))
-        return -1;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_QUEUE_ALERT_TO_CLOUD, &compiled_res))
+                return -1;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_QUEUE_ALERT_TO_CLOUD, &res))
+            return -1;
+    }
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
     SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, health_log_id));
     SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, (int64_t) unique_id));
 
     param = 0;
-    int rc = execute_insert(res);
+    int rc = sqlite3_step_monitored(res);
     if (unlikely(rc != SQLITE_DONE))
         error_report("Failed to insert alert in the submit queue %"PRIu32", rc = %d", unique_id, rc);
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
     return 0;
 }
 
@@ -146,13 +193,13 @@ done:
 //
 static int delete_alert_from_submit_queue(RRDHOST *host, int64_t first_seq_id, int64_t last_seq_id)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_DELETE_QUEUE_ALERT_TO_CLOUD, &res))
+    if (!PREPARE_STATEMENT(db_meta, SQL_DELETE_QUEUE_ALERT_TO_CLOUD, &res))
         return -1;
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
     SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, first_seq_id));
     SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, last_seq_id));
 
@@ -163,7 +210,7 @@ static int delete_alert_from_submit_queue(RRDHOST *host, int64_t first_seq_id, i
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    SQLITE_FINALIZE(res);
     return 0;
 }
 
@@ -215,7 +262,7 @@ static inline char *sqlite3_text_strdupz_empty(sqlite3_stmt *res, int iCol) {
 
 #define SQL_UPDATE_ALERT_VERSION                                                                                       \
     "INSERT INTO alert_version (health_log_id, unique_id, status, version, date_submitted)"                            \
-    " VALUES (@health_log_id, @unique_id, @status, @version, UNIXEPOCH())"                                         \
+    " VALUES (@health_log_id, @unique_id, @status, @version, UNIXEPOCH())"                                             \
     " ON CONFLICT(health_log_id) DO UPDATE SET status = excluded.status, version = excluded.version, "                 \
     " unique_id=excluded.unique_id, date_submitted=excluded.date_submitted"
 
@@ -223,27 +270,32 @@ static inline char *sqlite3_text_strdupz_empty(sqlite3_stmt *res, int iCol) {
 // Store a new alert transition along with the version after sending to the cloud
 //   - Update an existing alert with the updated version, status, transition and date submitted
 //
-static void sql_update_alert_version(int64_t health_log_id, int64_t unique_id, RRDCALC_STATUS status, uint64_t version)
+static void sql_update_alert_version(
+    int64_t health_log_id,
+    int64_t unique_id,
+    RRDCALC_STATUS status,
+    uint64_t version,
+    sqlite3_stmt **res)
 {
-    static __thread sqlite3_stmt *res = NULL;
-
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_UPDATE_ALERT_VERSION, &res))
-        return;
+    if (!*res) {
+        if (!PREPARE_STATEMENT(db_meta, SQL_UPDATE_ALERT_VERSION, res))
+            return;
+    }
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, health_log_id));
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, unique_id));
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int(res, ++param, status));
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, version));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(*res, ++param, health_log_id));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(*res, ++param, unique_id));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_int(*res, ++param, status));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(*res, ++param, version));
 
     param = 0;
-    int rc = sqlite3_step_monitored(res);
+    int rc = sqlite3_step_monitored(*res);
     if (rc != SQLITE_DONE)
         error_report("Failed to execute sql_update_alert_version");
 
 done:
-    REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    REPORT_BIND_FAIL(*res, param);
+    SQLITE_RESET(*res);
 }
 
 #define SQL_SELECT_ALERT_TO_DUMMY                                                                                      \
@@ -266,11 +318,12 @@ static void commit_alert_events(RRDHOST *host)
         return;
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     int64_t first_sequence_id = 0;
     int64_t last_sequence_id = 0;
 
+    sqlite3_stmt *res_version = NULL;
     param = 0;
     while (sqlite3_step_monitored(res) == SQLITE_ROW) {
 
@@ -283,7 +336,9 @@ static void commit_alert_events(RRDHOST *host)
         RRDCALC_STATUS status = (RRDCALC_STATUS)sqlite3_column_int(res, 3);
         int64_t health_log_id = sqlite3_column_int64(res, 4);
 
-        sql_update_alert_version(health_log_id, unique_id, status, version);
+        // Prepare the statement on the first time (res_version) then reuse it
+        // finalize when we are done
+        sql_update_alert_version(health_log_id, unique_id, status, version, &res_version);
     }
 
     if (first_sequence_id)
@@ -356,7 +411,7 @@ void health_alarm_log_populate(
     alarm_log->timezone = strdupz(rrdhost_abbrev_timezone(host));
     alarm_log->exec_path =  sqlite3_column_bytes(res, EXEC) ?
                                strdupz((char *)sqlite3_column_text(res, EXEC)) :
-                               strdupz((char *)string2str(host->health.health_default_exec));
+                               strdupz((char *)string2str(host->health.default_exec));
 
     alarm_log->conf_source = source ? strdupz(source) : strdupz("");
 
@@ -422,29 +477,27 @@ void health_alarm_log_populate(
     " AND hl.host_id = @host_id AND aq.host_id = hl.host_id AND hl.health_log_id = hld.health_log_id"                  \
     " ORDER BY aq.sequence_id ASC LIMIT "ACLK_MAX_ALERT_UPDATES
 
-static void aclk_push_alert_event(RRDHOST *host __maybe_unused)
+static void aclk_push_alert_event(RRDHOST *host, sqlite3_stmt **res, sqlite3_stmt **res_version)
 {
+    CLAIM_ID claim_id = claim_id_get();
 
-    char *claim_id = get_agent_claimid();
-    if (!claim_id || !host->node_id)
+    if (!claim_id_is_set(claim_id) || UUIDiszero(host->node_id))
         return;
 
-    sqlite3_stmt *res = NULL;
-
-    if (!PREPARE_STATEMENT(db_meta, SQL_SELECT_ALERT_TO_PUSH, &res)) {
-        freez(claim_id);
-        return;
+    if (!*res) {
+        if (!PREPARE_STATEMENT(db_meta, SQL_SELECT_ALERT_TO_PUSH, res))
+            return;
     }
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(*res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     char node_id_str[UUID_STR_LEN];
-    uuid_unparse_lower(*host->node_id, node_id_str);
+    uuid_unparse_lower(host->node_id.uuid, node_id_str);
 
     struct alarm_log_entry alarm_log;
     alarm_log.node_id = node_id_str;
-    alarm_log.claim_id = claim_id;
+    alarm_log.claim_id = claim_id.str;
 
     int64_t first_id = 0;
     int64_t last_id = 0;
@@ -452,8 +505,8 @@ static void aclk_push_alert_event(RRDHOST *host __maybe_unused)
     param = 0;
     RRDCALC_STATUS status;
     struct aclk_sync_cfg_t *wc = host->aclk_config;
-    while (sqlite3_step_monitored(res) == SQLITE_ROW) {
-        health_alarm_log_populate(&alarm_log, res, host, &status);
+    while (sqlite3_step_monitored(*res) == SQLITE_ROW) {
+        health_alarm_log_populate(&alarm_log, *res, host, &status);
         aclk_send_alarm_log_entry(&alarm_log);
         wc->alert_count++;
 
@@ -461,7 +514,9 @@ static void aclk_push_alert_event(RRDHOST *host __maybe_unused)
         if (first_id == 0)
             first_id = last_id;
 
-        sql_update_alert_version(alarm_log.health_log_id, alarm_log.unique_id, status, alarm_log.version);
+        // The statement to set the version will be compiled once and reset when done
+        // out caller will finalize the statement to release resources
+        sql_update_alert_version(alarm_log.health_log_id, alarm_log.unique_id, status, alarm_log.version, res_version);
 
         destroy_alarm_log_entry(&alarm_log);
     }
@@ -470,11 +525,11 @@ static void aclk_push_alert_event(RRDHOST *host __maybe_unused)
         nd_log(
             NDLS_ACCESS,
             NDLP_DEBUG,
-            "ACLK RES [%s (%s)]: ALERTS SENT from %ld - %ld",
+            "ACLK RES [%s (%s)]: ALERTS SENT from %lld - %lld",
             node_id_str,
             rrdhost_hostname(host),
-            first_id,
-            last_id);
+            (long long)first_id,
+            (long long)last_id);
 
         delete_alert_from_submit_queue(host, first_id, last_id);
         // Mark to do one more check
@@ -482,26 +537,31 @@ static void aclk_push_alert_event(RRDHOST *host __maybe_unused)
     }
 
 done:
-    REPORT_BIND_FAIL(res, param);
-    SQLITE_FINALIZE(res);
-
-    freez(claim_id);
+    REPORT_BIND_FAIL(*res, param);
+    SQLITE_RESET(*res);
 }
 
-#define SQL_DELETE_PROCESSED_ROWS                                                                                      \
-    "DELETE FROM alert_queue WHERE host_id = @host_id AND rowid between @row1 AND @row2"
+#define SQL_DELETE_PROCESSED_ROWS "DELETE FROM alert_queue WHERE host_id = @host_id AND rowid = @row"
 
-static void delete_alert_from_pending_queue(RRDHOST *host, int64_t row1, int64_t row2)
+static void delete_alert_from_pending_queue(RRDHOST *host, int64_t row)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_DELETE_PROCESSED_ROWS, &res))
-        return;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_DELETE_PROCESSED_ROWS, &compiled_res))
+                return;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_DELETE_PROCESSED_ROWS, &res))
+            return;
+    }
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, row1));
-    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, row2));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_int64(res, ++param, row));
 
     param = 0;
     int rc = sqlite3_step_monitored(res);
@@ -510,7 +570,10 @@ static void delete_alert_from_pending_queue(RRDHOST *host, int64_t row1, int64_t
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
 }
 
 #define SQL_REBUILD_HOST_ALERT_VERSION_TABLE                                                                           \
@@ -530,7 +593,7 @@ void rebuild_host_alert_version_table(RRDHOST *host)
         return;
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     param = 0;
     int rc = execute_insert(res);
@@ -543,7 +606,8 @@ void rebuild_host_alert_version_table(RRDHOST *host)
     if (!PREPARE_STATEMENT(db_meta, SQL_REBUILD_HOST_ALERT_VERSION_TABLE, &res))
         return;
 
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    param = 0;
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     param = 0;
     rc = execute_insert(res);
@@ -561,18 +625,25 @@ done:
 
 bool process_alert_pending_queue(RRDHOST *host)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    static __thread sqlite3_stmt *compiled_res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_PROCESS_ALERT_PENDING_QUEUE, &res))
-        return false;
+    if (is_health_thread) {
+        if (!compiled_res) {
+            if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_PROCESS_ALERT_PENDING_QUEUE, &compiled_res))
+                return false;
+        }
+        res = compiled_res;
+    } else {
+        if (!PREPARE_STATEMENT(db_meta, SQL_PROCESS_ALERT_PENDING_QUEUE, &res))
+            return false;
+    }
 
     int param = 0;
     int added =0, count = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     param = 0;
-    int64_t start_row = 0;
-    int64_t end_row = 0;
     while (sqlite3_step_monitored(res) == SQLITE_ROW) {
 
         int64_t health_log_id = sqlite3_column_int64(res, 0);
@@ -586,21 +657,19 @@ bool process_alert_pending_queue(RRDHOST *host)
                 added++;
         }
 
-        if (!start_row)
-            start_row = row;
-        end_row = row;
+        delete_alert_from_pending_queue(host, row);
 
         count++;
     }
 
-    if (start_row)
-        delete_alert_from_pending_queue(host, start_row, end_row);
-
     if(count)
         nd_log(NDLS_ACCESS, NDLP_NOTICE, "ACLK STA [%s (N/A)]: Processed %d entries, queued %d", rrdhost_hostname(host), count, added);
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    if (is_health_thread)
+        SQLITE_RESET(res);
+    else
+        SQLITE_FINALIZE(res);
     return added > 0;
 }
 
@@ -608,10 +677,8 @@ void aclk_push_alert_events_for_all_hosts(void)
 {
     RRDHOST *host;
 
-    // Checking if we shutting down
-    if (!service_running(SERVICE_ACLK))
-        return;
-
+    sqlite3_stmt *res = NULL;               // used to scan pending alerts to send
+    sqlite3_stmt *res_version = NULL;       // used to update the alert version
     dfe_start_reentrant(rrdhost_root_index, host) {
         if (!rrdhost_flag_check(host, RRDHOST_FLAG_ACLK_STREAM_ALERTS) ||
             rrdhost_flag_check(host, RRDHOST_FLAG_PENDING_CONTEXT_LOAD))
@@ -638,9 +705,11 @@ void aclk_push_alert_events_for_all_hosts(void)
             wc->send_snapshot = 0;
         }
         else
-            aclk_push_alert_event(host);
+            aclk_push_alert_event(host, &res, &res_version);
     }
     dfe_done(host);
+    SQLITE_FINALIZE(res);
+    SQLITE_FINALIZE(res_version);
 }
 
 void aclk_send_alert_configuration(char *config_hash)
@@ -777,16 +846,16 @@ done:
     "SELECT SUM(version) FROM health_log hl, alert_version av"                                                         \
     " WHERE hl.host_id = @host_uuid AND hl.health_log_id = av.health_log_id AND av.status <> -2"
 
-static uint64_t calculate_node_alert_version(RRDHOST *host)
+uint64_t calculate_node_alert_version(RRDHOST *host)
 {
-    static __thread sqlite3_stmt *res = NULL;
+    sqlite3_stmt *res = NULL;
 
-    if (!PREPARE_COMPILED_STATEMENT(db_meta, SQL_ALERT_VERSION_CALC, &res))
+    if (!PREPARE_STATEMENT(db_meta, SQL_ALERT_VERSION_CALC, &res))
         return 0;
 
     uint64_t version = 0;
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     param = 0;
     while (sqlite3_step_monitored(res) == SQLITE_ROW) {
@@ -795,22 +864,32 @@ static uint64_t calculate_node_alert_version(RRDHOST *host)
 
 done:
     REPORT_BIND_FAIL(res, param);
-    SQLITE_RESET(res);
+    SQLITE_FINALIZE(res);
     return version;
 }
 
 static void schedule_alert_snapshot_if_needed(struct aclk_sync_cfg_t *wc, uint64_t cloud_version)
 {
+    if (cloud_version == 1) {
+        nd_log(
+            NDLS_ACCESS,
+            NDLP_NOTICE,
+            "Cloud requested to skip alert version verification for host \"%s\", node \"%s\"",
+            rrdhost_hostname(wc->host),
+            wc->node_id);
+        return;
+    }
+
     uint64_t local_version = calculate_node_alert_version(wc->host);
     if (local_version != cloud_version) {
         nd_log(
             NDLS_ACCESS,
             NDLP_NOTICE,
-            "Scheduling alert snapshot for host \"%s\", node \"%s\" (version: cloud %zu, local %zu)",
+            "Scheduling alert snapshot for host \"%s\", node \"%s\" (version: cloud %llu, local %llu)",
             rrdhost_hostname(wc->host),
             wc->node_id,
-            cloud_version,
-            local_version);
+            (long long unsigned)cloud_version,
+            (long long unsigned)local_version);
 
         wc->send_snapshot = 1;
         rrdhost_flag_set(wc->host, RRDHOST_FLAG_ACLK_STREAM_ALERTS);
@@ -819,44 +898,14 @@ static void schedule_alert_snapshot_if_needed(struct aclk_sync_cfg_t *wc, uint64
         nd_log(
             NDLS_ACCESS,
             NDLP_DEBUG,
-            "Alert check on \"%s\", node \"%s\" (version: cloud %zu, local %zu)",
+            "Alert check on \"%s\", node \"%s\" (version: cloud %llu, local %llu)",
             rrdhost_hostname(wc->host),
             wc->node_id,
-            cloud_version,
-            local_version);
+            (unsigned long long)cloud_version,
+            (unsigned long long)local_version);
     wc->checkpoint_count++;
 }
 
-void aclk_process_send_alarm_snapshot(char *node_id, char *claim_id __maybe_unused, char *snapshot_uuid)
-{
-    nd_uuid_t node_uuid;
-
-    if (unlikely(!node_id || uuid_parse(node_id, node_uuid)))
-        return;
-
-    struct aclk_sync_cfg_t *wc;
-
-    RRDHOST *host = find_host_by_node_id(node_id);
-    if (unlikely(!host || !(wc = host->aclk_config))) {
-        nd_log(NDLS_ACCESS, NDLP_WARNING, "ACLK STA [%s (N/A)]: ACLK node id does not exist", node_id);
-        return;
-    }
-
-    nd_log(NDLS_ACCESS, NDLP_DEBUG,
-            "IN [%s (%s)]: Request to send alerts snapshot, snapshot_uuid %s",
-            node_id,
-            wc->host ? rrdhost_hostname(wc->host) : "N/A",
-            snapshot_uuid);
-
-    if (wc->alerts_snapshot_uuid && !strcmp(wc->alerts_snapshot_uuid,snapshot_uuid))
-        return;
-
-    wc->alerts_snapshot_uuid = strdupz(snapshot_uuid);
-
-    wc->send_snapshot = 1;
-    rrdhost_flag_set(host, RRDHOST_FLAG_ACLK_STREAM_ALERTS);
-}
-
 #define SQL_COUNT_SNAPSHOT_ENTRIES                                                                                     \
     "SELECT COUNT(1) FROM alert_version av, health_log hl "                                                            \
     "WHERE hl.host_id = @host_id AND hl.health_log_id = av.health_log_id AND av.status <> -2"
@@ -908,23 +957,21 @@ void send_alert_snapshot_to_cloud(RRDHOST *host __maybe_unused)
         return;
     }
 
-    char *claim_id = get_agent_claimid();
-    if (unlikely(!claim_id))
+    CLAIM_ID claim_id = claim_id_get();
+    if (unlikely(!claim_id_is_set(claim_id)))
         return;
 
-    // Check database for this node to see how many alerts we will need to put in the snapshot
-    int cnt = calculate_alert_snapshot_entries(&host->host_uuid);
-    if (!cnt) {
-        freez(claim_id);
+    // Check the database for this node to see how many alerts we will need to put in the snapshot
+    int cnt = calculate_alert_snapshot_entries(&host->host_id.uuid);
+    if (!cnt)
         return;
-    }
 
     sqlite3_stmt *res = NULL;
     if (!PREPARE_STATEMENT(db_meta, SQL_GET_SNAPSHOT_ENTRIES, &res))
         return;
 
     int param = 0;
-    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC));
+    SQLITE_BIND_FAIL(done, sqlite3_bind_blob(res, ++param, &host->host_id.uuid, sizeof(host->host_id.uuid), SQLITE_STATIC));
 
     nd_uuid_t local_snapshot_uuid;
     char snapshot_uuid_str[UUID_STR_LEN];
@@ -944,13 +991,13 @@ void send_alert_snapshot_to_cloud(RRDHOST *host __maybe_unused)
     struct alarm_log_entry alarm_log;
 
     alarm_snap.node_id = wc->node_id;
-    alarm_snap.claim_id = claim_id;
+    alarm_snap.claim_id = claim_id.str;
     alarm_snap.snapshot_uuid = snapshot_uuid;
     alarm_snap.chunks = chunks;
     alarm_snap.chunk = 1;
 
     alarm_log.node_id = wc->node_id;
-    alarm_log.claim_id = claim_id;
+    alarm_log.claim_id = claim_id.str;
 
     cnt = 0;
     param = 0;
@@ -969,7 +1016,7 @@ void send_alert_snapshot_to_cloud(RRDHOST *host __maybe_unused)
         version += alarm_log.version;
 
         if (cnt == ALARM_EVENTS_PER_CHUNK) {
-            if (aclk_connected)
+            if (aclk_online_for_alerts())
                 aclk_send_alarm_snapshot(snapshot_proto);
             cnt = 0;
             if (alarm_snap.chunk < chunks) {
@@ -985,18 +1032,16 @@ void send_alert_snapshot_to_cloud(RRDHOST *host __maybe_unused)
     nd_log(
         NDLS_ACCESS,
         NDLP_DEBUG,
-        "ACLK REQ [%s (%s)]: Sent! %d alerts snapshot, snapshot_uuid %s  (version = %zu)",
+        "ACLK REQ [%s (%s)]: Sent! %d alerts snapshot, snapshot_uuid %s  (version = %llu)",
         wc->node_id,
         rrdhost_hostname(host),
         cnt,
         snapshot_uuid,
-        version);
+        (long long unsigned)version);
 
 done:
     REPORT_BIND_FAIL(res, param);
     SQLITE_FINALIZE(res);
-
-    freez(claim_id);
 }
 
 // Start streaming alerts
@@ -1015,7 +1060,7 @@ void aclk_start_alert_streaming(char *node_id, uint64_t cloud_version)
         return;
     }
 
-    if (unlikely(!host->health.health_enabled)) {
+    if (unlikely(!host->health.enabled)) {
         nd_log(NDLS_ACCESS, NDLP_NOTICE, "ACLK STA [%s (N/A)]: Ignoring request to stream alert state changes, health is disabled.", node_id);
         return;
     }
@@ -1030,25 +1075,24 @@ void aclk_alert_version_check(char *node_id, char *claim_id, uint64_t cloud_vers
 {
     nd_uuid_t node_uuid;
 
-    if (unlikely(!node_id || !claim_id || !claimed() || uuid_parse(node_id, node_uuid)))
+    if (unlikely(!node_id || !claim_id || !is_agent_claimed() || uuid_parse(node_id, node_uuid)))
         return;
 
-    char *agent_claim_id = get_agent_claimid();
-    if (claim_id && agent_claim_id && strcmp(agent_claim_id, claim_id) != 0) {
-        nd_log(NDLS_ACCESS, NDLP_NOTICE, "ACLK REQ [%s (N/A)]: ALERTS CHECKPOINT VALIDATION REQUEST RECEIVED WITH INVALID CLAIM ID", node_id);
-        goto done;
+    CLAIM_ID agent_claim_id = claim_id_get();
+    if (claim_id && claim_id_is_set(agent_claim_id) && strcmp(agent_claim_id.str, claim_id) != 0) {
+        nd_log(NDLS_ACCESS, NDLP_NOTICE,
+               "ACLK REQ [%s (N/A)]: ALERTS CHECKPOINT VALIDATION REQUEST RECEIVED WITH INVALID CLAIM ID",
+               node_id);
+        return;
     }
 
     struct aclk_sync_cfg_t *wc;
     RRDHOST *host = find_host_by_node_id(node_id);
 
     if ((!host || !(wc = host->aclk_config)))
-        nd_log(NDLS_ACCESS, NDLP_NOTICE, "ACLK REQ [%s (N/A)]: ALERTS CHECKPOINT VALIDATION REQUEST RECEIVED FOR INVALID NODE", node_id);
+        nd_log(NDLS_ACCESS, NDLP_NOTICE,
+               "ACLK REQ [%s (N/A)]: ALERTS CHECKPOINT VALIDATION REQUEST RECEIVED FOR INVALID NODE",
+               node_id);
     else
         schedule_alert_snapshot_if_needed(wc, cloud_version);
-
-done:
-    freez(agent_claim_id);
 }
-
-#endif
