@@ -3,8 +3,7 @@
 #define NETDATA_SQLITE_DB_MIGRATION_H
 
 #include "daemon/common.h"
-#include "sqlite3.h"
-
+#include "database/sqlite/vendored/sqlite3.h"
 
 int perform_database_migration(sqlite3 *database, int target_version);
 int perform_context_database_migration(sqlite3 *database, int target_version);
