@@ -4,16 +4,19 @@
 #define NETDATA_SQLITE_HEALTH_H
 
 #include "daemon/common.h"
-#include "sqlite3.h"
+#include "database/sqlite/vendored/sqlite3.h"
+
+#define ALERT_TRANSITION_DELAY_LONG  (600)
+#define ALERT_TRANSITION_DELAY_SHORT (10)
+#define ALERT_TRANSITION_DELAY_NONE  (0)
 
 struct sql_alert_transition_data;
 struct sql_alert_config_data;
 struct rrd_alert_prototype;
 void sql_health_alarm_log_load(RRDHOST *host);
 void sql_health_alarm_log_save(RRDHOST *host, ALARM_ENTRY *ae);
-void sql_health_alarm_log_cleanup(RRDHOST *host, bool claimed);
+void sql_health_alarm_log_cleanup(RRDHOST *host);
 void sql_alert_store_config(struct rrd_alert_prototype *ap);
-void sql_aclk_alert_clean_dead_entries(RRDHOST *host);
 int sql_health_get_last_executed_event(RRDHOST *host, ALARM_ENTRY *ae, RRDCALC_STATUS *last_executed_status);
 void sql_health_alarm_log2json(RRDHOST *host, BUFFER *wb, time_t after, const char *chart);
 int health_migrate_old_health_log_table(char *table);
