@@ -2,6 +2,13 @@
 
 #include "prometheus.h"
 
+DEFINE_JUDYL_TYPED(PROM_CONTEXT_OPTIONS, PROMETHEUS_OUTPUT_OPTIONS);
+
+static void PROM_CONTEXT_OPTIONS_free_cb(Word_t index, PROMETHEUS_OUTPUT_OPTIONS options __maybe_unused) {
+    STRING *context_id = (STRING *)index;
+    string_freez(context_id);
+}
+
 // ----------------------------------------------------------------------------
 // PROMETHEUS
 // /api/v1/allmetrics?format=prometheus and /api/v1/allmetrics?format=prometheus_all_hosts
@@ -59,7 +66,7 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN
     }
 
     if (unlikely(
-            st->rrd_memory_mode == RRD_MEMORY_MODE_NONE &&
+            st->rrd_memory_mode == RRD_DB_MODE_NONE &&
             !(EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AS_COLLECTED))) {
         netdata_log_debug(
             D_EXPORTING,
@@ -88,9 +95,8 @@ static netdata_mutex_t prometheus_server_root_mutex = NETDATA_MUTEX_INITIALIZER;
  */
 void prometheus_clean_server_root()
 {
+    netdata_mutex_lock(&prometheus_server_root_mutex);
     if (prometheus_server_root) {
-        netdata_mutex_lock(&prometheus_server_root_mutex);
-
         struct prometheus_server *ps;
         for (ps = prometheus_server_root; ps; ) {
             struct prometheus_server *current = ps;
@@ -101,8 +107,8 @@ void prometheus_clean_server_root()
             freez(current);
         }
         prometheus_server_root = NULL;
-        netdata_mutex_unlock(&prometheus_server_root_mutex);
     }
+    netdata_mutex_unlock(&prometheus_server_root_mutex);
 }
 
 /**
@@ -149,24 +155,11 @@ static inline time_t prometheus_server_last_access(const char *server, RRDHOST *
  *
  * @param d a destination string.
  * @param s a source string.
- * @param usable the number of characters to copy.
+ * @param size the number of characters to copy.
  * @return Returns the length of the copied string.
  */
-inline size_t prometheus_name_copy(char *d, const char *s, size_t usable)
-{
-    size_t n;
-
-    for (n = 0; *s && n < usable; d++, s++, n++) {
-        register char c = *s;
-
-        if (!isalnum(c))
-            *d = '_';
-        else
-            *d = c;
-    }
-    *d = '\0';
-
-    return n;
+inline void prometheus_name_copy(char *d, const char *s, size_t size) {
+    prometheus_rrdlabels_sanitize_name(d, s, size);
 }
 
 /**
@@ -174,28 +167,13 @@ inline size_t prometheus_name_copy(char *d, const char *s, size_t usable)
  *
  * @param d a destination string.
  * @param s a source string.
- * @param usable the number of characters to copy.
+ * @param size the number of characters to copy.
  * @return Returns the length of the copied string.
  */
-inline size_t prometheus_label_copy(char *d, const char *s, size_t usable)
-{
-    size_t n;
-
-    // make sure we can escape one character without overflowing the buffer
-    usable--;
-
-    for (n = 0; *s && n < usable; d++, s++, n++) {
-        register char c = *s;
-
-        if (unlikely(c == '"' || c == '\\' || c == '\n')) {
-            *d++ = '\\';
-            n++;
-        }
-        *d = c;
-    }
-    *d = '\0';
-
-    return n;
+inline void prometheus_label_copy(char *d, const char *s, size_t size) {
+    // our label values are already compatible with prometheus label values
+    // so, just copy them
+    strncpyz(d, s, size - 1);
 }
 
 /**
@@ -299,8 +277,8 @@ static int format_prometheus_label_callback(const char *name, const char *value,
     char k[PROMETHEUS_ELEMENT_MAX + 1];
     char v[PROMETHEUS_ELEMENT_MAX + 1];
 
-    prometheus_name_copy(k, name, PROMETHEUS_ELEMENT_MAX);
-    prometheus_label_copy(v, value, PROMETHEUS_ELEMENT_MAX);
+    prometheus_name_copy(k, name, sizeof(k));
+    prometheus_label_copy(v, value, sizeof(v));
 
     if (*k && *v) {
         if (d->count > 0) buffer_strcat(d->instance->labels_buffer, ",");
@@ -341,8 +319,8 @@ static int format_prometheus_chart_label_callback(const char *name, const char *
     char k[PROMETHEUS_ELEMENT_MAX + 1];
     char v[PROMETHEUS_ELEMENT_MAX + 1];
 
-    prometheus_name_copy(k, name, PROMETHEUS_ELEMENT_MAX);
-    prometheus_label_copy(v, value, PROMETHEUS_ELEMENT_MAX);
+    prometheus_name_copy(k, name, sizeof(k));
+    prometheus_label_copy(v, value, sizeof(v));
 
     if (*k && *v)
         buffer_sprintf(wb, ",%s=\"%s\"", k, v);
@@ -364,6 +342,7 @@ struct host_variables_callback_options {
     SIMPLE_PATTERN *pattern;
     struct instance *instance;
     STRING *prometheus;
+    PROM_CONTEXT_OPTIONS_JudyLSet *context_options;
 };
 
 /**
@@ -487,10 +466,14 @@ static void generate_as_collected_from_metric(BUFFER *wb,
                                               int prometheus_collector,
                                               RRDLABELS *chart_labels)
 {
-    buffer_sprintf(wb, "%s_%s", p->prefix, p->context);
+    buffer_strcat(wb, p->prefix);
+    buffer_putc(wb, '_');
+    buffer_strcat(wb, p->context);
 
-    if (!homogeneous)
-        buffer_sprintf(wb, "_%s", p->dimension);
+    if (!homogeneous) {
+        buffer_putc(wb, '_');
+        buffer_strcat(wb, p->dimension);
+    }
 
     buffer_sprintf(wb, "%s{%schart=\"%s\"", p->suffix, p->labels_prefix, p->chart);
 
@@ -501,21 +484,23 @@ static void generate_as_collected_from_metric(BUFFER *wb,
 
     rrdlabels_walkthrough_read(chart_labels, format_prometheus_chart_label_callback, wb);
 
-    buffer_sprintf(wb, "%s} ", p->labels);
+    buffer_strcat(wb, p->labels);
+    buffer_putc(wb, '}');
+    buffer_putc(wb, ' ');
 
     if (prometheus_collector)
-        buffer_sprintf(
-            wb,
-            NETDATA_DOUBLE_FORMAT,
+        buffer_print_netdata_double(wb,
             (NETDATA_DOUBLE)p->rd->collector.last_collected_value * (NETDATA_DOUBLE)p->rd->multiplier /
             (NETDATA_DOUBLE)p->rd->divisor);
     else
-        buffer_sprintf(wb, COLLECTED_NUMBER_FORMAT, p->rd->collector.last_collected_value);
+        buffer_print_int64(wb, p->rd->collector.last_collected_value);
 
-    if (p->output_options & PROMETHEUS_OUTPUT_TIMESTAMPS)
-        buffer_sprintf(wb, " %"PRIu64"\n", timeval_msec(&p->rd->collector.last_collected_time));
-    else
-        buffer_sprintf(wb, "\n");
+    if (p->output_options & PROMETHEUS_OUTPUT_TIMESTAMPS) {
+        buffer_putc(wb, ' ');
+        buffer_print_uint64(wb, timeval_msec(&p->rd->collector.last_collected_time));
+    }
+
+    buffer_putc(wb, '\n');
 }
 
 static void prometheus_print_os_info(
@@ -630,9 +615,25 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
 
         prometheus_label_copy(chart,
                               (output_options & PROMETHEUS_OUTPUT_NAMES && st->name) ?
-                               rrdset_name(st) : rrdset_id(st), PROMETHEUS_ELEMENT_MAX);
-        prometheus_label_copy(family, rrdset_family(st), PROMETHEUS_ELEMENT_MAX);
-        prometheus_name_copy(context, rrdset_context(st), PROMETHEUS_ELEMENT_MAX);
+                               rrdset_name(st) : rrdset_id(st), sizeof(chart));
+        prometheus_label_copy(family, rrdset_family(st), sizeof(family));
+        prometheus_name_copy(context, rrdset_context(st), sizeof(context));
+
+        if(opts->output_options & PROMETHEUS_OUTPUT_HELP_TYPE) {
+            // we do not want to print HELP and TYPE for the same context twice
+            STRING *context_id = string_strdupz(context);
+            PROMETHEUS_OUTPUT_OPTIONS ctx_opts = PROM_CONTEXT_OPTIONS_GET(opts->context_options, (Word_t)context_id);
+            if (!(ctx_opts & PROMETHEUS_OUTPUT_HELP_TYPE)) {
+                // it is not printed for this context yet
+                ctx_opts = opts->output_options;
+                PROM_CONTEXT_OPTIONS_SET(opts->context_options, (Word_t)context_id, ctx_opts);
+            }
+            else {
+                // we have printed HELP and TYPE for this context already
+                opts->output_options &= ~PROMETHEUS_OUTPUT_HELP_TYPE;
+                string_freez(context_id);
+            }
+        }
 
         int as_collected = (EXPORTING_OPTIONS_DATA_SOURCE(opts->exporting_options)
                             == EXPORTING_SOURCE_DATA_AS_COLLECTED);
@@ -696,7 +697,7 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                     }
 
                     if (opts->output_options & PROMETHEUS_OUTPUT_HELP_TYPE) {
-                        generate_as_collected_prom_help(wb, prefix, context, units, suffix, st);
+                        generate_as_collected_prom_help(wb, prefix, context, units, p.suffix, st);
                         generate_as_collected_prom_type(wb, prefix, context, units, p.suffix, p.type);
                         opts->output_options &= ~PROMETHEUS_OUTPUT_HELP_TYPE;
                     }
@@ -708,7 +709,7 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                         prometheus_label_copy(
                             dimension,
                             (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
-                            PROMETHEUS_ELEMENT_MAX);
+                            sizeof(dimension));
                     }
                     else {
                         // the dimensions of the chart, do not have the same algorithm, multiplier or divisor
@@ -717,7 +718,7 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                         prometheus_name_copy(
                             dimension,
                             (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
-                            PROMETHEUS_ELEMENT_MAX);
+                            sizeof(dimension));
                     }
                     generate_as_collected_from_metric(wb, &p, homogeneous, prometheus_collector, st->rrdlabels);
                 }
@@ -728,8 +729,7 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                     NETDATA_DOUBLE value = exporting_calculate_value_from_stored_data(opts->instance, rd, &last_time);
 
                     if (!isnan(value) && !isinf(value)) {
-                        if (EXPORTING_OPTIONS_DATA_SOURCE(opts->exporting_options)
-                            == EXPORTING_SOURCE_DATA_AVERAGE)
+                        if (EXPORTING_OPTIONS_DATA_SOURCE(opts->exporting_options) == EXPORTING_SOURCE_DATA_AVERAGE)
                             suffix = "_average";
                         else if (EXPORTING_OPTIONS_DATA_SOURCE(opts->exporting_options)
                                  == EXPORTING_SOURCE_DATA_SUM)
@@ -738,11 +738,11 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                         prometheus_label_copy(
                             dimension,
                             (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
-                            PROMETHEUS_ELEMENT_MAX);
+                            sizeof(dimension));
 
                         if (opts->output_options & PROMETHEUS_OUTPUT_HELP_TYPE) {
                             generate_as_collected_prom_help(wb, prefix, context, units, suffix, st);
-                            generate_as_collected_prom_type(wb, prefix, context, units, p.suffix, "gauge");
+                            generate_as_collected_prom_type(wb, prefix, context, units, suffix, "gauge");
                             opts->output_options &= ~PROMETHEUS_OUTPUT_HELP_TYPE;
                         }
 
@@ -769,7 +769,7 @@ static int prometheus_rrdset_to_json(RRDSET *st, void *data)
                                            value,
                                            last_time * MSEC_PER_SEC);
                         else
-                        buffer_sprintf(wb, "%s_%s%s%s{%s%s} " NETDATA_DOUBLE_FORMAT "\n",
+                            buffer_sprintf(wb, "%s_%s%s%s{%s%s} " NETDATA_DOUBLE_FORMAT "\n",
                                            prefix,
                                            context,
                                            units,
@@ -812,6 +812,8 @@ static inline int prometheus_rrdcontext_callback(const DICTIONARY_ITEM *item, vo
     return HTTP_RESP_OK;
 }
 
+
+
 /**
  * Write metrics in Prometheus format to a buffer.
  *
@@ -832,12 +834,13 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
     const char *prefix,
     EXPORTING_OPTIONS exporting_options,
     int allhosts,
-    PROMETHEUS_OUTPUT_OPTIONS output_options)
+    PROMETHEUS_OUTPUT_OPTIONS output_options,
+    PROM_CONTEXT_OPTIONS_JudyLSet *context_options)
 {
     SIMPLE_PATTERN *filter = simple_pattern_create(filter_string, NULL, SIMPLE_PATTERN_EXACT, true);
 
     char hostname[PROMETHEUS_ELEMENT_MAX + 1];
-    prometheus_label_copy(hostname, rrdhost_hostname(host), PROMETHEUS_ELEMENT_MAX);
+    prometheus_label_copy(hostname, rrdhost_hostname(host), sizeof(hostname));
 
     format_host_labels_prometheus(instance, host);
 
@@ -883,7 +886,8 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
         .host_header_printed = 0,
         .pattern = filter,
         .instance = instance,
-        .prometheus = string_strdupz("prometheus")
+        .prometheus = string_strdupz("prometheus"),
+        .context_options = context_options,
     };
 
     // send custom variables set for the host
@@ -975,8 +979,13 @@ void rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(
         server,
         prometheus_exporter_instance->before);
 
+    PROM_CONTEXT_OPTIONS_JudyLSet context_options;
+    PROM_CONTEXT_OPTIONS_INIT(&context_options);
+
     rrd_stats_api_v1_charts_allmetrics_prometheus(
-        prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 0, output_options);
+        prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 0, output_options, &context_options);
+
+    PROM_CONTEXT_OPTIONS_FREE(&context_options, PROM_CONTEXT_OPTIONS_free_cb);
 }
 
 /**
@@ -1011,10 +1020,15 @@ void rrd_stats_api_v1_charts_allmetrics_prometheus_all_hosts(
         server,
         prometheus_exporter_instance->before);
 
+    PROM_CONTEXT_OPTIONS_JudyLSet context_options;
+    PROM_CONTEXT_OPTIONS_INIT(&context_options);
+
     dfe_start_reentrant(rrdhost_root_index, host)
     {
         rrd_stats_api_v1_charts_allmetrics_prometheus(
-            prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 1, output_options);
+            prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 1, output_options, &context_options);
     }
     dfe_done(host);
+
+    PROM_CONTEXT_OPTIONS_FREE(&context_options, PROM_CONTEXT_OPTIONS_free_cb);
 }
