@@ -41,6 +41,9 @@ struct netdata_static_thread {
     // internal use, to maintain a pointer to the created thread
     ND_THREAD *thread;
 
+    // a function to call to check it should be enabled or not
+    bool (*enable_routine) (void);
+
     // an initialization function to run before spawning the thread
     void (*init_routine) (void);
 
@@ -111,4 +114,7 @@ void nd_thread_rwspinlock_write_unlocked(void);
 #define nd_thread_rwspinlock_write_unlocked() debug_dummy()
 #endif
 
+void nd_thread_can_run_sql(bool exclude);
+bool nd_thread_runs_sql(void);
+
 #endif //NETDATA_THREADS_H
