@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "ad_charts.h"
+#include "ml_config.h"
 
 void ml_update_dimensions_chart(ml_host_t *host, const ml_machine_learning_stats_t &mls) {
     /*
@@ -20,7 +21,7 @@ void ml_update_dimensions_chart(ml_host_t *host, const ml_machine_learning_stats
                     id_buf,
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.machine_learning_status", // ctx
+                    "netdata.ml_status", // ctx
                     "Machine learning status", // title
                     "dimensions", // units
                     NETDATA_ML_PLUGIN, // plugin
@@ -62,7 +63,7 @@ void ml_update_dimensions_chart(ml_host_t *host, const ml_machine_learning_stats
                     id_buf, // id
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.metric_types", // ctx
+                    "netdata.ml_metric_types", // ctx
                     "Dimensions by metric type", // title
                     "dimensions", // units
                     NETDATA_ML_PLUGIN, // plugin
@@ -104,7 +105,7 @@ void ml_update_dimensions_chart(ml_host_t *host, const ml_machine_learning_stats
                     id_buf, // id
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.training_status", // ctx
+                    "netdata.ml_training_status", // ctx
                     "Training status of dimensions", // title
                     "dimensions", // units
                     NETDATA_ML_PLUGIN, // plugin
@@ -288,7 +289,7 @@ void ml_update_host_and_detection_rate_charts(ml_host_t *host, collected_number
             rrdset_flag_set(host->type_anomaly_rate_rs, RRDSET_FLAG_ANOMALY_DETECTION);
         }
 
-        spinlock_lock_cancelable(&host->type_anomaly_rate_spinlock);
+        spinlock_lock(&host->type_anomaly_rate_spinlock);
         for (auto &entry : host->type_anomaly_rate) {
             ml_type_anomaly_rate_t &type_anomaly_rate = entry.second;
 
@@ -305,7 +306,7 @@ void ml_update_host_and_detection_rate_charts(ml_host_t *host, collected_number
             type_anomaly_rate.anomalous_dimensions = 0;
             type_anomaly_rate.normal_dimensions = 0;
         }
-        spinlock_unlock_cancelable(&host->type_anomaly_rate_spinlock);
+        spinlock_unlock(&host->type_anomaly_rate_spinlock);
 
         rrdset_done(host->type_anomaly_rate_rs);
     }
@@ -399,113 +400,160 @@ void ml_update_host_and_detection_rate_charts(ml_host_t *host, collected_number
     }
 }
 
-void ml_update_training_statistics_chart(ml_training_thread_t *training_thread, const ml_training_stats_t &ts) {
+void ml_update_training_statistics_chart(ml_worker_t *worker, const ml_queue_stats_t &stats) {
     /*
      * queue stats
     */
     {
-        if (!training_thread->queue_stats_rs) {
+        if (!worker->queue_stats_rs) {
             char id_buf[1024];
             char name_buf[1024];
 
-            snprintfz(id_buf, 1024, "training_queue_%zu_stats", training_thread->id);
-            snprintfz(name_buf, 1024, "training_queue_%zu_stats", training_thread->id);
+            snprintfz(id_buf, 1024, "training_queue_%zu_ops", worker->id);
+            snprintfz(name_buf, 1024, "training_queue_%zu_ops", worker->id);
 
-            training_thread->queue_stats_rs = rrdset_create(
+            worker->queue_stats_rs = rrdset_create(
                     localhost,
                     "netdata", // type
                     id_buf, // id
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.queue_stats", // ctx
-                    "Training queue stats", // title
-                    "items", // units
+                    "netdata.ml_queue_ops", // ctx
+                    "Training queue operations", // title
+                    "count", // units
                     NETDATA_ML_PLUGIN, // plugin
                     NETDATA_ML_MODULE_TRAINING, // module
                     NETDATA_ML_CHART_PRIO_QUEUE_STATS, // priority
                     localhost->rrd_update_every, // update_every
                     RRDSET_TYPE_LINE// chart_type
             );
-            rrdset_flag_set(training_thread->queue_stats_rs, RRDSET_FLAG_ANOMALY_DETECTION);
+            rrdset_flag_set(worker->queue_stats_rs, RRDSET_FLAG_ANOMALY_DETECTION);
 
-            training_thread->queue_stats_queue_size_rd =
-                rrddim_add(training_thread->queue_stats_rs, "queue_size", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->queue_stats_popped_items_rd =
-                rrddim_add(training_thread->queue_stats_rs, "popped_items", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+            worker->queue_stats_num_create_new_model_requests_rd =
+                rrddim_add(worker->queue_stats_rs, "pushed create model", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            worker->queue_stats_num_create_new_model_requests_completed_rd =
+                rrddim_add(worker->queue_stats_rs, "popped create model", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+            worker->queue_stats_num_add_existing_model_requests_rd =
+                rrddim_add(worker->queue_stats_rs, "pushed add model", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+            worker->queue_stats_num_add_existing_model_requests_completed_rd =
+                rrddim_add(worker->queue_stats_rs, "popped add models", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
         }
 
-        rrddim_set_by_pointer(training_thread->queue_stats_rs,
-                              training_thread->queue_stats_queue_size_rd, ts.queue_size);
-        rrddim_set_by_pointer(training_thread->queue_stats_rs,
-                              training_thread->queue_stats_popped_items_rd, ts.num_popped_items);
+        rrddim_set_by_pointer(worker->queue_stats_rs,
+                              worker->queue_stats_num_create_new_model_requests_rd, stats.total_create_new_model_requests_pushed);
+        rrddim_set_by_pointer(worker->queue_stats_rs,
+                              worker->queue_stats_num_create_new_model_requests_completed_rd, stats.total_create_new_model_requests_popped);
 
-        rrdset_done(training_thread->queue_stats_rs);
+        rrddim_set_by_pointer(worker->queue_stats_rs,
+                              worker->queue_stats_num_add_existing_model_requests_rd, stats.total_add_existing_model_requests_pushed);
+        rrddim_set_by_pointer(worker->queue_stats_rs,
+                              worker->queue_stats_num_add_existing_model_requests_completed_rd, stats.total_add_existing_model_requests_popped);
+
+        rrdset_done(worker->queue_stats_rs);
+    }
+
+    {
+        if (!worker->queue_size_rs) {
+            char id_buf[1024];
+            char name_buf[1024];
+
+            snprintfz(id_buf, 1024, "training_queue_%zu_size", worker->id);
+            snprintfz(name_buf, 1024, "training_queue_%zu_size", worker->id);
+
+            worker->queue_size_rs = rrdset_create(
+                    localhost,
+                    "netdata", // type
+                    id_buf, // id
+                    name_buf, // name
+                    NETDATA_ML_CHART_FAMILY, // family
+                    "netdata.ml_queue_size", // ctx
+                    "Training queue size", // title
+                    "count", // units
+                    NETDATA_ML_PLUGIN, // plugin
+                    NETDATA_ML_MODULE_TRAINING, // module
+                    NETDATA_ML_CHART_PRIO_QUEUE_STATS, // priority
+                    localhost->rrd_update_every, // update_every
+                    RRDSET_TYPE_LINE// chart_type
+            );
+            rrdset_flag_set(worker->queue_size_rs, RRDSET_FLAG_ANOMALY_DETECTION);
+
+            worker->queue_size_rd =
+                rrddim_add(worker->queue_size_rs, "items", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+        }
+
+        ml_queue_size_t qs = ml_queue_size(worker->queue);
+        collected_number cn = qs.add_exisiting_model + qs.create_new_model;
+
+        rrddim_set_by_pointer(worker->queue_size_rs, worker->queue_size_rd, cn);
+        rrdset_done(worker->queue_size_rs);
     }
 
     /*
      * training stats
     */
     {
-        if (!training_thread->training_time_stats_rs) {
+        if (!worker->training_time_stats_rs) {
             char id_buf[1024];
             char name_buf[1024];
 
-            snprintfz(id_buf, 1024, "training_queue_%zu_time_stats", training_thread->id);
-            snprintfz(name_buf, 1024, "training_queue_%zu_time_stats", training_thread->id);
+            snprintfz(id_buf, 1024, "training_queue_%zu_time_stats", worker->id);
+            snprintfz(name_buf, 1024, "training_queue_%zu_time_stats", worker->id);
 
-            training_thread->training_time_stats_rs = rrdset_create(
+            worker->training_time_stats_rs = rrdset_create(
                     localhost,
                     "netdata", // type
                     id_buf, // id
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.training_time_stats", // ctx
+                    "netdata.ml_training_time_stats", // ctx
                     "Training time stats", // title
-                    "milliseconds", // units
+                    "microseconds", // units
                     NETDATA_ML_PLUGIN, // plugin
                     NETDATA_ML_MODULE_TRAINING, // module
                     NETDATA_ML_CHART_PRIO_TRAINING_TIME_STATS, // priority
                     localhost->rrd_update_every, // update_every
                     RRDSET_TYPE_LINE// chart_type
             );
-            rrdset_flag_set(training_thread->training_time_stats_rs, RRDSET_FLAG_ANOMALY_DETECTION);
+            rrdset_flag_set(worker->training_time_stats_rs, RRDSET_FLAG_ANOMALY_DETECTION);
 
-            training_thread->training_time_stats_allotted_rd =
-                rrddim_add(training_thread->training_time_stats_rs, "allotted", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_time_stats_consumed_rd =
-                rrddim_add(training_thread->training_time_stats_rs, "consumed", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_time_stats_remaining_rd =
-                rrddim_add(training_thread->training_time_stats_rs, "remaining", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
+            worker->training_time_stats_allotted_rd =
+                rrddim_add(worker->training_time_stats_rs, "allotted", NULL, 1, 1000, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_time_stats_consumed_rd =
+                rrddim_add(worker->training_time_stats_rs, "consumed", NULL, 1, 1000, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_time_stats_remaining_rd =
+                rrddim_add(worker->training_time_stats_rs, "remaining", NULL, 1, 1000, RRD_ALGORITHM_INCREMENTAL);
         }
 
-        rrddim_set_by_pointer(training_thread->training_time_stats_rs,
-                              training_thread->training_time_stats_allotted_rd, ts.allotted_ut);
-        rrddim_set_by_pointer(training_thread->training_time_stats_rs,
-                              training_thread->training_time_stats_consumed_rd, ts.consumed_ut);
-        rrddim_set_by_pointer(training_thread->training_time_stats_rs,
-                              training_thread->training_time_stats_remaining_rd, ts.remaining_ut);
+        rrddim_set_by_pointer(worker->training_time_stats_rs,
+                              worker->training_time_stats_allotted_rd, stats.allotted_ut);
+        rrddim_set_by_pointer(worker->training_time_stats_rs,
+                              worker->training_time_stats_consumed_rd, stats.consumed_ut);
+        rrddim_set_by_pointer(worker->training_time_stats_rs,
+                              worker->training_time_stats_remaining_rd, stats.remaining_ut);
 
-        rrdset_done(training_thread->training_time_stats_rs);
+        rrdset_done(worker->training_time_stats_rs);
     }
 
     /*
      * training result stats
     */
     {
-        if (!training_thread->training_results_rs) {
+        if (!worker->training_results_rs) {
             char id_buf[1024];
             char name_buf[1024];
 
-            snprintfz(id_buf, 1024, "training_queue_%zu_results", training_thread->id);
-            snprintfz(name_buf, 1024, "training_queue_%zu_results", training_thread->id);
+            snprintfz(id_buf, 1024, "training_queue_%zu_results", worker->id);
+            snprintfz(name_buf, 1024, "training_queue_%zu_results", worker->id);
 
-            training_thread->training_results_rs = rrdset_create(
+            worker->training_results_rs = rrdset_create(
                     localhost,
                     "netdata", // type
                     id_buf, // id
                     name_buf, // name
                     NETDATA_ML_CHART_FAMILY, // family
-                    "netdata.training_results", // ctx
+                    "netdata.ml_training_results", // ctx
                     "Training results", // title
                     "events", // units
                     NETDATA_ML_PLUGIN, // plugin
@@ -514,37 +562,48 @@ void ml_update_training_statistics_chart(ml_training_thread_t *training_thread,
                     localhost->rrd_update_every, // update_every
                     RRDSET_TYPE_LINE// chart_type
             );
-            rrdset_flag_set(training_thread->training_results_rs, RRDSET_FLAG_ANOMALY_DETECTION);
+            rrdset_flag_set(worker->training_results_rs, RRDSET_FLAG_ANOMALY_DETECTION);
 
-            training_thread->training_results_ok_rd =
-                rrddim_add(training_thread->training_results_rs, "ok", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_results_invalid_query_time_range_rd =
-                rrddim_add(training_thread->training_results_rs, "invalid-queries", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_results_not_enough_collected_values_rd =
-                rrddim_add(training_thread->training_results_rs, "not-enough-values", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_results_null_acquired_dimension_rd =
-                rrddim_add(training_thread->training_results_rs, "null-acquired-dimensions", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-            training_thread->training_results_chart_under_replication_rd =
-                rrddim_add(training_thread->training_results_rs, "chart-under-replication", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+            worker->training_results_ok_rd =
+                rrddim_add(worker->training_results_rs, "ok", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_results_invalid_query_time_range_rd =
+                rrddim_add(worker->training_results_rs, "invalid-queries", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_results_not_enough_collected_values_rd =
+                rrddim_add(worker->training_results_rs, "not-enough-values", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_results_null_acquired_dimension_rd =
+                rrddim_add(worker->training_results_rs, "null-acquired-dimensions", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            worker->training_results_chart_under_replication_rd =
+                rrddim_add(worker->training_results_rs, "chart-under-replication", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
         }
 
-        rrddim_set_by_pointer(training_thread->training_results_rs,
-                              training_thread->training_results_ok_rd, ts.training_result_ok);
-        rrddim_set_by_pointer(training_thread->training_results_rs,
-                              training_thread->training_results_invalid_query_time_range_rd, ts.training_result_invalid_query_time_range);
-        rrddim_set_by_pointer(training_thread->training_results_rs,
-                              training_thread->training_results_not_enough_collected_values_rd, ts.training_result_not_enough_collected_values);
-        rrddim_set_by_pointer(training_thread->training_results_rs,
-                              training_thread->training_results_null_acquired_dimension_rd, ts.training_result_null_acquired_dimension);
-        rrddim_set_by_pointer(training_thread->training_results_rs,
-                              training_thread->training_results_chart_under_replication_rd, ts.training_result_chart_under_replication);
+        rrddim_set_by_pointer(worker->training_results_rs,
+                              worker->training_results_ok_rd, stats.item_result_ok);
+        rrddim_set_by_pointer(worker->training_results_rs,
+                              worker->training_results_invalid_query_time_range_rd, stats.item_result_invalid_query_time_range);
+        rrddim_set_by_pointer(worker->training_results_rs,
+                              worker->training_results_not_enough_collected_values_rd, stats.item_result_not_enough_collected_values);
+        rrddim_set_by_pointer(worker->training_results_rs,
+                              worker->training_results_null_acquired_dimension_rd, stats.item_result_null_acquired_dimension);
+        rrddim_set_by_pointer(worker->training_results_rs,
+                              worker->training_results_chart_under_replication_rd, stats.item_result_chart_under_replication);
 
-        rrdset_done(training_thread->training_results_rs);
+        rrdset_done(worker->training_results_rs);
     }
 }
 
-void ml_update_global_statistics_charts(uint64_t models_consulted) {
-    if (Cfg.enable_statistics_charts) {
+void ml_update_global_statistics_charts(uint64_t models_consulted,
+                                        uint64_t models_received,
+                                        uint64_t models_sent,
+                                        uint64_t models_ignored,
+                                        uint64_t models_deserialization_failures,
+                                        uint64_t memory_consumption,
+                                        uint64_t memory_new,
+                                        uint64_t memory_delete)
+{
+    if (!Cfg.enable_statistics_charts)
+        return;
+
+    {
         static RRDSET *st = NULL;
         static RRDDIM *rd = NULL;
 
@@ -571,4 +630,98 @@ void ml_update_global_statistics_charts(uint64_t models_consulted) {
 
         rrdset_done(st);
     }
+
+    {
+        static RRDSET *st = NULL;
+        static RRDDIM *rd_received = NULL;
+        static RRDDIM *rd_sent = NULL;
+        static RRDDIM *rd_ignored = NULL;
+        static RRDDIM *rd_deserialization_failures = NULL;
+
+        if (unlikely(!st)) {
+            st = rrdset_create_localhost(
+                    "netdata" // type
+                    , "ml_models_streamed" // id
+                    , NULL // name
+                    , NETDATA_ML_CHART_FAMILY // family
+                    , NULL // context
+                    , "KMeans models streamed" // title
+                    , "models" // units
+                    , NETDATA_ML_PLUGIN // plugin
+                    , NETDATA_ML_MODULE_DETECTION // module
+                    , NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS // priority
+                    , localhost->rrd_update_every // update_every
+                    , RRDSET_TYPE_LINE // chart_type
+            );
+
+            rd_received = rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            rd_sent = rrddim_add(st, "sent", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            rd_ignored = rrddim_add(st, "ignored", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            rd_deserialization_failures = rrddim_add(st, "deserialization failures", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+        }
+
+        rrddim_set_by_pointer(st, rd_received, (collected_number) models_received);
+        rrddim_set_by_pointer(st, rd_sent, (collected_number) models_sent);
+        rrddim_set_by_pointer(st, rd_ignored, (collected_number) models_ignored);
+        rrddim_set_by_pointer(st, rd_deserialization_failures, (collected_number) models_deserialization_failures);
+
+        rrdset_done(st);
+    }
+
+    {
+        static RRDSET *st = NULL;
+        static RRDDIM *rd_memory_consumption = NULL;
+
+        if (unlikely(!st)) {
+            st = rrdset_create_localhost(
+                    "netdata" // type
+                    , "ml_memory_used" // id
+                    , NULL // name
+                    , NETDATA_ML_CHART_FAMILY // family
+                    , NULL // context
+                    , "ML memory usage" // title
+                    , "bytes" // units
+                    , NETDATA_ML_PLUGIN // plugin
+                    , NETDATA_ML_MODULE_DETECTION // module
+                    , NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS // priority
+                    , localhost->rrd_update_every // update_every
+                    , RRDSET_TYPE_LINE // chart_type
+            );
+
+            rd_memory_consumption = rrddim_add(st, "used", NULL, 1024, 1, RRD_ALGORITHM_ABSOLUTE);
+        }
+
+        rrddim_set_by_pointer(st, rd_memory_consumption, (collected_number) memory_consumption / (1024));
+        rrdset_done(st);
+    }
+
+    {
+        static RRDSET *st = NULL;
+        static RRDDIM *rd_memory_new = NULL;
+        static RRDDIM *rd_memory_delete = NULL;
+
+        if (unlikely(!st)) {
+            st = rrdset_create_localhost(
+                    "netdata" // type
+                    , "ml_memory_ops" // id
+                    , NULL // name
+                    , NETDATA_ML_CHART_FAMILY // family
+                    , NULL // context
+                    , "ML memory operations" // title
+                    , "count" // units
+                    , NETDATA_ML_PLUGIN // plugin
+                    , NETDATA_ML_MODULE_DETECTION // module
+                    , NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS // priority
+                    , localhost->rrd_update_every // update_every
+                    , RRDSET_TYPE_LINE // chart_type
+            );
+
+            rd_memory_new = rrddim_add(st, "new", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+            rd_memory_delete = rrddim_add(st, "delete", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+        }
+
+        rrddim_set_by_pointer(st, rd_memory_new, (collected_number) memory_new);
+        rrddim_set_by_pointer(st, rd_memory_delete, (collected_number) memory_delete);
+        rrdset_done(st);
+    }
 }
